/******************************************************************************
 CNBrainPan.c

							The NBrainPan Class
		
	SUPERCLASS = CBitMapPane
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).

 ******************************************************************************/

#include "Global.h"
#include "CNBrainPan.h"
#include "CNMyGlobals.h"
#include "CNAxonViewTask.h"
#include "CPaneBorder.h"
#include "CWindow.h"
#include "CCluster.h"
#include "MainWindow.h"
#include "ResourceDefs.h"
#include "AxonController.h"
#include "LongCoordinates.h"
#include "CNOViewTask.h"
#include "CNOViewCmdTask.h"
#include "CClipboard.h"
#include "Events.h"
#include "CNNervousClipboard.h"
#include "CNDendriteViewTask.h"
#include "CNStimulatorViewTask.h"
#include "CNSomaViewTask.h"
#include "CNSynapseViewTask.h"
#include "CList.h"
#include "NervousSystemDoc.h"
#include "CNOViewOwner.h"
#include "CNObjectView.h"


// External declarations from "GlobalVars.c"
extern CBartender	*gBartender;		/* Manages all menus				*/
extern CClipboard	*gClipboard;		/* Copies and Pastes data			*/
extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern CNervousSystemDoc *gNLimbo;


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IViewTemp

 ******************************************************************************/

void	CNBrainPan::IViewTemp		(CView			*anEnclosure,
								 CBureaucrat	*aSupervisor,
								 Ptr			viewData)
{
	inherited::IViewTemp (anEnclosure, aSupervisor, viewData);

	// any additional initialization for your subclass
	wantsClicks = TRUE;
	SetCanBeGopher(TRUE);
	InitMyDataExtent();
	AdjustBounds();  // Can't use "bounds = frame" because bounds is in CPanorama units.
	fBeingDisposed = FALSE;
	
	// Initialize "itsSelection" array.
	itsSelection = NULL;
	ResetSelectionList();
	
} /* IViewTemp */


/******************************************************************************
 GetMyNervousSysDoc

	Travel back up the supervisor hierarchy to this object's supervising document.
 ******************************************************************************/

CNervousSystemDoc *CNBrainPan::GetMyNervousSysDoc(void)
{
	CBureaucrat *aBureaucrat = NULL;
	Boolean		fFoundBureaucrat = FALSE;
	
	if (itsSupervisor) {
		
		// Prime the while loop.
		aBureaucrat = itsSupervisor;
		
		while ((!(fFoundBureaucrat)) && (aBureaucrat)) {
			if (member(aBureaucrat, CNervousSystemDoc))
				fFoundBureaucrat = TRUE;
			else
				aBureaucrat = aBureaucrat->itsSupervisor;
		}
		
		if (fFoundBureaucrat)
			return ((CNervousSystemDoc *) aBureaucrat);
		else 
			return (NULL);
	}
}


/******************************************************************************
 ChangeMyDataExtent

	If "tempLPt" is outside of myDataExtent, then stretch myDataExtent to encompase.
 ******************************************************************************/

void	CNBrainPan::ChangeMyDataExtent(LongPt *currLPt)
{
	LongPt		tempLPt;
	LongRect	tempMyDataExtent;
	LongRect	tempLRect;
	LongRect	combinedLRect;
	
	// Initialize working copies of myDataExtent and tempLPt.
	tempLPt = *currLPt;
	tempMyDataExtent = myDataExtent;
	
	if (!(PtInLongRect(&tempLPt, &tempMyDataExtent))) {
		
		SetLongRect(&tempLRect, tempLPt.h, tempLPt.v, tempLPt.h, tempLPt.v);
		UnionLongRect(&tempMyDataExtent, &tempLRect, &combinedLRect);
		myDataExtent = combinedLRect;

		AdjustBounds();
	}
}	


/******************************************************************************
 CalcMyDataExtent 

 ******************************************************************************/

void	CNBrainPan::CalcMyDataExtent(void) {
	long		i;
	LongRect	newExtent;
	CNObjectView *nextSubview = NULL;
	long		numItems;
	LongRect	subviewFrame;
	
	// Initialize the newExtent.
	SetLongRect(&newExtent, 0, 0, 0, 0);
	
	// Compose the bounding extents of all the subview items.
	numItems = itsSubviews->GetNumItems();
	for (i = 1; i <= numItems; ++i) {
	
		// Access the next item in the subview list.
		nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
		
		// Compute the bounding rect of this subview item in CNBrainPan coords.
		nextSubview->GetFrame(&subviewFrame);
		nextSubview->FrameToEnclR(&subviewFrame);
		
		// Union the bounds of this subview item to the accumulated total data bounds.
		UnionLongRect(&newExtent, &subviewFrame, &newExtent);
	}
	
	// This extra assignment is a requirement of "shadowing."
	myDataExtent = newExtent;
	
	// Re-adjust the "bounds" of this CPanorama to reflect the new "myDataExtent".
	AdjustBounds();
}


/******************************************************************************
 AdjustBounds 

	Adjust the bounds of my NBrainPan so that the right and bottom of the bounds
	are equal to the Max(frame, myDataExtent).right && .bottom.
	If the bounds are smaller than the frame then there will be area in the 
	visible pane where the DoClicks are not correctly sent to CNBrainPan.
 ******************************************************************************/
void	CNBrainPan::AdjustBounds(void) {
	LongRect    oldBounds = bounds;     /* Save old bounds. */
	LongRect	newBounds = oldBounds;  /* Initialize new bounds. */
	
	// "myDataExtent" and "frame" are in pixels, but bounds is in "panorama" units.
	
	if (myDataExtent.right > frame.right) 
		newBounds.right = myDataExtent.right / hScale;
	else 
		newBounds.right = frame.right / hScale;
	
	if (myDataExtent.bottom > frame.bottom) 
		newBounds.bottom = myDataExtent.bottom / vScale;
	else 
		newBounds.bottom = frame.bottom / vScale;
	
	if (!EqualLongRect( &oldBounds, &newBounds))
		SetBounds(&newBounds);
}


/******************************************************************************
 ObtainTraveler 
 
 	The double "**" for the "aNOView" argument is needed because the value of
 	this argument is changed by this method.  A completely new CNObjectView is 
 	created to replace the old "aNOView" in the new graphical context.  It may
 	be useful to think of the movie "The Invasion of the Body Snatchers" as a 
 	metaphor for this switch -- what comes in ("aNOView") and what goes out ("aNOView")
 	look identical, but they aren't. 
******************************************************************************/

void CNBrainPan::ObtainTraveler(CNObjectView **aNOView)
{
	CNObjectView *tempNOView = NULL;
	
	if (!(IncludesSubview(*aNOView))) {
		
		// Make an actual copy of *aNOView with this set as itsEnclosure.
		tempNOView = (*aNOView)->CopyNOViewInto(this, this);
		
		// Tell everyone concerned that I am being transplanted.
		(*aNOView)->BroadcastChange(NOBJECTVIEW_REPLACING_ANOTHER, ((void *) tempNOView));
		
		// The replacement is now complete, Dispose of the original, and make the switch.
		(*aNOView)->Dispose();
		(*aNOView) = NULL;
		(*aNOView) = tempNOView;
		tempNOView = NULL;
	
		// Refresh this object if its visible.
		(*aNOView)->Refresh();
	}
}


/******************************************************************************
 IncludesSubview
 
 	Return TRUE if the argument aNOView is already included in itsSubview list.
******************************************************************************/

Boolean CNBrainPan::IncludesSubview(CNObjectView *aNOView)
{
	CList		*tempSubviews = NULL;
	Boolean		fIncludes = FALSE;
	
	if ((aNOView) && (itsSubviews)) {
	
		tempSubviews = GetSubviews();  // Create a working copy of itsSubviews
		
		if (tempSubviews->Includes(aNOView))
			fIncludes = TRUE;
		
		tempSubviews->Dispose();
	}
	
	return (fIncludes);
}


/******************************************************************************
 GetSubviews
 
******************************************************************************/

CList	*CNBrainPan::GetSubviews(void)
{
	CList		*tempSubviews = NULL;

	if (itsSubviews) 
		tempSubviews = (CList *) itsSubviews->Copy();

	return (tempSubviews);
}


/******************************************************************************
 IsBeingDisposed 
 
******************************************************************************/

Boolean CNBrainPan::IsBeingDisposed(void)
{
	return fBeingDisposed;
}


/******************************************************************************
 EncloseWith

	Adjust the size of "this" if the new subview does not fit within the existing frame.
	Note: a call to ChangeSize() will automatically recalculate "aSubview's" aperture,
	so we don't have to call aSubview->CalcAperture() here.
 ******************************************************************************/

void	CNBrainPan::EncloseWith(CNObjectView *aSubview)
{
}


/******************************************************************************
 ShrinkWrapWithout

	Shrink the size of "this" to its new possibly smaller size because aSubview is removed.
 ******************************************************************************/

void	CNBrainPan::ShrinkWrapWithout(CNObjectView *aSubview)
{
}
	
	
/******************************************************************************
 AddSubview

 ******************************************************************************/

void	CNBrainPan::AddSubview(
	CView		*theSubview)
{
	LongRect	theFrameLong;
	Rect		theFrame;
	
	inherited::AddSubview(theSubview);
}

	
/******************************************************************************
 RemoveNOViews

 ******************************************************************************/

void	CNBrainPan::RemoveNOViews(CNOViewOwner *aNOVOwner) 
{
	ResetSelectionList();

	// If aNOView was given as an argument Transplant it to Limbo.
	// This Transplant strips it out of "this"
	if (aNOVOwner) 
		aNOVOwner->TravelAllTo(gNLimbo->GetItsMainPane());
}


/******************************************************************************
 AddNOVObjects

 ******************************************************************************/

void	CNBrainPan::AddNOVObjects(CNOViewOwner *aNOVOwner) 
{	
	ResetSelectionList();
	
	if (aNOVOwner) {
		
		// Prep "itsSelection" for re-assignment.
		itsSelection->Dispose();
		itsSelection = NULL;	
	
		// Adopt "aNOVOwner" as the new "itsSelection".
		itsSelection = aNOVOwner->CopyNOVOwnerFor(this);
		
		// Reset the graphical environment of the owned NOViews.
		itsSelection->TravelAllTo(this);

		// Tell all the new members of aNOVOwner that they are selected.
		itsSelection->SetFSelectedAll(TRUE);
	}
}


/******************************************************************************
 DoCommand {an OVERRIDE}

	Handle Cut, Copy, Paste, Clear, and Select All commands.
	
 ******************************************************************************/

void	CNBrainPan::DoCommand(long theCommand) 
{
	CNOViewCmdTask	*aNOVCmdTask = NULL;
	
	if ((theCommand == cmdCut) || (theCommand == cmdCopy) || (theCommand == cmdPaste) || 
			(theCommand == cmdClear) || (theCommand == cmdSelectAll)) {
		
		aNOVCmdTask = new(CNOViewCmdTask);
		
		switch (theCommand) {
			case cmdCut:
				aNOVCmdTask->INOViewCmdTask(kGMyUndoCut, theCommand, this);
				break;
			case cmdCopy:				
				aNOVCmdTask->INOViewCmdTask(kGMyUndoCopy, theCommand, this);
				break;
			case cmdPaste:				
				aNOVCmdTask->INOViewCmdTask(kGMyUndoPaste, theCommand, this);
				break;
			case cmdClear:
				aNOVCmdTask->INOViewCmdTask(kGMyUndoClear, theCommand, this);
				break; 						
			case cmdSelectAll:
				aNOVCmdTask->INOViewCmdTask(kGMyUndoSelectAll, theCommand, this);
				break; 			
		}
		
		Notify(aNOVCmdTask);
		aNOVCmdTask->Do();

	}
	else
		inherited::DoCommand(theCommand);
}
	

/******************************************************************************
 ResetSelectionList   {ORIGINAL: to manage CNBrainPan::itsSelection}

 ******************************************************************************/

void	CNBrainPan::ResetSelectionList(void) 
{
	if (itsSelection) {
		itsSelection->SetFSelectedAll(FALSE);
		itsSelection->SetEmptyAll();
	}
	else {
		itsSelection = new(CNOViewOwner);
		itsSelection->INOViewOwner(itsSupervisor);
	}
}


/******************************************************************************
 AddToSelection   {ORIGINAL: to manage CNBrainPan::itsSelection}

 ******************************************************************************/

void	CNBrainPan::AddToSelection(CNObjectView *newChosen) 
{
	if (newChosen) {
		newChosen->SetFSelected(TRUE);
		
		if (!itsSelection) {
			itsSelection = new(CNOViewOwner);
			itsSelection->INOViewOwner(itsSupervisor);
		}
		
		itsSelection->AcquireNOView(newChosen);
	}
}


/******************************************************************************
 RemoveFromSelection   {ORIGINAL: to manage CNBrainPan::itsSelection}

 ******************************************************************************/

void	CNBrainPan::RemoveFromSelection(CNObjectView *newUnChosen) 
{
	if (newUnChosen) {
		newUnChosen->SetFSelected(FALSE);
		
		if (itsSelection)
			itsSelection->RelinquishNOView(newUnChosen);
	}
}


/******************************************************************************
 UnionWithSelection   {ORIGINAL: to manage CNBrainPan::itsSelection}

 ******************************************************************************/

void	CNBrainPan::UnionWithSelection(CNOViewOwner *aNOVOwner) 
{
	if (aNOVOwner) {
		aNOVOwner->SetFSelectedAll(TRUE);
		
		if (!itsSelection) {
			itsSelection = new(CNOViewOwner);
			itsSelection->INOViewOwner(itsSupervisor);
		}
		
		itsSelection->Union(aNOVOwner);
	}
}


/******************************************************************************
 NotWithSelection   {ORIGINAL: to manage CNBrainPan::itsSelection}

 ******************************************************************************/

void	CNBrainPan::NotWithSelection(CNOViewOwner *aNOVOwner) 
{
	if (aNOVOwner) {
		aNOVOwner->SetFSelectedAll(FALSE);
		
		if (itsSelection)
			itsSelection->Not(aNOVOwner);
	}
}


/******************************************************************************
 CopyItsSelection   {ORIGINAL: to manage CNBrainPan::itsSelection}
 
		Return a copy of itsSelection (not an actual copy)
 ******************************************************************************/

CNOViewOwner	*CNBrainPan::CopyItsSelection(CBureaucrat *aSupervisor)
{
	CNOViewOwner *aNOVOwner = NULL;
	if (itsSelection) 
		aNOVOwner = itsSelection->CopyNOVOwnerFor(aSupervisor);

	return(aNOVOwner);
}


/******************************************************************************
 UpdateMenus {OVERRIDE}

		Enable appropriate Edit commands
 ******************************************************************************/

void	CNBrainPan::UpdateMenus(void)
{
	inherited::UpdateMenus();
	
	gBartender->EnableCmd(cmdSelectAll);
	
	if (itsSelection) 
		if (!(itsSelection->IsEmpty())) {
			gBartender->EnableCmd(cmdCut);
			gBartender->EnableCmd(cmdCopy);
			gBartender->EnableCmd(cmdClear);
		}
	
	if (!( ((CNNervousClipboard *) gClipboard)->PrivateNOVOwnerIsEmpty() )) 
		gBartender->EnableCmd(cmdPaste);
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a NBrainPan
 ******************************************************************************/

void	CNBrainPan::Dispose()
{
	fBeingDisposed = TRUE;

	if (itsSelection) 
		itsSelection->Dispose();
	
	inherited::Dispose();			/* Pass message on to superclass	*/
}


/******************************************************************************
 DoClick {an OVERRIDE}

		Mouse click inside the NBrainPan
 ******************************************************************************/

void	CNBrainPan::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{
	CNOViewTask	*theNOViewTask = NULL;
	Rect			bounds;
	Boolean			fSelectionTask = FALSE;
	Boolean			notifiable = TRUE;
	LongPt			trackStart;
	LongRect		limitsRect;  // Defines the where the window can auto scroll during TrackMouse
	
	LongToQDRect( &this->bounds, &bounds);
	QDToLongPt( hitPt, &trackStart);

	if (!(PtInRect(hitPt, &bounds))) {
		return;
	}

	/* Any new item creation will reset the selection to No-Items-Selected.	*/	
	if (gNTool != POINTER) 	
		ResetSelectionList();
	
	switch (gNTool) {
	
		case POINTER:
			fSelectionTask = TRUE;
			if (!(modifierKeys & shiftKey))   	// The "shift" key is not down.
				ResetSelectionList();
			break;
			
		case METER:
			break;
			
		case STIMULATORCELL:
			theNOViewTask = new(CNStimulatorViewTask);
			((CNStimulatorViewTask*) theNOViewTask)->INStimulatorViewTask(kGMyUndoCreateItem, this);
			break;
			
		case DENDRITE:
			theNOViewTask = new(CNDendriteViewTask);
			((CNDendriteViewTask*) theNOViewTask)->INDendriteViewTask(kGMyUndoCreateItem, this);
			break;
			
		case SOMA:
			theNOViewTask = new(CNSomaViewTask);
			((CNSomaViewTask*) theNOViewTask)->INSomaViewTask(kGMyUndoCreateItem, this);
			break;
			
		case AXON:
			theNOViewTask = new(CNAxonViewTask);
			((CNAxonViewTask*) theNOViewTask)->INAxonViewTask(kGMyUndoCreateItem, this);
			break;
			
		case SYNAPSE:
			theNOViewTask = new(CNSynapseViewTask);
			((CNSynapseViewTask*) theNOViewTask)->INSynapseViewTask(kGMyUndoCreateItem, this);
			break;
			
		case SIMULATIONSTART:
			break;
			
		case SIMULATIONSTOP:
			break;

		default:
			printf("The tool selected was not in the switch list, CNBrainPan::DoClick()\n");
			exit(1);
			break;
	}
	
	ForceNextPrepare();
	Prepare();
	
	if (fSelectionTask) ;
	else {
		SetLongRect(&limitsRect, MININT, MININT, MAXINT, MAXINT);
		
		TrackMouse(theNOViewTask, &trackStart, &limitsRect);

		if (notifiable) {			/* Make sure we don't re-notify in	*/
									/*   the case of a resumed drag		*/
			itsSupervisor->Notify(theNOViewTask);
		}
	}
}
	

/******************************************************************************
 Draw

 ******************************************************************************/

void	CNBrainPan::Draw	(Rect		*area)
{
	// note that 'area' is usually ignored; it has no relationship
	// to the size of the pane; it merely indicates what portion
	// (in QuickDraw coordinates) of the pane needs to be drawn
	
	Rect		theFrame;
	PenState	savePen;
	long		i;
	CNObjectView *nextSubview = NULL;
	long		numItems;
	LongRect	subviewAperture;
	LongRect	thisArea;
	LongRect	tempLongRect;
	Rect		tempRect;

	ForceNextPrepare();
	Prepare();

	GetPenState (&savePen);
	PenNormal ();
	FrameToQDR (&frame, &theFrame);
	
	// My code starts here.
	
	// Erase the Rect before redrawing it.
	EraseRect(area);
	
	// We only need to draw the subviews of this object, the object itself has not graphics.
	if (itsSubviews) {
	
		// Convert area to this CNBrainPan's frame coords.
		QDToFrameR(area, &thisArea);
		
		// Compose the bounding extents of all the subview items.
		numItems = itsSubviews->GetNumItems();
		for (i = 1; i <= numItems; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// Compute the aperture of this subview item in CNBrainPan coords.
			nextSubview->GetAperture(&subviewAperture);
			nextSubview->FrameToEnclR(&subviewAperture);
			
			// If the aperture of the subview intersects with the area to be drawn,
			// then draw the subview.
			if (SectLongRect(&thisArea, &subviewAperture, &tempLongRect)) {
				nextSubview->EnclToFrameR(&tempLongRect);
				LongToQDRect(&tempLongRect, &tempRect);
				nextSubview->Draw(&tempRect);
			}
		}
	}  // end of "if (itsSubviews) {"
	
	// My code ends here.

	SetPenState (&savePen);
} /* Draw */


/******************************************************************************
 INBrainPan

		Initialize a NBrainPan object
 ******************************************************************************/

void	CNBrainPan::INBrainPan(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing)
{
	CPanorama::IPanorama(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);

	wantsClicks = TRUE;
	SetCanBeGopher(TRUE);
	InitMyDataExtent();
	fBeingDisposed = FALSE;
}



/******************************************************************************
 InitMyDataExtent   :   original

 ******************************************************************************/

void	CNBrainPan::InitMyDataExtent(void) {
	LongRect tempRect;  // for shadowing.
	
	SetLongRect(&tempRect, 0, 0, 0, 0);
	myDataExtent = tempRect;
}


/******************************************************************************
 ChangeSize  :  overrides CPane::ChangeSize

		Change the size of a Pane. The delta rectangle parameter specifies
		how each side of the pane should change. Positive values mean a
		move to the right or down, negative values to the left or up.
 ******************************************************************************/

void	CNBrainPan::ChangeSize(
	register Rect		*delta,			/* Movement for each side			*/
	Boolean				redraw)			/* Redraw Pane or not?				*/
{
    
    LongRect    oldAperture = aperture;     /* Save old aperture                */
    LongRect	oldFrame = frame;
	Rect 		tmp;

	if (redraw) {
		Refresh();						/* Force redraw of area currently	*/
										/*   occupied by the pane			*/
		RefreshBorder();				/*  and redraw border if there is one */
	}
	
	ResizeFrame(delta);					/* Change dimensions of pane		*/
						
    CalcAperture();                     /* Determine new aperture           */
    
    /* My changes start here. */
    AdjustBounds();
	/* My changes end here. */

	if (redraw) {
		Refresh();						/* Force draw of pane at new size	*/
		RefreshBorder();	/* TCL 1.1.1 DLP 10/1/91 */

										/* If autoRefresh is FALSE (and		*/
										/*   other safety checks), we do	*/
										/*   not want to redraw the area	*/
										/*   which is common to the old and	*/
										/*   new pane size. Note that the	*/
										/*   Refresh method will have set	*/
										/*   the port for us already.		*/
		if (!autoRefresh && ReallyVisible()) {
			if (SectLongRect(&aperture, &oldAperture, &oldAperture))
			{
				
				FrameToQDR( &oldAperture, &tmp);
				ValidRect( &tmp);
			}
			if (itsBorder)
			{
				Rect oldBorder, newBorder;
				
				FrameToWindR( &oldFrame, &oldBorder);
				itsBorder->CalcBorderRect( &oldBorder);
				FrameToWindR( &frame, &newBorder);
				itsBorder->CalcBorderRect( &newBorder);
				
				if (SectRect( &oldBorder, &newBorder, &tmp))
				{
					GetWindow()->Prepare();
					ValidRect( &tmp);
					Prepare();
				}
				
			}
		}
	}
	
	if (itsSubviews != NULL)
		itsSubviews->DoForEach1((EachFunc1) Pane_AdjustToEnclosure, (long)delta);
}
	







