/******************************************************************************
 CNDendriteViewTask.c

							The NDendriteViewTask Class
		
	SUPERCLASS = CNOViewTask
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).

 ******************************************************************************/

#include "Global.h"
#include "CNDendriteViewTask.h"
#include "LongCoordinates.h"
#include "NervousSystemGlobals.h"
#include "CNDendriteView.h"
#include <limits.h>
#include "CNHotSpotView.h"
#include <CArray.h>
#include "CNBrainPan.h"


/******************************************************************************
 INDendriteViewTask

		Initialize a NDendriteViewTask object
 ******************************************************************************/

void	CNDendriteViewTask::INDendriteViewTask(
	short		aNameIndex, 
	CNBrainPan *aNBrainPan)
{
	inherited::INOViewTask(aNameIndex, aNBrainPan);
	
	penWidth = BRUSH_WIDTH_DENDRITE;
	GetIndPattern(&gPattern, sysPatListID, 1);

	// Initialize Graphic Data needed for "Redo".
	itsGraphicData = NULL;
	itsGraphicData = new(CArray);
	itsGraphicData->IArray(sizeof(LongPt));
	numGraphicData = 0;
}


/******************************************************************************
 CalcGModelBoundRect

	Calculate the bounding rectangle of the points contained in itsGraphicalData.
	We assume that there is at least one point in this task's graphical model.
 ******************************************************************************/

void	CNDendriteViewTask::CalcGModelBoundRect(LongRect *boundRect) {
	LongRect	tempRect;
	long		numPtsTaskGM;
	long		i;
	LongPt		nextPt;
		
	// Init tempRect.
	SetLongRect(&tempRect, LONG_MAX, LONG_MAX, LONG_MIN, LONG_MIN);

	// Calc the bounding rectangle of this task's graphical model.
	numPtsTaskGM = itsGraphicData->GetNumItems();
	for (i = 1; i <= numPtsTaskGM; ++i) {
		itsGraphicData->GetItem(&nextPt, i);
		if (nextPt.h < tempRect.left) 
			tempRect.left = nextPt.h;
		if (nextPt.v < tempRect.top) 
			tempRect.top = nextPt.v;
		if (nextPt.h > tempRect.right) 
			tempRect.right = nextPt.h;
		if (nextPt.v > tempRect.bottom) 
			tempRect.bottom = nextPt.v;
	}

	// The assignment
	*boundRect = tempRect;
}


/******************************************************************************
 CalcHSVBoundRect    

 ******************************************************************************/

void	CNDendriteViewTask::CalcHSVBoundRect(LongRect *boundingRect, LongPt aLocationPt)
{
	LongRect	tempRect;
	long		tempWidth;
	long		halfTempWidth;
	LongPt		halfWayPt;
	LongPt		offsetPt;
	
	// Initialize the bounding rect, adjusted for the pen width.
	tempWidth = DENDRITE_HOTPT_WIDTH + BRUSH_WIDTH_HOTSPOT;
	SetLongRect(&tempRect, 0, 0, tempWidth, tempWidth);
	
	// Put the bounding rect into window coordinates.
	halfTempWidth = tempWidth / 2;
	SetLongPt(&halfWayPt, halfTempWidth, halfTempWidth);
	offsetPt = aLocationPt;
	SubLongPt(&halfWayPt, &offsetPt);
	OffsetLongRect(&tempRect, offsetPt.h, offsetPt.v);
	
	// Make the assignment.
	*boundingRect = tempRect;
}


/******************************************************************************
 CreateNewNOView     {this method is an override}

	Create a new instance of CNDendriteView, and bind it to the ivar "itsNewNOView".
	Initialize itsNewNOView with the information from this task.
	Two CNHotSpotViews are added as subviews to itsNewNOView: one for the starting
	point and one for the end point.  itsNewNOView is auotmatically resized to 
	enclose the two subviews (addSubview does this for us).
	
	Note: this method is an almost exact copy of CNAxonViewTask::CreateNewNOView.
	
	We assume that there is at least one point in this task's graphical model.
 ******************************************************************************/

void	CNDendriteViewTask::CreateNewNOView(void) {
	CArray		*tempTaskGModel = NULL;  // A working copy of this task's itsGraphicalData
	long		numPtsTaskGM = 0;
	RgnHandle 	tempTaskGRegion;
	LongRect	boundRectGData;  // A bounding Rect for the view of the graphic model
	LongRect	boundRectHSVStart;  // A bounding Rect for the HotSpotView attached to the start point
	LongRect	boundRectHSVEnd;  // A bounding Rect for the HotSpotView attached to the end point
	LongPt		tempPt;
	CNHotSpotView 	*nHotSpotStart = NULL;
	CNHotSpotView 	*nHotSpotEnd = NULL;
	long		i;
	LongPt		localCoordOffsetPt;
	LongPt		nextPt;
		
  // Init working copies of this task's graphic data.
	tempTaskGModel = (CArray *) itsGraphicData->Copy();
	numPtsTaskGM = tempTaskGModel->GetNumItems();
	tempTaskGRegion = NewRgn();
	
	
  // Create a new instance of CNDendriteView.
  
	// Calc boundRectGData of this task's graphical model, and adjust for pen width.
	// In Window coordinates.
	CalcGModelBoundRect(&boundRectGData);
	boundRectGData.right += BRUSH_WIDTH_DENDRITE;   // Extra room for the brush width.
	boundRectGData.bottom += BRUSH_WIDTH_DENDRITE;  // The brush paints down & right of the mouse pt.
	
	// Create the new CNDendriteView instance, to fit the size of the mathematical model.
	itsNewNOView = new(CNDendriteView);
	((CNDendriteView*) itsNewNOView)->INDendriteView(itsNBrainPan, itsNBrainPan,
				(boundRectGData.right - boundRectGData.left), 
				(boundRectGData.bottom - boundRectGData.top), 
				boundRectGData.left, boundRectGData.top,
				sizFIXEDSTICKY, sizFIXEDSTICKY,
				0, 0, NULL);
	
	
  // Use the information recorded here in the CNDendriteViewTask's interaction with user to 
  // correctly initialize the new instance of CNDendriteView {::itsGraphicData, and ::itsRegion}.
	
	// Init and assign itsNewNOView's graphical model.
	// Init: Shift all the temp graphical model's points to the axon's local coordinate system.
	SetLongPt(&localCoordOffsetPt, boundRectGData.left, boundRectGData.top);
	for (i = 1; i <= numPtsTaskGM; ++i) {
		tempTaskGModel->GetItem(&nextPt, i);
		SubLongPt(&localCoordOffsetPt, &nextPt);
		tempTaskGModel->SetItem(&nextPt, i);
	}
	// Assign:
	((CNDendriteView *) itsNewNOView)->itsGraphicData = tempTaskGModel;  // I assume no auto dispose.
	
	// Init and Assign itsNewNOView's itsRegion.
	InitRegionWithGModel(tempTaskGRegion, tempTaskGModel);
	CopyRgn(tempTaskGRegion, itsNewNOView->itsRegion);
	
	// Initialize itsNBrainPan->itsSelection to point to this new object.
	itsNBrainPan->AddToSelection(itsNewNOView);
	
  // Create two new CNHotSpotView instances, one for the start point and one for the end.
  // These CNHotSpotView instances are subviews of itsNewNOView, and are enclosed by it.

	// Calc boundRectHSVStart of the HotSpotView that-will-be-attached to the start point, and  
	// adjust for pen width.  In Window coordinates.
	itsGraphicData->GetItem(&tempPt, 1);  // Get a copy of the start point.
	CalcHSVBoundRect(&boundRectHSVStart, tempPt);
	InsetLongRect(&boundRectHSVStart, -HALF_BRUSH_WIDTH_HOTSPOT, -HALF_BRUSH_WIDTH_HOTSPOT);
	
	nHotSpotStart = new(CNHotSpotView);
	((CNHotSpotView*) nHotSpotStart)->INHotSpotView(itsNewNOView, itsNewNOView, 
				(boundRectHSVStart.right - boundRectHSVStart.left), 
				(boundRectHSVStart.bottom - boundRectHSVStart.top), 
				(boundRectHSVStart.left - boundRectGData.left), 
				(boundRectHSVStart.top - boundRectGData.top),
				sizFIXEDSTICKY, sizFIXEDSTICKY,
				0, 0);
  
	// Calc boundRectHSVEnd of the HotSpotView that-will-be-attached to the end point, and  
	// adjust for pen width.  In Window coordinates.
	itsGraphicData->GetItem(&tempPt, numPtsTaskGM);  // Get a copy of the end point.
	CalcHSVBoundRect(&boundRectHSVEnd, tempPt);
	InsetLongRect(&boundRectHSVEnd, -HALF_BRUSH_WIDTH_HOTSPOT, -HALF_BRUSH_WIDTH_HOTSPOT);
	
	nHotSpotEnd = new(CNHotSpotView);
	((CNHotSpotView*) nHotSpotEnd)->INHotSpotView(itsNewNOView, itsNewNOView, 
				(boundRectHSVEnd.right - boundRectHSVEnd.left), 
				(boundRectHSVEnd.bottom - boundRectHSVEnd.top), 
				(boundRectHSVEnd.left - boundRectGData.left), 
				(boundRectHSVEnd.top - boundRectGData.top),
				sizFIXEDSTICKY, sizFIXEDSTICKY,
				0, 0);	


  // Do housekeeping.
	DisposeRgn(tempTaskGRegion);
	tempTaskGModel = NULL;
	nHotSpotStart = NULL;
	nHotSpotEnd = NULL;
}


/******************************************************************************
 InitRegionWithGModel 

 ******************************************************************************/

void	CNDendriteViewTask::InitRegionWithGModel(RgnHandle tempRegion, CArray *tempGraphicData) {
	long		i;
	LongPt		currPt;
	LongPt		prevPt;
	LongRect	currRect;
	LongRect	prevRect;
	RgnHandle 	newAdditionRegion;
	long		numItems;
	
	numItems = tempGraphicData->GetNumItems();
	if (numItems > 0) {
		// Initialize tempRegion.
		SetEmptyRgn(tempRegion);
		
		// Draw all the converted points into the CNObjectView's own "tempRegion".
		tempGraphicData->GetItem(&prevPt, 1);
		for (i = 2; i <= numItems; ++i) {
			tempGraphicData->GetItem(&currPt, i);
			CalcPenRect( &currPt, &currRect);
			CalcPenRect( &prevPt, &prevRect);
			
			newAdditionRegion = NewRgn();
			CalcDrawRgn(&currRect, &prevRect, newAdditionRegion);
			
			UnionRgn(tempRegion, newAdditionRegion, tempRegion);
			DisposeRgn(newAdditionRegion);
			
			prevPt = currPt;	
		}
	}
}


/******************************************************************************
 AppendGraphicDataPt

	Append a new point to the objectTask's model of its graphical representation.
	These points are all in the coordinate system of itsEnclosure.
	We will convert to local coordinates before we copy the Graphics Model to 
	the new CNDendriteView object.
 ******************************************************************************/

void CNDendriteViewTask::AppendGraphicDataPt(LongPt *newPoint) {

	LongPt tempPt = *newPoint;  // We shadow *newPoint with tempPt.
	
	// Increment before indexing because the array starts at "1" instead of "0".
	++numGraphicData;
	
	// Append the new point.
	itsGraphicData->InsertAtIndex(&tempPt, numGraphicData);
}


/******************************************************************************
 Dispose     "Dispose of a NOViewTask object"
 
	We do not Dispose of "*itsNBrainPan", or "*itsNewNOView", they are only pointers
	to objects who's existance is independent of this task.  The scopes of these 
	local pointers will expire automatically when the task object is Disposed.
 ******************************************************************************/

void	CNDendriteViewTask::Dispose()
{
	itsGraphicData->Dispose();	// Throw out its graphical model needed for "Redo".

	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 BeginTracking

		Mouse down tracking is about to begin at the specified starting
		point. 
 ******************************************************************************/

void	CNDendriteViewTask::BeginTracking(
	LongPt		*startPt)
{
	inherited::BeginTracking(startPt);
	
	// Append the first point to the array of graphic data points for the item.
	AppendGraphicDataPt(startPt);
}


/******************************************************************************
 UseTool

		Mouse down tracking has ended because the user has released the
		mouse button.
 ******************************************************************************/

void	CNDendriteViewTask::UseTool(
	LongPt		*currPt,
	LongPt		*prevPt)
{
	LongRect	currRect;
	LongRect	prevRect;
	RgnHandle 	newAdditionRegion;
	
	CalcPenRect( currPt, &currRect);
	CalcPenRect( prevPt, &prevRect);
		
	// Calculate and draw on the screen the new addition to the region.
	newAdditionRegion = NewRgn();
	CalcDrawRgn(&currRect, &prevRect, newAdditionRegion);
	itsNBrainPan->Prepare();
	FillRgn(newAdditionRegion, &gPattern);

	// Add the new drawing to the existing region.
	UnionRgn(itsRegion, newAdditionRegion, itsRegion);
	DisposeRgn(newAdditionRegion);
	
	// Append to the array of graphic data points for the item.
	// This is how we implement a "draw" program instead of a "paint" program.  
	// This item's array is model of its graphical representation.
	AppendGraphicDataPt(currPt);
}




