/******************************************************************************
 CNNervousClipboard.c

							The NNervousClipboard Class
	
	This is a subclass of TCL's CClipboard; and so, uses the hooks that CClipboard 
	provides for using a private scrap and any application-specific information.
	
	SUPERCLASS = CClipboard
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).

	Note: you must call EmptyScrap before PutData; and EmptyPrivateScrap before PutPrivateScrap.

 ******************************************************************************/

#include "Global.h"
#include "Constants.h"
#include "Commands.h"
#include "CNNervousClipboard.h"
#include "CScrollPane.h"
#include "CPicture.h"
#include "CDesktop.h"
#include "CBartender.h"
#include "CEditText.h"
#include "CApplication.h"
#include "CWindow.h"
#include "CPanorama.h"
#include "CNBrainPan.h"
#include "CNMyGlobals.h"
#include "CNOViewOwner.h"
#include "LongCoordinates.h"
#include "NervousSystemDoc.h"


/*** Global Variables ***/
extern CDesktop		*gDesktop;			/* The visible Desktop				*/
extern CBartender	*gBartender;		/* Manages all menus				*/
extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern Boolean		gInBackground;		/* In background under MultiFinder	*/
extern CNervousSystemDoc *gNLimbo;


/******************************************************************************
 INNervousClipboard

		Initialize a NNervousClipboard object
 ******************************************************************************/

void	CNNervousClipboard::INNervousClipboard(
    CApplication    *aSupervisor,       /* Boss in chain of command         */
	Boolean			hasWindow)			/* Can contents be displayed?		*/
{
	CClipboard::IClipboard(aSupervisor, hasWindow);	/* Initialize superclass			*/
	
	itsPrivateNOVOwner = NULL;
	itsPrivateType = '    ';
}


/******************************************************************************
 UpdateDisplay

 ******************************************************************************/

void	CNNervousClipboard::UpdateDisplay()
{
	Handle			theData = NULL;
	long			theType;
	ScrapStatus		sStatus;
	
	//Rect tempRect;
	//LongRect tempLongRect;
	CNOViewOwner *aNOVOwner = NULL;
	
	if (windowVisible) {
		sStatus = Status();
		if ((sStatus == GLOBAL_SCRAP_NEWER) ||
			((sStatus == SCRAPS_THE_SAME) && (itsPrivateType == '    ')))
			inherited::UpdateDisplay();
		
		else {
			if (itsContents) {
				itsContents->Refresh();
				
				// If there is some data in the clipboard, Transplant it to Limbo before
				// making a new graphical environment to put it into.
				// This call must come before "itsContents->Dispose()"
				if (itsPrivateNOVOwner)
					itsPrivateNOVOwner->TravelAllTo(gNLimbo->GetItsMainPane());

				itsContents->Dispose();
				itsContents = NULL;
				itsScrollPane->itsPanorama = NULL;
			}
			
			if (itsPrivateNOVOwner)
			{
				itsContents = MakeClipViewPrivate();
				if (itsContents)
				{
					itsScrollPane->InstallPanorama( itsContents);
					
					// Make a shadow copy of itsPrivateNOVOwner and send it to itsContents.
					aNOVOwner = itsPrivateNOVOwner->CopyNOVOwnerFor(gNLimbo);
					((CNBrainPan *) itsContents)->AddNOVObjects(aNOVOwner);
					aNOVOwner->Dispose();
					
					// Since this is the clipboard, we don't want things auto-selected.
					((CNBrainPan *) itsContents)->ResetSelectionList();

					itsContents->Refresh();
					
				//	itsContents->GetAperture(&tempLongRect);
				//	LongToQDRect(&tempLongRect, &tempRect);
				//	itsContents->Draw(&tempRect);
				}
			}
		}
	}
}


/******************************************************************************
 MakeClipViewPrivate
 
	Make a view corresponding to the given clipboard data type. MakeClipViewPrivate
	is responsible for disposing the dataHandle when appropriate. Override
	this method to make views for different types of data.
 ******************************************************************************/

CPanorama* CNNervousClipboard::MakeClipViewPrivate(void)
{
	CNBrainPan	*clipView = NULL;
	
	clipView = new(CNBrainPan);
	
	clipView->INBrainPan(itsScrollPane, this,
									1, 1, 0, 0, sizELASTIC, sizELASTIC);
	clipView->FitToEnclosure(TRUE, TRUE);

	
//	clipView->IViewRes ('Pano', 128, itsScrollPane, this);

	return clipView;
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a Director
 ******************************************************************************/

void	CNNervousClipboard::Dispose()
{
	EmptyPrivateScrap();
	
	inherited::Dispose();
}


/******************************************************************************
 GetPrivateScrap

		Retrieve data from our private scrap. Return
		FALSE if no data is found.
 ******************************************************************************/

Boolean	CNNervousClipboard::GetPrivateScrap(ResType theType, CNOViewOwner **thePrivateData)
{
	if ((itsPrivateNOVOwner) && (theType == itsPrivateType)) {
		*thePrivateData = itsPrivateNOVOwner->CopyNOVOwnerActualFor(gNLimbo->GetItsMainPane());
		return TRUE;
	}
	else
		return FALSE;
}


/******************************************************************************
 PutPrivateScrap

		Place data on the private scrap
 ******************************************************************************/

void	CNNervousClipboard::PutPrivateScrap(
	ResType		theType,				/* Resource type of the data		*/
	CNOViewOwner 	*aNOVOwner)
{
	if (theType == 'NSEL') {
		
		// Insurance: init itsPrivateNOVOwner for accepting the new data.
		EmptyPrivateScrap();
		
		// Assign "itsPrivateNOVOwner" to the new private data.
		itsPrivateNOVOwner = aNOVOwner->CopyNOVOwnerFor(this);

		//  Do house-keeping.
		itsPrivateType = theType;
		PrivateChanged();
	}
}
	

/******************************************************************************
 EmptyPrivateScrap

	Clear the current contents of the global scrap.
 ******************************************************************************/

void CNNervousClipboard::EmptyPrivateScrap(void)
{
	if (itsPrivateNOVOwner) {
		itsPrivateNOVOwner->DisposeAll();
		itsPrivateNOVOwner = NULL;
	}
	
	itsPrivateType = '    ';
}


/******************************************************************************
 PrivateNOVOwnerIsEmpty 

 ******************************************************************************/

Boolean	CNNervousClipboard::PrivateNOVOwnerIsEmpty(void) 
{
	long numMembers;
	
	if (!itsPrivateNOVOwner)
		return TRUE;
	else {
		return (itsPrivateNOVOwner->IsEmpty());
	}
}



/******************************************************************************
 ConvertGlobal

		Take data from the global desk scrap and put it in our private
		scrap, performing any necessary conversions. Subclasses which
		support a private scrap must override this method.
 ******************************************************************************/

void	CNNervousClipboard::ConvertGlobal()
{
}										/* Null Method						*/


/******************************************************************************
 ConvertPrivate

		Place information from our private scrap into the global desk scrap,
		performing any necessary conversions. Standard types for the
		global desk scrap are 'PICT' and 'TEXT'. Subclasses which support
		a private scrap must override this method.
 ******************************************************************************/

void	CNNervousClipboard::ConvertPrivate()
{
}										/* Null Method						*/



