/******************************************************************************
 CNNeuronView.c
	
	The NNeuronView Class
	
	SUPERCLASS = CNOVCollection
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 ******************************************************************************/

#include "Global.h"
#include "CNNeuronView.h"
#include <stdio.h>
#include <string.h>
#include "CList.h"
#include <QuickDraw.h>
#include "CNOVDraggerTask.h"
#include "CNMyGlobals.h"
#include "CNOViewOwner.h"
#include "CPaneBorder.h"
#include "CNSomaView.h"
#include "CNCompositeModel.h"
#include "CNHotSpotView.h"


// External global variables used here.
extern short gClicks;


/******************************************************************************
 INNeuronView

		Initialize a NNeuronView object
 ******************************************************************************/

void	CNNeuronView::INNeuronView(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin, 
	long			aVFrameOrigin, 
	CNNeuronModel	*aNNeuronModel)
{
	CNOVCollection::INOVCollection(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, 
					aHFrameOrigin, aVFrameOrigin);

	// Initialize the instance of the NModelElem managed by this view.
	if (aNNeuronModel)
		myNModelElem = aNNeuronModel->CopyModel();
	else {
		myNModelElem = new(CNNeuronModel);
		((CNNeuronModel *) myNModelElem)->INNeuronModel();
	}
}


/******************************************************************************
 CopyNOViewInto {OVERRIDE}
 
******************************************************************************/

CNObjectView *CNNeuronView::CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor)
{
	CNNeuronView		*theCopy = NULL;
	
	// Create a new CNObjectView as a copy.
	theCopy = new(CNNeuronView);

	// Initialize the new object to be a copy of "this".
	theCopy->INNeuronView(anEnclosure, aSupervisor, width, height, 
				hEncl, vEncl, hSizing, vSizing, hFrameOrigin, vFrameOrigin, 
				((CNNeuronModel *) myNModelElem));
	
	// Initialize a flag saying that "this" is being copied.  This flag is used
	// in calls to Enclose with to know whether subviews need to be moved into 
	fBeingCopied = TRUE;
	theCopy->fBeingCopied = TRUE;

	INOViewCopy(theCopy);
	
	INModelCopy(theCopy);
		
	// Copying completed.
	fBeingCopied = FALSE;
	theCopy->fBeingCopied = FALSE;
	
	return theCopy;
}


/******************************************************************************
 INModelCopy 
 
 	Initialize the model of a copy of this NOView.
	Copy all the connections from the original model of this object.
******************************************************************************/

void CNNeuronView::INModelCopy(CNObjectView *theCopy)
{
	(theCopy->myNModelElem)->MakeConnections(theCopy->BuildListOfModelsOfSubviews());
}


/******************************************************************************
 INOViewCopy 
 
 	Initialize a copy of this NOView.
******************************************************************************/

void CNNeuronView::INOViewCopy(CNObjectView *theCopy)
{
	inherited::INOViewCopy(theCopy);
}


/******************************************************************************
 SetCopiedModelsIdenticalToOriginals  		{behavior is specific to children}
 
******************************************************************************/

void CNNeuronView::SetCopiedModelsIdenticalToOriginals(
	CNObjectView *aNOView, 
	CNObjectView *aCopiedNOView)
{
	CNModel			*nextModel = NULL;
	CNModel			*nextCopiedModel = NULL;
	
	if ((aNOView) && (aCopiedNOView)) {
	
		//INV: both aNOView and aCopiedNOView are CNObjectView's.

		// Access the models of these object subviews.
		nextModel = aNOView->GetModel();
		nextCopiedModel = aCopiedNOView->GetModel();
		
		// Set the copied model to be identical to the original model.
		nextCopiedModel->MakeIdentical(nextModel);
	}
}


/******************************************************************************
 FindHSpotWithinTraveledObj  		{this is an override}
 
******************************************************************************/

void CNNeuronView::FindHSpotWithinTraveledObj(
	CNObjectView *aNOView, 
	CNObjectView *aCopiedNOView, 
	CNHotSpotView **aNHSpot)
{
	int		objectSubviewIndex = -1;
	CNObjectView	*anObjectView = NULL;
	CNHotSpotView	*aTraveledHotSpotView = NULL;
	
	if ((aNOView) && (aCopiedNOView) && (*aNHSpot)) {
		
		// Initialize CNObjectView information.
		anObjectView = (CNObjectView *) (*aNHSpot)->itsEnclosure;
		objectSubviewIndex = anObjectView->GetIndexOfSubview((*aNHSpot));
		if (anObjectView != ((CNObjectView *) aNOView)) return;
		
		// Access the corresponding CNHotSpotView within the copy.
		aTraveledHotSpotView = (CNHotSpotView *) (aCopiedNOView->itsSubviews)->
				NthItem(objectSubviewIndex);
		
		// Make the assignment;
		(*aNHSpot) = aTraveledHotSpotView;
	}
}


/******************************************************************************
 ContainsSoma

	Determines whether one of the immediate subviews of "this" is a CNSomaView.
	This method does not recursively search the visual hierarchy of "this" to all
	of its sub-sub-... views; we assume that we are only interested in whether
	an immediate subview is a soma, and not if it encloses a subview at all.
 ******************************************************************************/

Boolean CNNeuronView::ContainsSoma(void)
{
	long			numItems = 0;
	long			i;
	CNObjectView	*nextSubview = NULL;
	Boolean			fContainsSoma = FALSE;

	if (itsSubviews) {
		
		numItems = itsSubviews->GetNumItems();
		for (i = 1; i <= numItems; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// Test whether nextSubview is a CNSomaView.
			if (member(nextSubview, CNSomaView)) {
				fContainsSoma = TRUE;
				break;
			}
		}
	}
	
	return fContainsSoma;
}


/******************************************************************************
 Dispose 

		Dispose of a NNeuronView
 ******************************************************************************/

void	CNNeuronView::Dispose()
{
		/* makes sure that the last undo task is disposed. Since the 	*/
		/* undo tasks usually have references back to this pane, 		*/
		/* it would be bad if this pane was disposed and the task was 	*/
		/* not informed.												*/

	inherited::Dispose();			/* Pass message on to superclass	*/
}


