/******************************************************************************
 CNOVCollection.c
	
	The NOVCollection Class
	
	SUPERCLASS = CNObjectView
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 ******************************************************************************/

#include "Global.h"
#include "CNOVCollection.h"
#include <stdio.h>
#include <string.h>
#include "CList.h"
#include <QuickDraw.h>
#include "CNOVDraggerTask.h"
#include "CNMyGlobals.h"
#include "CNOViewOwner.h"
#include "CPaneBorder.h"
#include "CNBrainPan.h"
#include "CNHotSpotView.h"


// External global variables used here.
extern short gClicks;


/******************************************************************************
 INOVCollection

		Initialize a NOVCollection object
 ******************************************************************************/

void CNOVCollection::INOVCollection(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin, 
	long			aVFrameOrigin)
{
	CNObjectView::INObjectView(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, 
					aHFrameOrigin, aVFrameOrigin);
	
	wantsClicks = TRUE;
	SetCanBeGopher(TRUE);
}


/******************************************************************************
 INModelCopy 
 
 	A place-holder method.
******************************************************************************/

void CNOVCollection::INModelCopy(CNObjectView *theCopy)
{
}


/******************************************************************************
 SetCopiedModelsIdenticalToOriginals  		{behavior is specific to children}
 
******************************************************************************/

void CNOVCollection::SetCopiedModelsIdenticalToOriginals(
	CNObjectView *aNOView, 
	CNObjectView *aCopiedNOView)
{

}


/******************************************************************************
 FindHSpotWithinTraveledObj  		{behavior is specific to children}
 
******************************************************************************/

void CNOVCollection::FindHSpotWithinTraveledObj(
	CNObjectView *aNOView, 
	CNObjectView *aCopiedNOView, 
	CNHotSpotView **aNHSpot)
{

}


/******************************************************************************
 ObtainTravelerForCollection  		
 
 This method is an overload of CNObject::ObtainTraveler that takes a second argument
 to a CNHotSpot and maintains that pointer over to the copy.
******************************************************************************/

void CNOVCollection::ObtainTravelerForCollection(CNObjectView ** aNOView, CNHotSpotView **aNHSpot)
{
	CNObjectView *tempNOView = NULL;
	LongPt		placePt;
	LongRect	tempSubviewFrame;
	Point		tempPt;
	
	if (!(IncludesSubview(*aNOView))) {
		
		// Before putting *aNOView into its new graphical environment, we must
		// translate it from its coordinate system into the coordinate system of
		// its new enclosure.
		(*aNOView)->GetFrame(&tempSubviewFrame);
		SetLongPt(&placePt, tempSubviewFrame.left, tempSubviewFrame.top);
		(*aNOView)->FrameToWind(&placePt, &tempPt);
		WindToFrame(tempPt, &placePt);
		(*aNOView)->Place(placePt.h, placePt.v, TRUE);
		
		// Make an actual copy of *aNOView with this set as itsEnclosure.
		tempNOView = (*aNOView)->CopyNOViewInto(this, this);
		
		// Tell everyone concerned that I am being transplanted.
		(*aNOView)->BroadcastChange(NOBJECTVIEW_REPLACING_ANOTHER, ((void *) tempNOView));
		
		// Find where "aNHSpot" belongs within the copy.
		FindHSpotWithinTraveledObj(*aNOView, tempNOView, aNHSpot);
		
		// Initialize the models of "tempNOView" so that they are identical to the 
		// original models, and not just copies.
		SetCopiedModelsIdenticalToOriginals(*aNOView, tempNOView);
		
		// The replacement is now complete, Dispose of the original, and make the switch.
		(*aNOView)->Dispose();
		(*aNOView) = NULL;
		(*aNOView) = tempNOView;
		tempNOView = NULL;

		// Refresh this object if its visible.
		(*aNOView)->Refresh();
	}

	// Get rid of the old region, and replace it with a new region.
	DisposeRgn(itsRegion);
	itsRegion = CalcCombinedSubviewRegion();
}


/******************************************************************************
 EncloseWith

	This is a overwrite of CNObjectView::EncloseWith.  A Neuron object needs slightly 
	different functionality for enclosing new subviews.

	Since an instance of CNOVCollection, like "this", does not have its own graphical 
	model, it doesn't have a shape or location without its subviews.  So when the first
	subview is added, shrink wrap "this" to this first subview.
	
	Note: although "aSubview" is enclosed by "this" it is still in the coordinate system
	of its old Enclosure; we make use of this information and then Place "aSubview" where
	it needs to go within "this."
 ******************************************************************************/

void CNOVCollection::EncloseWith(CNObjectView *aSubview)
{	
	// If this is the first subview, then do shrink wrapping too.
	// If "this" is not being copied then position it according to its first subview.
	if ((GetNumSubviews() == 1) && (aSubview) && (!fBeingCopied)) {
		
		InitLocationByFirstSubview(aSubview);
		
		// Move firstSubview to where it needs to go within "this."
		aSubview->Place(0, 0, TRUE);

		inherited::EncloseWith(aSubview);
	}
	
	else 
		inherited::EncloseWith(aSubview);
}


/******************************************************************************
 InitLocationByFirstSubview 

	A Neuron object needs 
	slightly different functionality for enclosing new subviews, because it doesn't have 
	any graphic value by itself.
 ******************************************************************************/

void	CNOVCollection::InitLocationByFirstSubview(CNObjectView	*firstSubview)
{
	LongPt		placePt;
	LongRect	tempSubviewFrame;
	Point		tempPt;
	
	// INV: (itsSubviews) && (GetNumSubviews() == 1) && "firstSubview" is being added now.
	
	if (firstSubview) {	
	
		// Move "this" to the right spot.
		// Note: firstSubview is still in the coordinate system of its previous enclosure
		// (the same enclosure that now encloses "this"), so we can just move "this" to the 
		// spot where firstSubview thinks it is.
		// Move "this" to where the "firstSubview" is in the window.
		firstSubview->GetFrame(&tempSubviewFrame);
		SetLongPt(&placePt, tempSubviewFrame.left, tempSubviewFrame.top);
		firstSubview->FrameToWind(&placePt, &tempPt);
		WindToFrame(tempPt, &placePt);
		FrameToEncl(&placePt);
		Place(placePt.h, placePt.v, TRUE);
	}
}
	
	
/******************************************************************************
 DoClickEnclosure {OVERRIDE}

	This method is almost identical to CNObjectView::DoClick.  Instances of CNOVCollection
	need to handle clicks, but not directly, because they do not have any graphic objects
	of their own.  For example, when I click on an NAxonView that is part of a NOVCollection
	the "DoClick" is automatically sent to NAxonView, from there we have to implement a
	way for NAxonView's NOVCollection enclosure to do the actual work.  This method is 
	where that work is actually done.
	
	The only substantial difference between this method and CNObjectView::DoClick is
	in the call to "INOVDraggerTask".
 ******************************************************************************/

void	CNOVCollection::DoClickEnclosure(
	Point		hitPt,
	short		modifierKeys,
	long			when,
	CNObjectView 	*aNOView)
{
	Rect			tempFrame;
	LongRect		aNOVBounds;
	LongRect		tempFrameLong;
	Boolean			notifiable = TRUE;
	LongPt			trackStart;
	CNOVDraggerTask	*theNOVDragTask = NULL;  
	LongPt			tempLongPt;
	
	// INV: (gNTool == POINTER) && (aNOView)

	// Double check that we should be here.
	GetFrame(&tempFrameLong);
	LongToQDRect(&tempFrameLong, &tempFrame);
	if (!(PtInRect(hitPt, &tempFrame))) return;
	
	// Initialize stuff used in the Call to TrackMouse for CNDraggerTask.
	itsNBrainPan->GetBounds(&aNOVBounds);
	QDToLongPt( hitPt, &trackStart);
	
	// "this" belongs to a CNOVCollection, and (gNTool == POINTER), so 
	// itsEnclosure needs to do the dragging and selecting work.
	if (member(itsEnclosure, CNOVCollection)) {
	
		// Initialize stuff used to pass the work to another object.
		// The hitPt must be converted into itsEnclosure's coordinate system.
		QDToLongPt(hitPt, &tempLongPt);
		FrameToEncl(&tempLongPt);
		LongToQDPt(&tempLongPt, &hitPt);
				
		((CNOVCollection *) itsEnclosure)->DoClickEnclosure(hitPt, modifierKeys, when, aNOView);
	}
	
	// "this" is a CNOVCollection, "itsEnclosure" isn't, and (gNTool == POINTER),
	// so "this" needs to do its own selection and dragging work.
	else {   
		
		// The "shift" key is down, so just edit the selection list.
		if (modifierKeys & shiftKey) {  	// The "shift" key IS down.
			if (!fSelected) {
				itsNBrainPan->AddToSelection(this);
			}
			else {
				itsNBrainPan->RemoveFromSelection(this);
			}
		}
		
		// Try to "Drag" the selection, maybe after reseting it to only "this".
		else {  							// The "shift" key is NOT down.
			
			// First click is for selecting and/or dragging.
			if (gClicks == 1) {
				
				// If "this" isn't already part of the selection list then reset the
				// selection list and make this the only member.
				if (!fSelected) {
					itsNBrainPan->ResetSelectionList();
					itsNBrainPan->AddToSelection(this);
				}
				
				// Initialize the DragTask.
				theNOVDragTask = new(CNOVDraggerTask);
				theNOVDragTask->INOVDraggerTask(kGMyUndoMoveItem, this, aNOView, itsNBrainPan);
				
				// Try to drag.  This "TrackMouse" needs to be done by itsNBrainPan -- 
				// that's where it's being dragged.
				FrameToItsNBrainPan(&trackStart);
				itsNBrainPan->TrackMouse(theNOVDragTask, &trackStart, &aNOVBounds);
				itsNBrainPan->Refresh();
				
				// If I didn't actually "Drag", then Dispose of the false-alarm DragTask.
				if (!(theNOVDragTask->DidYouDrag())) {	// A selection click, not a drag.				
					theNOVDragTask->Dispose();
				}
				
				// I did actually drag so init pointers to the DragTask for future use.
				else {  	// A drag click.						
					if (notifiable) {			
												
						itsSupervisor->Notify(theNOVDragTask);
					}
				}
			}
			
			// Second click is for getting the dialog box.
			else if (gClicks == 2) {
				// Perform a dialog with the user about the parameters of this item.
				aNOView->DoDialog();
			}
		}
	}
}
	

/******************************************************************************
 ShrinkWrapWithout    {this is an override}

	This is a overwrite of CNObjectView::ShrinkWrapWithout.  A Neuron object needs 
	slightly different functionality for enclosing new subviews, because it doesn't have 
	any graphic value by itself, so itsRegion is not used.
	
	Shrink the size of "this" to its new possibly smaller size because aSubview is removed.
 ******************************************************************************/

void	CNOVCollection::ShrinkWrapWithout(CNObjectView *aSubview)
{
	LongRect	combinedFrameRect;
	long		i;
	long		numSubviews = 0;
	CNObjectView	*nextSubview = NULL;
	LongRect	subviewFrameRect;
	LongRect	thisFrameRect;
	LongRect	subviewDelta;
	Rect		tempRect;
	Boolean		fInitialized = FALSE;

  	// Calculate the new frame of "this" without the subview "aSubview".
	if (IncludesSubview(aSubview)) {
		
		numSubviews = itsSubviews->GetNumItems();
		
		// Unravel the loop one step to initialize "combinedFrameRect".
		i = 1;
		while ((i <= numSubviews) && (!(fInitialized))) {

			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// Initialize with the frame of the first subview that isn't == to aSubview.
			if (nextSubview != aSubview) {
				nextSubview->GetFrame(&subviewFrameRect);
				nextSubview->FrameToEnclR(&subviewFrameRect);
				combinedFrameRect = subviewFrameRect;
				fInitialized = TRUE;
			}
			
			// Increment counter.
			++i;
		}
		
		// Now do the rest of the loop.
		for (; i <= numSubviews; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// Accumulate all the rest of nextSubviews' frames, except "aSubview's".
			if (nextSubview != aSubview) {
				nextSubview->GetFrame(&subviewFrameRect);
				nextSubview->FrameToEnclR(&subviewFrameRect);
				UnionLongRect(&subviewFrameRect, &combinedFrameRect, &combinedFrameRect);
			}
		}
	
		// Change the size of "this" to accomodate the new subview.  Determine if "this" needs 
		// to be shrunk.
		GetFrame(&thisFrameRect);
		if (!(EqualLongRect(&thisFrameRect, &combinedFrameRect))) {
		
			// INV: "this" needs some shrinking.
		
			SetLongRect(&subviewDelta, 0, 0, 0, 0);
			subviewDelta.left = 	combinedFrameRect.left - 	thisFrameRect.left;
			subviewDelta.right = 	combinedFrameRect.right - 	thisFrameRect.right;
			subviewDelta.top = 		combinedFrameRect.top - 	thisFrameRect.top;
			subviewDelta.bottom = 	combinedFrameRect.bottom - 	thisFrameRect.bottom;
			LongToQDRect(&subviewDelta, &tempRect);
			ChangeSize(&tempRect,TRUE);
		}
	}
}
	
	
/******************************************************************************
 Draw		{this is an overrite}

	CNOVCollection requires slightly different behavior on "Draw" than CNObjectView
	because CNOVCollection doesn't have any real graphical data itself.  A call to "Draw"
	here is only a call to "Draw" all of "this's" visible subviews, without "Draw"ing
	this::itsRegion.  this::itsRegion is only used for pick-correlation.
 ******************************************************************************/
 
void	CNOVCollection::Draw(Rect *area)
{
	long		numSubviews;
	long		i;
	CNObjectView *nextSubview = NULL;
	LongRect	subviewAperture;
	Rect		tempRect;
	
	// Draw all of "this"::subviews	
	if (itsSubviews) {
	
		// Compose the bounding extents of all the subview items.
		numSubviews = itsSubviews->GetNumItems();
		for (i = 1; i <= numSubviews; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// If "nextSubview" is visible, then tell it to draw itself.
			if (nextSubview->ReallyVisible()) {
				GetAperture(&subviewAperture);
				LongToQDRect(&subviewAperture, &tempRect);
				nextSubview->Draw(&tempRect);
			}
		}
	}
}


/******************************************************************************
 Dispose 

		Dispose of a NOVCollection
 ******************************************************************************/

void	CNOVCollection::Dispose()
{
		/* makes sure that the last undo task is disposed. Since the 	*/
		/* undo tasks usually have references back to this pane, 		*/
		/* it would be bad if this pane was disposed and the task was 	*/
		/* not informed.												*/

	inherited::Dispose();			/* Pass message on to superclass	*/
}


