/******************************************************************************
 CNOVConnectTask.c

							The NOVConnectTask Class
		
	SUPERCLASS = CTask
	
	Copyright  1993 by Daniel McFarlane. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CNOVConnectTask.h"
#include "CNObjectView.h"
#include "CNHotSpotView.h"
#include "CNOVCollection.h"
#include "CNNeuronView.h"
#include "CNNetworkView.h"
#include "CNBrainPan.h"
#include "CNSomaView.h"
#include "CList.h"
#include "CNHotSpotView.h"
#include "CNOVConnectChore.h"
#include "CApplication.h"
#include "NervousSystemDoc.h"
#include "CNModelElem.h"
#include "CNModel.h"
#include "CNCompositeModel.h"
#include "CNOViewOwner.h"

extern CApplication	*gApplication;
extern CNervousSystemDoc *gNLimbo;


/******************************************************************************
 INOVConnectTask

		Initialize a NOVConnectTask
 ******************************************************************************/

void	CNOVConnectTask::INOVConnectTask(
	CNHotSpotView *aNHotSpotFixed, 
	CNHotSpotView *aNHotSpotDragged,
	CNBrainPan *aNBrainPan)
{
	// The call to ITask doesn't need an "aNameIndex" argument, this task will not be 
	// directly undoable by the user from the menu.
	CTask::ITask(-1);
	
	itsNBrainPan = aNBrainPan;
	
	// Initialize the NHotSpots pointers.
	itsNHotSpotFixed = aNHotSpotFixed;
	itsNHotSpotDragged = aNHotSpotDragged;
	
	// Initialize the kind of pair of NOView's we are connecting.
	connectKind = noPair;
	
	// Initialize the iVar object pointers.
	fixedTopEncl = NULL;
	draggedTopEncl = NULL;
	fixedNeuron = NULL;
	draggedNeuron = NULL;
	
	// Do we have a pair of NHotSpots to connect?
	if ((itsNHotSpotFixed) && (itsNHotSpotDragged)) {

		// Initialize the top-level-NOView-enclosures of the two NHotSpots passed in -- the
		// NObjectViews in their visual hierarchies that are directly enclosed by a NBrainPan.
		fixedTopEncl = itsNHotSpotFixed->GetMyTopNOVEnclosure();
		draggedTopEncl = itsNHotSpotDragged->GetMyTopNOVEnclosure();
		
		// If either "fixedTopEncl" or "draggedTopEncl" are NNetworkViews, then we need to
		// also know the NNeuronView[s] associated with the hotpoints.
		// These iVar's are used in "ConnectObjNet" and "ConnectNetObj."
		if (member(fixedTopEncl, CNNetworkView))
			fixedNeuron = itsNHotSpotFixed->GetMyNeuronEnclosure();
		if (member(draggedTopEncl, CNNetworkView))
			draggedNeuron = itsNHotSpotDragged->GetMyNeuronEnclosure();
		
		// The type of top-level-NOView-enclosures of the two NHotSpots determines the 
		// procedure for making the connection, so we are initializing an iVar that represents
		// the type of pair we are connecting.
		if (member(fixedTopEncl, CNNetworkView)) {
			if (member(draggedTopEncl, CNNetworkView))
				connectKind = netNet;
			else if (member(draggedTopEncl, CNNeuronView))
				connectKind = netNeu;
			else
				connectKind = netObj;
		}
		else if (member(fixedTopEncl, CNNeuronView)) {
			if (member(draggedTopEncl, CNNetworkView))
				connectKind = neuNet;
			else if (member(draggedTopEncl, CNNeuronView))
				connectKind = neuNeu;
			else
				connectKind = neuObj;
		}
		else {
			if (member(draggedTopEncl, CNNetworkView))
				connectKind = objNet;
			else if (member(draggedTopEncl, CNNeuronView)) 
				connectKind = objNeu;
			else
				connectKind = objObj;
		}
	}
}


/******************************************************************************
 GetDraggedHSpot
 
******************************************************************************/

CNHotSpotView *CNOVConnectTask::GetDraggedHSpot(void)
{
	return (itsNHotSpotDragged);
}


/******************************************************************************
 GetFixedHSpot
 
******************************************************************************/

CNHotSpotView *CNOVConnectTask::GetFixedHSpot(void)
{
	return (itsNHotSpotFixed);
}


/******************************************************************************
 ConnectModels
 
 	Make the connection in the model.
******************************************************************************/

Boolean CNOVConnectTask::ConnectModels(CNModel **aFixedModel, CNModel **aDraggedModel)
{
	CNObjectView	*aNOViewFixed = NULL;
	CNObjectView	*aNOViewDragged = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;
		
	// Get working access to the dragged and fixed CNObjectViews.
	aNOViewFixed = (CNObjectView *) itsNHotSpotFixed->itsEnclosure;
	aNOViewDragged = (CNObjectView *) itsNHotSpotDragged->itsEnclosure;
	
	if (aNOViewFixed && aNOViewDragged) {
		
		aNMElemFixed = aNOViewFixed->myNModelElem;
		aNMElemDragged = aNOViewDragged->myNModelElem;
		
		// Assign returned parameters.
		*aFixedModel = aNMElemFixed;
		*aDraggedModel = aNMElemDragged;
		
		// Do the actual connection.
		if (aNMElemFixed && aNMElemDragged) {
			return (CNModelElem::connect (aNMElemDragged, aNMElemFixed) != 0);
		}
		else
			return FALSE;
	}
	else 
		return FALSE;
}


/******************************************************************************
 ConnectNetNet
 
	Transfer the contents (the NNeuronViews) from one net into the other, and
	Dispose of the empty net.
******************************************************************************/

void CNOVConnectTask::ConnectNetNet(void)
{
	CNOViewOwner	*aNOVOwner = NULL;
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNetworkModel	*aSecondNNetworkModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;
	
	// Do the model connection.
	ConnectModels(&aNMElemFixed, &aNMElemDragged);
	aNNetworkModel = (CNNetworkModel *) fixedTopEncl->GetModel();
	aSecondNNetworkModel = (CNNetworkModel *) draggedTopEncl->GetModel();
	aNNetworkModel->fold_in_network(aSecondNNetworkModel);
	aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 

	// Initialize an object for refering to all the subview objects of the dragged NNetworkView.
	aNOVOwner = new (CNOViewOwner);
	aNOVOwner->INOViewOwner(gNLimbo);
	aNOVOwner->InitToAllSubviews(draggedTopEncl);
	
	// Transfer all the subview NNeuronView objects from the "draggedTopEncl" NNetworkView
	// to the "fixedTopEncl" NNetworkView.
	aNOVOwner->TravelAllTo(fixedTopEncl);
	
	// Now Dispose of "draggedTopEncl."
	draggedTopEncl->Dispose();
	draggedTopEncl = NULL;
}


/******************************************************************************
 ConnectNetNeu
 
******************************************************************************/

void CNOVConnectTask::ConnectNetNeu(void)
{
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;

	((CNOVCollection *) fixedTopEncl)->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);

	// Do the model connection.
	ConnectModels(&aNMElemFixed, &aNMElemDragged);
	aNNeuronModel = (CNNeuronModel *) draggedTopEncl->GetModel();
	aNNetworkModel = (CNNetworkModel *) fixedTopEncl->GetModel();
	aNNetworkModel->add_neuron(aNNeuronModel);
	aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
}


/******************************************************************************
 ConnectNetObj
 
******************************************************************************/

void CNOVConnectTask::ConnectNetObj(void)
{
	CNNeuronView	*aNNeuronView = NULL;
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;

	// A Neuron can have only one soma, so if draggedTopEncl is a CNSomaView then make
	// it into a new CNNeuronView, and put it into the NNetworkView "fixedTopEncl."
	if ((member(draggedTopEncl, CNSomaView)) && 
			(((CNNeuronView *) fixedNeuron)->ContainsSoma())) {
		
		// The fixed Soma becomes the only subview of a new NNeuronView.
		aNNeuronView = new(CNNeuronView);
		aNNeuronView->INNeuronView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		aNNeuronView->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);
		
		((CNOVCollection *) fixedTopEncl)->ObtainTravelerForCollection(((CNObjectView **) &aNNeuronView), 
				&itsNHotSpotFixed);
		
		// Maintain the CNHotSpot pointer to the dragged HS.
		//itsNHotSpotDragged = (aNNeuronView->GetSomaView())->GetHotSpotView();
		
		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) aNNeuronView->GetModel();
		aNNeuronModel->add_model(aNMElemDragged);
		aNNetworkModel = (CNNetworkModel *) fixedTopEncl->GetModel();
		aNNetworkModel->add_neuron(aNNeuronModel);
		aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
	}
	
	else {
		fixedNeuron->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);	

		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) fixedNeuron->GetModel();
		aNNeuronModel->add_model(aNMElemDragged);
	}
}


/******************************************************************************
 ConnectNeuNet
 
******************************************************************************/

void CNOVConnectTask::ConnectNeuNet(void)
{
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;

	((CNOVCollection *) draggedTopEncl)->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);

	// Do the model connection.
	ConnectModels(&aNMElemFixed, &aNMElemDragged);
	aNNeuronModel = (CNNeuronModel *) fixedTopEncl->GetModel();
	aNNetworkModel = (CNNetworkModel *) draggedTopEncl->GetModel();
	aNNetworkModel->add_neuron(aNNeuronModel);
	aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
}


/******************************************************************************
 ConnectNeuNeu
 
******************************************************************************/

void CNOVConnectTask::ConnectNeuNeu(void)
{
	CNNetworkView	*aNNetworkView = NULL;
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNNeuronModel	*aSecondNNeuronModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;

	// Both of these NNeuronViews are enclosed by a new NNetworkView.
	aNNetworkView = new(CNNetworkView);
	aNNetworkView->INNetworkView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
			sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
	
	aNNetworkView->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);
	aNNetworkView->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);

	// Do the model connection.
	ConnectModels(&aNMElemFixed, &aNMElemDragged);
	aNNeuronModel = (CNNeuronModel *) fixedTopEncl->GetModel();
	aSecondNNeuronModel = (CNNeuronModel *) draggedTopEncl->GetModel();
	aNNetworkModel = (CNNetworkModel *) aNNetworkView->GetModel();
	aNNetworkModel->add_neuron(aNNeuronModel);
	aNNetworkModel->add_neuron(aSecondNNeuronModel);
	aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
}


/******************************************************************************
 ConnectNeuObj
 
******************************************************************************/

void CNOVConnectTask::ConnectNeuObj(void)
{
	CNNeuronView	*aNNeuronView = NULL;
	CNNetworkView	*aNNetworkView = NULL;
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNNeuronModel	*aSecondNNeuronModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;

	// A Neuron can have only one soma, so if aNObjectView is a CNSomaView then make
	// aNObjectView into a new CNNeuronView, and put it and aNNeuronView into a new
	// CNNetworkView.
	if ((((CNNeuronView *) fixedTopEncl)->ContainsSoma()) && 
			(member(draggedTopEncl, CNSomaView))) {
		
		// The fixed Soma becomes the only subview of a new NNeuronView.
		aNNeuronView = new(CNNeuronView);
		aNNeuronView->INNeuronView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		aNNeuronView->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);
		
		// The new NNeuronView and the old NNeuronView, fixedTopEncl, are enclosed by 
		// a new NNetworkView.
		aNNetworkView = new(CNNetworkView);
		aNNetworkView->INNetworkView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		
		aNNetworkView->ObtainTravelerForCollection(((CNObjectView **) &aNNeuronView),  
					&itsNHotSpotDragged);
		aNNetworkView->ObtainTravelerForCollection(((CNObjectView **) &fixedTopEncl), 
					 &itsNHotSpotFixed);

		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) aNNeuronView->GetModel();
		aNNeuronModel->add_model(aNMElemDragged);
		aSecondNNeuronModel = (CNNeuronModel *) fixedTopEncl->GetModel();
		aNNetworkModel = (CNNetworkModel *) aNNetworkView->GetModel();
		aNNetworkModel->add_neuron(aNNeuronModel);
		aNNetworkModel->add_neuron(aSecondNNeuronModel);
		aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
	}
	
	else 
		((CNOVCollection *) fixedTopEncl)->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);
		
		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) fixedTopEncl->GetModel();
		aNNeuronModel->add_model(aNMElemDragged);
}


/******************************************************************************
 ConnectObjNet
 
	Note: we are using the iVar "draggedNeuron" here to make the connection, 
	because we can't directly connect an NObjectView to a NNetworkView.
	NNetworkView's are composed exclusively by NNeuronViews.
******************************************************************************/

void CNOVConnectTask::ConnectObjNet(void)
{
	CNNeuronView	*aNNeuronView = NULL;
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;

	// A Neuron can have only one soma, so if fixedTopEncl is a CNSomaView then make
	// it into a new CNNeuronView, and put it into the NNetworkView "draggedTopEncl."
	if ((member(fixedTopEncl, CNSomaView)) && 
			(((CNNeuronView *) draggedNeuron)->ContainsSoma())) {
		
		// The fixed Soma becomes the only subview of a new NNeuronView.
		aNNeuronView = new(CNNeuronView);
		aNNeuronView->INNeuronView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		aNNeuronView->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);
		
		((CNOVCollection *) draggedTopEncl)->ObtainTravelerForCollection(((CNObjectView **) &aNNeuronView), 
					&itsNHotSpotDragged);

		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) aNNeuronView->GetModel();
		aNNeuronModel->add_model(aNMElemFixed);
		aNNetworkModel = (CNNetworkModel *) draggedTopEncl->GetModel();
		aNNetworkModel->add_neuron(aNNeuronModel);
		aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
	}
	
	else {
		draggedNeuron->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);

		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) draggedNeuron->GetModel();
		aNNeuronModel->add_model(aNMElemFixed);
	}
}


/******************************************************************************
 ConnectObjNeu
 
******************************************************************************/

void CNOVConnectTask::ConnectObjNeu(void)
{
	CNNeuronView	*aNNeuronView = NULL;
	CNNetworkView	*aNNetworkView = NULL;
	CNNetworkModel	*aNNetworkModel = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNNeuronModel	*aSecondNNeuronModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;

	// A Neuron can have only one soma, so if aNObjectView is a CNSomaView then make
	// aNObjectView into a new CNNeuronView, and put it and aNNeuronView into a new
	// CNNetworkView.
	if ((member(fixedTopEncl, CNSomaView)) && 
			(((CNNeuronView *) draggedTopEncl)->ContainsSoma())) {
		
		// The fixed Soma becomes the only subview of a new NNeuronView.
		aNNeuronView = new(CNNeuronView);
		aNNeuronView->INNeuronView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		aNNeuronView->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);
		
		// The new NNeuronView and the old NNeuronView, draggedTopEncl, are enclosed by 
		// a new NNetworkView.
		aNNetworkView = new(CNNetworkView);
		aNNetworkView->INNetworkView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		
		aNNetworkView->ObtainTravelerForCollection(((CNObjectView **) &aNNeuronView), 
					&itsNHotSpotFixed);
		aNNetworkView->ObtainTravelerForCollection(((CNObjectView **) &draggedTopEncl), 
					&itsNHotSpotDragged);

		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) aNNeuronView->GetModel();
		aNNeuronModel->add_model(aNMElemFixed);
		aSecondNNeuronModel = (CNNeuronModel *) draggedTopEncl->GetModel();
		aNNetworkModel = (CNNetworkModel *) aNNetworkView->GetModel();
		aNNetworkModel->add_neuron(aNNeuronModel);
		aNNetworkModel->add_neuron(aSecondNNeuronModel);
		aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
	}
	
	else {
		((CNOVCollection *) draggedTopEncl)->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);
		
		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) draggedTopEncl->GetModel();
		aNNeuronModel->add_model(aNMElemFixed);
	}
}


/******************************************************************************
 ConnectObjObj
 
******************************************************************************/

void CNOVConnectTask::ConnectObjObj(void)
{
	CNNeuronView	*aNNeuronView = NULL;
	CNNeuronView	*aSecondNNeuronView = NULL;
	CNNetworkView	*aNNetworkView = NULL;
	CNNeuronModel	*aNNeuronModel = NULL;
	CNNeuronModel	*aSecondNNeuronModel = NULL;
	CNNetworkModel	*aNNetworkModel = NULL;
	CNModel			*aNMElemFixed = NULL;
	CNModel			*aNMElemDragged = NULL;
	
	// A Neuron can have only one soma, so if both firstObject and secondObject are
	// CNSomaView then make them two new CNNeurons and put them into a new CNNetwork.
	if ((member(fixedTopEncl, CNSomaView)) && (member(draggedTopEncl, CNSomaView))) {
		
		// The fixed Soma becomes the only subview of a new NNeuronView.
		aNNeuronView = new(CNNeuronView);
		aNNeuronView->INNeuronView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		aNNeuronView->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);
		
		// The dragged Soma becomes the only subview of another new NNeuronView.
		aSecondNNeuronView = new(CNNeuronView);
		aSecondNNeuronView->INNeuronView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		aSecondNNeuronView->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);
		
		// Both of these new NNeuronViews are enclosed by a new NNetworkView.
		aNNetworkView = new(CNNetworkView);
		aNNetworkView->INNetworkView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		
		// Do the visual connection.
		aNNetworkView->ObtainTravelerForCollection(((CNObjectView **) &aNNeuronView), 
					&itsNHotSpotFixed);
		aNNetworkView->ObtainTravelerForCollection(((CNObjectView **) &aSecondNNeuronView), 
					&itsNHotSpotDragged);

		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) aNNeuronView->GetModel();
		aNNeuronModel->add_model(aNMElemFixed);
		aSecondNNeuronModel = (CNNeuronModel *) aSecondNNeuronView->GetModel();
		aSecondNNeuronModel->add_model(aNMElemDragged);
		aNNetworkModel = (CNNetworkModel *) aNNetworkView->GetModel();
		aNNetworkModel->add_neuron(aNNeuronModel);
		aNNetworkModel->add_neuron(aSecondNNeuronModel);
		aNNetworkModel->define_connection (aNMElemFixed, aNMElemDragged); 
	}
	
	else {
		aNNeuronView = new(CNNeuronView);
		aNNeuronView->INNeuronView(itsNBrainPan, itsNBrainPan, 0, 0, 0, 0, 
				sizFIXEDSTICKY, sizFIXEDSTICKY, 0, 0, NULL);
		
		// Do the visual connection.
		aNNeuronView->ObtainTravelerForCollection(&fixedTopEncl, &itsNHotSpotFixed);
		aNNeuronView->ObtainTravelerForCollection(&draggedTopEncl, &itsNHotSpotDragged);
		
		// Do the model connection.
		ConnectModels(&aNMElemFixed, &aNMElemDragged);
		aNNeuronModel = (CNNeuronModel *) aNNeuronView->GetModel();
		aNNeuronModel->add_model(aNMElemFixed);
		aNNeuronModel->add_model(aNMElemDragged);
	}
}


/******************************************************************************
 Do		Perform a NOVConnectTask
 
	We use an urgent chore to do the actual work of connecting, because the connection
	can't be done within the Drag Task of one of the object being connected.
 ******************************************************************************/

void	CNOVConnectTask::Do()
{
	CNOVConnectChore	*aNOVConnectChore = NULL;
		
	aNOVConnectChore = new(CNOVConnectChore);
	aNOVConnectChore->INOVConnectChore(this);
	gApplication->AssignUrgentChore(aNOVConnectChore);
}


/******************************************************************************
 DoForReal		Perform a NOVConnectTask

	// Note: The "Connect" methods would be a good application of method overloading, 
	// if Think's TCL supported method overloading (which it doesn't).
 ******************************************************************************/

void	CNOVConnectTask::DoForReal()
{
	// If the model connection was successful, then do the visual connection.
	// Note: you could put this switch within an "if" statement to check for 
	// legal types of connections.
	switch (connectKind) {
		case netNet:
			ConnectNetNet();
			break;
			
		case netNeu:
			ConnectNetNeu();
			break;
			
		case netObj:
			ConnectNetObj();	
			break;
			
		case neuNet:
			ConnectNeuNet();
			break;
			
		case neuNeu:
			ConnectNeuNeu();
			break;
			
		case neuObj:
			ConnectNeuObj();
			break;
			
		case objNet:
			ConnectObjNet();
			break;
			
		case objNeu:
			ConnectObjNeu();
			break;
			
		case objObj:
			ConnectObjObj();
			break;
			
		case noPair:
			break;
			
		default:
			printf("Never heard of that connect kind, {CNOVConnectTask::INOVConnectTask}\n");
			exit(1);
			break;
	}
}


/******************************************************************************
 Undo	Undo a task.
	
 ******************************************************************************/

void	CNOVConnectTask::Undo()
{
	switch (connectKind) {
		case netNet:
			break;
			
		case netNeu:
			break;
			
		case netObj:	
			break;
			
		case neuNet:
			break;
			
		case neuNeu:
			break;
			
		case neuObj:
			break;
			
		case objNet:
			break;
			
		case objNeu:
			break;
			
		case objObj:
			break;
			
		case noPair:
			break;
			
		default:
			printf("Never heard of that connect kind, {CNOVConnectTask::INOVConnectTask}\n");
			exit(1);
			break;
	}	
	
	inherited::Undo();
}


/******************************************************************************
 Redo	Redo a task which has been undone.
 
 ******************************************************************************/

void	CNOVConnectTask::Redo()
{
	switch (connectKind) {
		case netNet:
			break;
			
		case netNeu:
			break;
			
		case netObj:	
			break;
			
		case neuNet:
			break;
			
		case neuNeu:
			break;
			
		case neuObj:
			break;
			
		case objNet:
			break;
			
		case objNeu:
			break;
			
		case objObj:
			break;
			
		case noPair:
			break;
			
		default:
			printf("Never heard of that connect kind, {CNOVConnectTask::INOVConnectTask}\n");
			exit(1);
			break;
	}	

	inherited::Undo();
}


/******************************************************************************
 Dispose     "Dispose of a CNOVConnectTask object"
 
 ******************************************************************************/

void	CNOVConnectTask::Dispose()
{
	inherited::Dispose();
}


