/******************************************************************************
 CNOViewOwner.c

	Instances of this class can "own" an arbitrary group of instances of CNObjectView.
		
	SUPERCLASS = CBureaucrat
	
	Copyright  1993 by Daniel McFarlane. All rights reserved.
	
	Note: The ownership of NObjectViews is maintained with CCollaboration-Dependencies.
	So the list of dependent objects is kept current in "itsProviders".
	Whenever we need to do work on the list of owned objects, we first make
	a working copy of itsProviders, this is called myNOVPossessions.
	myNOVPossessions needs to be local and re-created each time some new work is begun, 
	because itsProviders may have been changed by some previous work.

 ******************************************************************************/

#include "CNOViewOwner.h"
#include "CList.h"
#include "CNBrainPan.h"
#include "CNObjectView.h"
#include "CPane.h"
#include <oops.h> 
#include <stdio.h>
#include "CView.h"



/******************************************************************************
 INOViewOwner
 
 	Initialize a NOViewOwner. 
******************************************************************************/

void CNOViewOwner::INOViewOwner(CBureaucrat *aSupervisor)
{
	CBureaucrat::IBureaucrat( aSupervisor);
	SetLongRect(&itsBoundingRect, 0, 0, 0, 0);
}


/******************************************************************************
 InitToAllSubviews
 
 	Initialize this to posses all the subviews of a given NObjectView. 
******************************************************************************/

void CNOViewOwner::InitToAllSubviews(CNObjectView *aNObjectView)
{
	CList			*allTheSubviews = NULL;
	long			numSubviews = 0;
	long			i;
	CNObjectView	*nextSubview = NULL;
	
	// "This" must be empty before I can initialize it.
	SetEmptyAll();
		
	// Acquire each of the subview objects of "aNObjectView".
	allTheSubviews = aNObjectView->GetSubviews();
	if (allTheSubviews) {
		
		numSubviews = allTheSubviews->GetNumItems();
		for (i = 1; i <= numSubviews; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) allTheSubviews->NthItem(i);
			
			// Acquire "nextSubview"
			AcquireNOView(nextSubview);
		}
		
		// HouseKeeping
		allTheSubviews->Dispose();
	}
}


/******************************************************************************
 TravelAllTo
 
 	Travel all my possessions from their current graphical environment (their 
 	itsEnclosure) into a new graphical environment.
******************************************************************************/

		// a "DoForEach1" function for telling myNOVPossessions to travel to a new
		// graphical environment.
		static void CNOViewOwner::SendObtainTraveler(CObject *member, long infoAddr)
		{
			CPane 			*aPane = (CPane *) infoAddr;
			CNObjectView 	*aNOView = (CNObjectView *) member;
			
			if (member(aPane, CNObjectView)) 
				((CNObjectView *) aPane)->ObtainTraveler(&aNOView);

			if (member(aPane, CNBrainPan)) 
				((CNBrainPan *) aPane)->ObtainTraveler(&aNOView);
		}

void CNOViewOwner::TravelAllTo(CPane *aPane)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions
	
	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();
		
		// Do transplation work.
		// This is not a real "trasplantation"; it's more like the movie "The Invasion of the
		// Body Snatchers" where the objects are actually replaced by exact copies placed into
		// the new environments.  The replacements are automatically added to "this".
		myNOVPossessions->DoForEach1((EachFunc1) CNOViewOwner::SendObtainTraveler, 
				(long) aPane);
	}
}


/******************************************************************************
 GetPossessions
 
******************************************************************************/

CList	*CNOViewOwner::GetPossessions(void)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		// Return a copy of my possessions..
		return ((CList *) myNOVPossessions->Copy());

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
	else
		return (NULL);
}


/******************************************************************************
 CopyNOVOwnerActualFor
 
 		Create a new copy of this, complete with actual copies of all the object
 		and not just references to the same objects like CNOViewOwner::CopyNOVOwnerFor.
******************************************************************************/

CNOViewOwner *CNOViewOwner::CopyNOVOwnerActualFor(CNBrainPan *newNBrainPan)
{
	CNOViewOwner	*theCopy = NULL;
	long			numMembers = 0;
	long			i;
	CNObjectView	*nextNOView = NULL;
	CNObjectView	*copyTempNOView = NULL;
	CList			*myNOVPossessions = NULL;		// working list of this' possessions
	
	// Create a new CNOViewOwner to be theCopy.
	theCopy = new(CNOViewOwner);
	theCopy->INOViewOwner(newNBrainPan);

	// Make actual copies of the NOViews this ownes, and assign their ownership to theCopy.
	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		numMembers = myNOVPossessions->GetNumItems();
		for (i = 1; i <= numMembers; ++i) {
		
			// Make an actual copy of this object.
			nextNOView = (CNObjectView *) myNOVPossessions->NthItem(i);
			copyTempNOView = nextNOView->CopyNOViewInto(newNBrainPan, newNBrainPan);
			theCopy->AcquireNOView(copyTempNOView);
		}

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}

	return theCopy;
}


/******************************************************************************
 CopyNOVOwnerFor
 
 		Create a new copy of this, that points to the same NObjectViews.
******************************************************************************/

CNOViewOwner *CNOViewOwner::CopyNOVOwnerFor(CBureaucrat *aSupervisor)
{
	CNOViewOwner		*theCopy = NULL;
	long				numMembers = 0;
	long				i;
	CNObjectView	*nextNOView = NULL;
	CList		*myNOVPossessions = NULL;		// working list of this' possessions
	
	// Create a new CNOViewOwner to be theCopy.
	theCopy = new(CNOViewOwner);
	theCopy->INOViewOwner(aSupervisor);
	
	// Copy the ownership information from "this" to theCopy.
	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		numMembers = myNOVPossessions->GetNumItems();
		for (i = 1; i <= numMembers; ++i) {
		
			// Add a dependency between theCopy and each of this' owned NOViews.
			nextNOView = (CNObjectView *) myNOVPossessions->NthItem(i);
			theCopy->AcquireNOView(nextNOView);
		}

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}

	return theCopy;
}


/******************************************************************************
 AcquireNOView
 
 	Setup a dependency to a newly acquired object.
******************************************************************************/

void CNOViewOwner::AcquireNOView(CNObjectView *aNOView)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	// this can only acquire instances of CNObjectView.
	if (member(aNOView, CNObjectView)) {
			
		// If myNOVPossessions already contains aNOView then don't add it again.
		if (itsProviders) {
	
			// create a temporary working copy of my possessions.
			myNOVPossessions = (CList *) itsProviders->Copy();
			
			if (myNOVPossessions->Includes(aNOView))
				return;

			// Do house-keeping.
			myNOVPossessions->Dispose();
		}
		
		// Initialize a dependancy with the new object -- "acquire" it.
		DependUpon(aNOView);
	}
}


/******************************************************************************
 RelinquishNOView
 
 	Cancel a dependency to an acquired object.
******************************************************************************/

void CNOViewOwner::RelinquishNOView( CNObjectView *aNOView)
{
	// this can only relinquished instances of CNObjectView.
	if (member(aNOView, CNObjectView))
		CancelDependency(aNOView);
}


/******************************************************************************
 Union
 
 	This method allows for the acquisition of an entire list (NOViewOwner) of 
 	new members, all at once.
******************************************************************************/

		// a "DoForEach1" function for for acquiring a new myNOVPossessions member.
		static void CNOViewOwner::UnionEach(CObject *member, long infoAddr)
		{
			CNOViewOwner *aNOVOwner = (CNOViewOwner *) infoAddr;

			aNOVOwner->AcquireNOView((CNObjectView *) member);
		}

void CNOViewOwner::Union(CNOViewOwner *aNOVOwner)
{
	CList	*tempList = NULL;

	if (aNOVOwner) {

		// create a temporary working copy of "aNOVOwner".
		tempList = aNOVOwner->GetPossessions();

		tempList->DoForEach1((EachFunc1) CNOViewOwner::UnionEach, 
				(long) this);

		// Do house-keeping.
		tempList->Dispose();
	}
}


/******************************************************************************
 Not
 
 	This method allows for the relinquish-ment of an entire list (NOViewOwner) of 
 	members, all at once.
******************************************************************************/

		// a "DoForEach1" function for for relinquishing a myNOVPossessions member.
		static void CNOViewOwner::NotEach(CObject *member, long infoAddr)
		{
			CNOViewOwner *aNOVOwner = (CNOViewOwner *) infoAddr;

			aNOVOwner->RelinquishNOView((CNObjectView *) member);
		}

void CNOViewOwner::Not(CNOViewOwner *aNOVOwner)
{
	CList	*tempList = NULL;

	if (aNOVOwner) {

		// create a temporary working copy of "aNOVOwner".
		tempList = aNOVOwner->GetPossessions();

		tempList->DoForEach1((EachFunc1) CNOViewOwner::NotEach, 
				(long) this);

		// Do house-keeping.
		tempList->Dispose();
	}
}


/******************************************************************************
 IsEmpty
 
******************************************************************************/

Boolean CNOViewOwner::IsEmpty(void)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		return (myNOVPossessions->IsEmpty());

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
	else
		return TRUE;
}


/******************************************************************************
 Dispose
 
 	Dispose of the NOViewOwner, but not what it ownes.
******************************************************************************/

void CNOViewOwner::Dispose( void)
{
	inherited::Dispose();
}


/******************************************************************************
 DisposeAll
 
 	DisposeAll of the NOViewOwner and all its NOViews.
******************************************************************************/

void CNOViewOwner::DisposeAll( void)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DisposeAll();
	}
		
	inherited::Dispose();
}


/******************************************************************************
 ToggleMoveToGBookmarkAll
 
******************************************************************************/

		// a "DoForEach" function for telling myNOVPossessions members to ToggleMoveToGBookmark.
		static void CNOViewOwner::SendToggleMoveToGBookmark(CObject *member)
		{
			((CNObjectView *) member)->ToggleMoveToGBookmark();
		}

void CNOViewOwner::ToggleMoveToGBookmarkAll(void)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DoForEach((EachFunc) SendToggleMoveToGBookmark);

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}


/******************************************************************************
 SetFSelectedAll
 
******************************************************************************/

		// a "DoForEach1" function for telling myNOVPossessions their fSelected status.
		static void CNOViewOwner::SetMemberFSelectionStatus(CObject *member, long status)
		{
			((CNObjectView *) member)->SetFSelected((Boolean) status);
		}

void CNOViewOwner::SetFSelectedAll(Boolean fStatus)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DoForEach1((EachFunc1) CNOViewOwner::SetMemberFSelectionStatus, 
				(long) fStatus);

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}


/******************************************************************************
 SetFBeingDraggedAll
 
******************************************************************************/

		// a "DoForEach1" function for telling myNOVPossessions their fBeingDragged status.
		static void CNOViewOwner::SetMemberFBeingDraggedStatus(CObject *member, long status)
		{
			((CNObjectView *) member)->SetFBeingDragged((Boolean) status);
		}

void CNOViewOwner::SetFBeingDraggedAll(Boolean fStatus)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DoForEach1((EachFunc1) CNOViewOwner::SetMemberFBeingDraggedStatus, 
				(long) fStatus);

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}


/******************************************************************************
 SetFDrawHotSpotsAll
 
******************************************************************************/

		// a "DoForEach1" function for telling myNOVPossessions their fDrawHotSpots status.
		static void CNOViewOwner::SetMemberFDrawHotSpots(CObject *member, long status)
		{
			((CNObjectView *) member)->SetFDrawHotSpot((Boolean) status);
		}

void CNOViewOwner::SetFDrawHotSpotsAll(Boolean fStatus)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DoForEach1((EachFunc1) CNOViewOwner::SetMemberFDrawHotSpots, 
				(long) fStatus);

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}


/******************************************************************************
 ProviderChanged
 
 	Receive notification that a provider changed.
 	This is where we get automatic housekeeping on all NOViewOwner::myNOVPossessions
 	lists, because when an object changes it sends a BroadcastChange message to 
 	all the instances of NOViewOwner that refer to it.  This insures that all 
 	the NOViewOwner lists are automatically maintained and always up-to-date.
******************************************************************************/

void CNOViewOwner::ProviderChanged( CCollaborator *aProvider, long reason, void* info)
{
	switch (reason) {
			
		case NOBJECTVIEW_REPLACING_ANOTHER:
			// There is no call to RelinquishNOView here for the original NOView;
			// the original is ->Disposed of as part of the transplantation.
			AcquireNOView((CNObjectView *) info);
 			break;
	}

	inherited::ProviderChanged( aProvider, reason, info);
}


/******************************************************************************
 MoveByAll
 
******************************************************************************/

		// a "DoForEach1" function for telling myNOVPossessions their fBeingDragged infoAddr.
		static void CNOViewOwner::SendMoveBy(CObject *member, long infoAddr)
		{
			LongPt *delta = (LongPt *) infoAddr;

			((CNObjectView *) member)->MoveBy(delta);
		}

void CNOViewOwner::MoveByAll(LongPt *delta)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DoForEach1((EachFunc1) CNOViewOwner::SendMoveBy, 
				(long) delta);

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}


/******************************************************************************
 PrintAll
 
******************************************************************************/

void CNOViewOwner::PrintAll(void)
{
	int				i;
	int				numMembers = 0;
	CNObjectView	*nextNOView = NULL;
	char			*tempEnclName;
	char			*tempChar;
	unsigned long 	tempAddr;
	CList		*myNOVPossessions = NULL;		// working list of this' possessions
	
	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		numMembers = myNOVPossessions->GetNumItems();
		for (i = 1; i <= numMembers; ++i) {
		
			nextNOView = (CNObjectView *) myNOVPossessions->NthItem((long) i);
			
			tempEnclName = class_name(nextNOView->itsEnclosure);
			tempChar = class_name(nextNOView);
			tempAddr = ((unsigned long) nextNOView);
			printf("%d\t%lX\t%s\t%s\n", i, tempAddr, tempChar, tempEnclName);
		}
		printf("\n");

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}


/******************************************************************************
 SetEmptyAll
 
 	Relinquish ownership of all NOViews.
******************************************************************************/

		// a "DoForEach1" function for for Relinquishing myNOVPossessions members.
		static void CNOViewOwner::SetEmptyEach(CObject *member, long infoAddr)
		{
			CNOViewOwner *aNOVOwner = (CNOViewOwner *) infoAddr;

			aNOVOwner->RelinquishNOView((CNObjectView *) member);
		}

void CNOViewOwner::SetEmptyAll(void)
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DoForEach1((EachFunc1) CNOViewOwner::SetEmptyEach, 
				(long) this);

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}


/******************************************************************************
 GetBoundingRect
 
		Return a copy of itsBoundingRect.
******************************************************************************/

void CNOViewOwner::GetBoundingRect(LongRect *myBoundingRect) 
{
	long		i;
	LongRect	newBoundingRect;
	CNObjectView *nextMember = NULL;
	long		numMembers;
	LongRect	memberBoundingRect;
	CList		*myNOVPossessions = NULL;		// working list of this' possessions
	
	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();
	
		// Initialize the newBoundingRect.
		SetLongRect(&newBoundingRect, 0, 0, 0, 0);
		
		// Compose the bounding Rect of all the list's members.
		numMembers = myNOVPossessions->GetNumItems();
		
		if (numMembers <= 0) ;
		
		else if (numMembers >= 1) {
		
			// Unravel the list one step to initialize the newBoundingRect.
			nextMember = (CNObjectView *) myNOVPossessions->NthItem(1);
			nextMember->GetFrame(&memberBoundingRect);
			nextMember->FrameToEnclR(&memberBoundingRect);
			newBoundingRect = memberBoundingRect;
			
			for (i = 2; i <= numMembers; ++i) {
			
				// Access the next item in the subview list.
				nextMember = (CNObjectView *) myNOVPossessions->NthItem(i);
				
				// Compute the bounding rect of this subview item in CNBrainPan coords.
				nextMember->GetFrame(&memberBoundingRect);
				nextMember->FrameToEnclR(&memberBoundingRect);
				
				// Union the bounds of this subview item to the accumulated total data bounds.
				UnionLongRect(&newBoundingRect, &memberBoundingRect, &newBoundingRect);
			}
		
			// This extra assignment is a requirement of "shadowing."
			itsBoundingRect = newBoundingRect;
		}

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
	else
		SetLongRect(&itsBoundingRect, 0, 0, 0, 0);

	// Prepare the returned argument.
	*myBoundingRect = itsBoundingRect;
}


/******************************************************************************
 InitGraphicalBookmarkAll
 
******************************************************************************/

		// a "DoForEach" function for telling myNOVPossessions members to InitGraphicalBookmark.
		static void CNOViewOwner::SendInitGraphicalBookmark(CObject *member)
		{
			((CNObjectView *) member)->InitGraphicalBookmark();
		}

void CNOViewOwner::InitGraphicalBookmarkAll(void) 
{
	CList		*myNOVPossessions = NULL;		// working list of this' possessions

	if (itsProviders) {

		// create a temporary working copy of my possessions.
		myNOVPossessions = (CList *) itsProviders->Copy();

		myNOVPossessions->DoForEach((EachFunc) SendInitGraphicalBookmark);

		// Do house-keeping.
		myNOVPossessions->Dispose();
	}
}













