/******************************************************************************
 CNOViewTask.c

							The NOViewTask Class
		
	SUPERCLASS = CMouseTask
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
	
	Instances of this class manage the interactive specification of instances of 
	CNObjectView in a NervousSystem document's CNBrainPan.  After an object is 
	interactively specified, "this" creates an actual instance of the object and 
	attaches it to the CNBrainPan; and then releases control back to 
	CNBrainPan::DoClick.
	
	I employ the following three statements as the motivations behind the structural
	design of this class (the statements are phrased from the voice of this class):
	1)  "If I start something, I want total control overe it until I say it's 
	finished; then I'll give-up the finished product.  If control were shared with
	CNBrainPan::DoClick for example, and I were interrupted in the middle of 
	my job, I could leave behind a half-created Frankenstein's monster."
	2)  "I gather all the information for what type of object is wanted, giving feedback
	to the user.  Then, I create the object in one stroke.  A construction site that
	stays open long and over several owners is a dangerous place; an accident waiting
	to happen."
	3)  This task knows everything about constructing CNObjectView.  "I make them
	and others use them.  Not, I get them sort of made and then others take over, and then
	maybe I'll get them back for a while ...." 

 ******************************************************************************/

#include "Global.h"
#include "CNOViewTask.h"
#include "LongCoordinates.h"
#include "LongQD.h"
#include "NervousSystemDoc.h"
#include "CNBrainPan.h"
#include "CNObjectView.h"
#include "CApplication.h"
#include "CSwitchboard.h"


extern CNervousSystemDoc *gNLimbo;


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 INOViewTask

		Initialize a NOViewTask object
 ******************************************************************************/

void	CNOViewTask::INOViewTask(
	short		aNameIndex,
	CNBrainPan *aNBrainPan)
{
	CMouseTask::IMouseTask(aNameIndex);
	
	itsNBrainPan = aNBrainPan;
	itsNewNOView = NULL;
	itsRegion = NewRgn();
}


/******************************************************************************
 KeepTracking

		Continuous mouse tracking while the mouse button is down. The
		current, previous, and starting mouse locations are passed by
		reference so the method may change their values. For example,
		the values may be changed to implement constrained drawing.
 ******************************************************************************/

void	CNOViewTask::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongRect		currRect;
	
	// Change the size of "itsNBrainPan's" "myDataExtent" so that it encompases 
	// currPt, and update "itsNBrainPan->bounds" accordingly.  This is required 
	// for the AutoScoll to work.
	itsNBrainPan->ChangeMyDataExtent(currPt);

	if (itsNBrainPan->AutoScroll(currPt) || !EqualLongPt(currPt, prevPt)) {
		itsNBrainPan->GetFrame(&currRect);
		PinInRect(&currRect, currPt);
		UseTool(currPt, prevPt);
	}
}


/******************************************************************************
 EndTracking

		Mouse down tracking has ended because the user has released the
		mouse button.
 ******************************************************************************/

void	CNOViewTask::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	// "Track" the last point. 
	KeepTracking(currPt, prevPt, startPt);
	
	// INV: "itsRegion", and this task's graphic model are now complete.
	
	// Create a new instance of CNObjectView and bind it to"itsNewNOView";
	CreateNewNOView();
	
	// Adjust my CNBrainPan's myDataExtent Rect to include the new addition.
	itsNBrainPan->CalcMyDataExtent();
	
	// Erase the task's "itsRegion" from the CNBrainPan --
	// from now on it will be Drawn from *itsNewNOView.
	itsNBrainPan->Prepare();
	EraseRgn(itsRegion);
}


/******************************************************************************
 Dispose     "Dispose of a NOViewTask object"
 
	We do not Dispose of "*itsNBrainPan", or "*itsNewNOView", they are only pointers
	to objects who's existance is independent of this task.  The scopes of these 
	local pointers will expire automatically when the task object is Disposed.
	However, if "*itsNewNOView" is Undone and not Redone, then Dispose it.
 ******************************************************************************/

void	CNOViewTask::Dispose()
{
	DisposeRgn(itsRegion);

	// If the new object that was created here, "itsNewNOView" was Undone, but 
	// not Redone, then it is in limbo and needs to be disposed of with this task.
	if (!(itsNewNOView->IsVisible())) 
		itsNewNOView->Dispose();
	
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 Undo {OVERRIDE}

	Hide "itsNewNOView" here in CNOViewTask limbo.
	Later, if the user chooses "Redo", then just bring it back from limbo; if not 
	then Dispose of it when the task is disposed of.
 ******************************************************************************/

void	CNOViewTask::Undo()
{	
	CNBrainPan	*aNBrainPan = NULL;
	
	aNBrainPan = gNLimbo->GetItsMainPane();
	aNBrainPan->ObtainTraveler(&itsNewNOView);
	
	// Do housekeeping on itsNBrainPan::myDataExtent.
	itsNBrainPan->CalcMyDataExtent();

	inherited::Undo();
}


/******************************************************************************
 Redo     {this is an override}

	When the user choose "Undo" we hid "itsNewNOView" here in CNOViewTask limbo.
	Now, we just bring it back from limbo.
 ******************************************************************************/

void	CNOViewTask::Redo()
{
	itsNBrainPan->ObtainTraveler(&itsNewNOView);
	
	// Do housekeeping on itsNBrainPan::myDataExtent.
	itsNBrainPan->CalcMyDataExtent();

	inherited::Undo();
}


/******************************************************************************
 BeginTracking

		Mouse down tracking is about to begin at the specified starting
		point. Subclasses should override this method if they wish to
		perform preparations before tracking or to alter the startPt
		(startPt could be altered to implement "snap to grid" for a
		drawing program).
 ******************************************************************************/

void	CNOViewTask::BeginTracking(
	LongPt		*startPt)
{
	SetEmptyRgn(itsRegion);
}


/******************************************************************************
 UseTool

		Mouse down tracking has ended because the user has released the
		mouse button.
 ******************************************************************************/

void	CNOViewTask::UseTool(
	LongPt		*currPt,
	LongPt		*prevPt)
{
}										/* Null Method						*/


/******************************************************************************
 CalcDrawRgn

		Calculate the draw region affected by a pen. What a pain. We have
		to tweak the region depending on where the current point is in
		relation to the previous point because the pen hangs below and
		to the right of its location.
 ******************************************************************************/

void	CNOViewTask::CalcDrawRgn(
	LongRect	*currRect,
	LongRect	*prevRect,
	RgnHandle	drawRgn)
{
	itsNBrainPan->Prepare();
	
	OpenRgn();
	
	if (currRect->left > prevRect->left) {
		if (currRect->top > prevRect->top) {
			LMoveTo(currRect->right, currRect->top);
			LLineTo(currRect->right, currRect->bottom);
			LLineTo(currRect->left, currRect->bottom);
			LLineTo(prevRect->left, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->top);
			LLineTo(prevRect->right, prevRect->top);
			LLineTo(currRect->right, currRect->top);
		
		} else {
			LMoveTo(currRect->left, currRect->top);
			LLineTo(currRect->right, currRect->top);
			LLineTo(currRect->right, currRect->bottom);
			LLineTo(prevRect->right, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->top);
			LLineTo(currRect->left, currRect->top);
		}
		
	} else {
		if (currRect->top > prevRect->top) {
			LMoveTo(currRect->right, currRect->bottom);
			LLineTo(currRect->left, currRect->bottom);
			LLineTo(currRect->left, currRect->top);
			LLineTo(prevRect->left, prevRect->top);
			LLineTo(prevRect->right, prevRect->top);
			LLineTo(prevRect->right, prevRect->bottom);
			LLineTo(currRect->right, currRect->bottom);
		
		} else {
			LMoveTo(currRect->left, currRect->bottom);
			LLineTo(currRect->left, currRect->top);
			LLineTo(currRect->right, currRect->top);
			LLineTo(prevRect->right, prevRect->top);
			LLineTo(prevRect->right, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->bottom);
			LLineTo(currRect->left, currRect->bottom);
		}
	}
	
	CloseRgn(drawRgn);
}


/******************************************************************************
 CalcPenRect

		Calculate the rectangle affected by a pen tool located at a
		specified point
 ******************************************************************************/

void	CNOViewTask::CalcPenRect(
	LongPt		*thePt,
	LongRect	*penRect)
{
	penRect->left = thePt->h;			/* Pen hangs below and to the right	*/
	penRect->top = thePt->v;			/*   of its coordinate location		*/
	penRect->right = thePt->h + penWidth;
	penRect->bottom = thePt->v + penWidth;
}


/******************************************************************************
 CreateNewNOView     {to be overridden}

 ******************************************************************************/

void	CNOViewTask::CreateNewNOView(void) {
}








