/******************************************************************************
 CNObjectView.c

	The NObjectView Class
	
	SUPERCLASS = CPane
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the GNU General Public 
	License Version 2, published by the Free Software Foundation, Inc.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
	GNU General Public License Version 2 for more details.
	You should have received a copy of the GNU General Public License
	along with this program (see the Programmer's Guide); if not, write to the 
	Free Software Foundation, Inc., 675 Massachusetts Ave., Cambridge, MA  02139, USA.
	
	This object has a list it maintains of all the objects is encloses both directly
	and indirectly through its subviews.  This list is called "itsNOVOwner".  
	This object also has and maintains another list called "itsHotSpotsNOVOwner", 
	which is similar to "itsNOVOwner", except it refers only to those enclosed objects
	(both direct and indirectly enclosed) that are instances of CNHotSpotView.
	"itsHotSpotsNOVOwner" and "itsNOVOwner" are maintained for efficiency reasons
	related to dragging and connecting objects.
		
 ******************************************************************************/

#include "Global.h"
#include "CPrinter.h"
#include "Constants.h"      
#include "CWindow.h"
#include "CList.h"
#include "LongQD.h"
#include "MainWindow.h"
#include "CNObjectView.h"
#include "CNOVDraggerTask.h"
#include "CNSomaView.h"
#include "CNSynapseView.h"
#include "CView.h"
#include "CNOViewOwner.h"
#include "CNHotSpotView.h"
#include "NervousSystemApp.h"
#include "NervousSystemDoc.h"
#include "CPaneBorder.h"
#include "CNModel.h"
#include "NervousSystemDoc.h"
#include "CBureaucrat.h"
#include <Quickdraw.h>
#include "CNOVCollection.h"
#include "CNNeuronView.h"
#include "CNBrainPan.h"
#include "CDirectorOwner.h"


// External global variables used here.
extern short gClicks;
extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern CApplication	*gApplication;
extern CNervousSystemDoc *gNLimbo;

	
/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 INObjectView

		Initialize a NObjectView object
 ******************************************************************************/

void CNObjectView::INObjectView(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin,
	long 			aVFrameOrigin)
{
	// Init FrameOrigin so that ::IPaneX's call to CalcFrame is ready with the info it needs.
	hFrameOrigin = aHFrameOrigin;
	vFrameOrigin = aVFrameOrigin;
	
	CPane::IPane(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);
	
	itsRegion = NewRgn();
	
	// Initialize itsNBrainPan.
	SetItsNBrainPan(GetMyNBrainPan());
	
	InitGraphicalBookmark();
	itsNOVOwner = NULL;
	itsHotSpotsNOVOwner = NULL;
	fSelected = FALSE;
	fBeingDragged = FALSE;
	fBeingCopied = FALSE;
	fDrawHotSpots = FALSE;
	fBeingDisposed = FALSE;
	myNModelElem = NULL;
	
	// Ensure that itsEnclosure fully encloses "this".
	if (member(anEnclosure, CNObjectView))
		((CNObjectView *) anEnclosure)->EncloseWith(this);
	else if (member(anEnclosure, CNBrainPan))
		((CNBrainPan *) anEnclosure)->EncloseWith(this);
	
	// The border is used for debugging, so the frame of this NObjectView will be visible.
	/*
	itsBorder = NULL;
	itsBorder = new (CPaneBorder);
	itsBorder->IPaneBorder(kBorderFrame);
	*/
}


/******************************************************************************
 GetIndexOfSubview

 ******************************************************************************/

int CNObjectView::GetIndexOfSubview(CNObjectView *aSubview)
{
	int			index = -1;
	
	// Get the index of aSubview in this' itsSubview list.
	if (itsSubviews) 
		index = itsSubviews->FindIndex(aSubview);

	// Return the value of index;
	return (index);
}


/******************************************************************************
 BuildListOfModelsOfSubviews

	This method returns a list of pointers to the models of the subviews of this.
 ******************************************************************************/

CList	*CNObjectView::BuildListOfModelsOfSubviews(void)
{
	long			i;
	CNObjectView 	*nextSubview = NULL;
	long			numSubviews;
	CList			*listOfModels = NULL;
	CNModel			*nextModel = NULL;
	
	// if itsSubviews, then build a list of pointers to their models.
	if (itsSubviews) {
		
		// Initialize the list.
		listOfModels = new(CList);
		listOfModels->IList();
		
		// Step throught the list of itsSubviews one at a time.
		numSubviews = itsSubviews->GetNumItems();
		for (i = 1; i <= numSubviews; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
		
			// Get a pointer to the model for this subview and append it to the list.
			nextModel = nextSubview->GetModel();
			listOfModels->Append(nextModel);
		}
	}
	
	return (listOfModels);
}


/******************************************************************************
 GetModel

 ******************************************************************************/

CNModel		*CNObjectView::GetModel(void)
{
	return (myNModelElem);
}


/******************************************************************************
 Draw

		Assumption: the area has already been EraseRect'ed, so we do not need to 
		call EraseRgn(itsRegion).
 ******************************************************************************/

void	CNObjectView::Draw(Rect *area)
{
	Rect		theFrame;
	PenState	savePen;
	long		i;
	CNObjectView *nextSubview = NULL;
	long		numItems;
	LongRect	subviewAperture;
	LongRect	thisArea;
	LongRect	tempLongRect;
	Rect		tempRect;
	Pattern		dragPattern;
	
	// Initialize the Drawing environment.
	GetPenState (&savePen);
	PenNormal();

	// Draw "this"
	ForceNextPrepare();
	Prepare();
	//EraseRect(&area);
	
	// If "this" is being dragged then it looks different.
	if (fBeingDragged) {
		GetIndPattern(&dragPattern, sysPatListID, 4);
		PenPat(&dragPattern);
		
		FrameRgn(itsRegion);
		
		GetIndPattern(&dragPattern, sysPatListID, 1);
		PenPat(&dragPattern);
	}
	
	// If "this" is selected then is looks different.
	else if (fSelected)
		FrameRgn(itsRegion);
	
	// "this" looks normal.
	else {
		GetIndPattern(&gPattern, sysPatListID, 1);
		FillRgn(itsRegion, gPattern);
	}
		
	// Draw all of "this"::subviews	
	if (itsSubviews) {
	
		// Convert area to this CNBrainPan's frame coords.
		QDToFrameR(area, &thisArea);
		
		// Compose the bounding extents of all the subview items.
		numItems = itsSubviews->GetNumItems();
		for (i = 1; i <= numItems; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// Compute the aperture of this subview item in CNBrainPan coords.
			nextSubview->GetAperture(&subviewAperture);
			nextSubview->FrameToEnclR(&subviewAperture);
			
			// If the aperture of the subview intersects with the area to be drawn,
			// then draw the subview.
			if (SectLongRect(&thisArea, &subviewAperture, &tempLongRect)) {
				nextSubview->EnclToFrameR(&tempLongRect);
				LongToQDRect(&tempLongRect, &tempRect);
				nextSubview->Draw(&tempRect);
			}
		}
	}  // end of "if (itsSubviews) {"
	
	SetPenState (&savePen);
}
	

/******************************************************************************
 ObtainTraveler 
 
 	The double "**" for the "aNOView" argument is needed because the value of
 	the argument "aNOView" is changed by this method.  A completely new CNObjectView is 
 	created to replace the old "aNOView" in the new graphical context.  It may
 	be useful to think of the movie "The Invasion of the Body Snatchers" as a 
 	metaphor for this switch -- what comes in ("aNOView") and what goes out ("aNOView")
 	look identical, but they aren't. 
******************************************************************************/

void CNObjectView::ObtainTraveler(CNObjectView **aNOView)
{
	CNObjectView *tempNOView = NULL;
	LongPt		placePt;
	LongRect	tempSubviewFrame;
	Point		tempPt;
	
	if (!(IncludesSubview(*aNOView))) {
		
		// Before putting *aNOView into its new graphical environment, we must
		// translate it from its coordinate system into the coordinate system of
		// its new enclosure.
		(*aNOView)->GetFrame(&tempSubviewFrame);
		SetLongPt(&placePt, tempSubviewFrame.left, tempSubviewFrame.top);
		(*aNOView)->FrameToWind(&placePt, &tempPt);
		WindToFrame(tempPt, &placePt);
		(*aNOView)->Place(placePt.h, placePt.v, TRUE);
		
		// Make an actual copy of *aNOView with this set as itsEnclosure.
		tempNOView = (*aNOView)->CopyNOViewInto(this, this);
		
		// Tell everyone concerned that I am being transplanted.
		(*aNOView)->BroadcastChange(NOBJECTVIEW_REPLACING_ANOTHER, ((void *) tempNOView));
		
		// The replacement is now complete, Dispose of the original, and make the switch.
		(*aNOView)->Dispose();
		(*aNOView) = NULL;
		(*aNOView) = tempNOView;
		tempNOView = NULL;

		// Refresh this object if its visible.
		(*aNOView)->Refresh();
	}
}


/******************************************************************************
 CalcCombinedSubviewRegion

	A recursive method that compiles all visible subviews' "itsRegion"s into one
	region.  This method is used for pick-correlation in CNOVCollection.
 ******************************************************************************/

RgnHandle	CNObjectView::CalcCombinedSubviewRegion()
{
	RgnHandle 	combinedRegion;
	RgnHandle 	tempRegion;
	long		i;
	CNObjectView *nextSubview = NULL;
	long		numSubviews;
	
	// Initialize "combinedRegion".
	combinedRegion = NewRgn();
	if (ReallyVisible())
		CopyRgn(itsRegion, combinedRegion);
	
	// Compose a Union of all of my subviews' "itsRegion"s (only the visible ones).
	if (itsSubviews) {
		
		// Compose the bounding extents of all the subview items.
		numSubviews = itsSubviews->GetNumItems();
		for (i = 1; i <= numSubviews; ++i) {
		
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// Get "nextSubview's" composed "itsRegion"
			tempRegion = nextSubview->CalcCombinedSubviewRegion();
			
			// Move "tempRegion" into the coordinates of itsEnclosure "this".
			OffsetRgn(tempRegion, nextSubview->hEncl, nextSubview->vEncl);
			
			// Compose "tempRegion" into "combinedRegion"
			UnionRgn(combinedRegion, tempRegion, combinedRegion);
			
			// Dispose of the temporary Region so we can use it again.
			DisposeRgn(tempRegion);
		}
	}
	
	return (combinedRegion);
}


/******************************************************************************
 GetOrigin

		Return the frame origin of a Pane. 
 ******************************************************************************/

void	CNObjectView::GetFrameOrigin(
	long		*theHFrameOrigin,
	long		*theVFrameOrigin)
{
	*theHFrameOrigin = hFrameOrigin;				/* Return instance variables		*/
	*theVFrameOrigin = vFrameOrigin;
}

	
/******************************************************************************
 AddSubview

 ******************************************************************************/

void	CNObjectView::AddSubview(
	CView		*theSubview)
{
	inherited::AddSubview(theSubview);
	
	// Initialize a relationship between this object and theSubview.
	DependUpon(theSubview);
	
	// Update this object's list of owned objects.
	if (!itsNOVOwner) {
		itsNOVOwner = new(CNOViewOwner);
		itsNOVOwner->INOViewOwner(this);
	}
	itsNOVOwner->AcquireNOView((CNObjectView *) theSubview);
	itsNOVOwner->Union(((CNObjectView *) theSubview)->itsNOVOwner);
	
	// Tell all dependents to update their lists of owned objects.
	BroadcastChange(NOBJECTVIEW_SUBVIEW_ADDED, ((void *) theSubview));
	
	// Update this object's list of HotSpots.
	if (member(theSubview, CNHotSpotView)) {
	
		if (!itsHotSpotsNOVOwner) {
			itsHotSpotsNOVOwner = new(CNOViewOwner);
			itsHotSpotsNOVOwner->INOViewOwner(this);
		}
	
		itsHotSpotsNOVOwner->AcquireNOView((CNObjectView *) theSubview);
		itsHotSpotsNOVOwner->Union(((CNObjectView *) theSubview)->itsHotSpotsNOVOwner);
		
		// Tell all dependents to update their lists of owned objects.
		BroadcastChange(NHOTSPOT_SUBVIEW_ADDED, ((void *) theSubview));
	}
}


/******************************************************************************
 GetMyNervousSysDoc

	Travel back up the supervisor hierarchy to this object's supervising document.
 ******************************************************************************/

CNervousSystemDoc *CNObjectView::GetMyNervousSysDoc(void)
{
	CBureaucrat *aBureaucrat = NULL;
	Boolean		fFoundBureaucrat = FALSE;
	
	if (itsSupervisor) {
		
		// Prime the while loop.
		aBureaucrat = itsSupervisor;
		
		while ((!(fFoundBureaucrat)) && (aBureaucrat)) {
			if (member(aBureaucrat, CNervousSystemDoc))
				fFoundBureaucrat = TRUE;
			else
				aBureaucrat = aBureaucrat->itsSupervisor;
		}
		
		if (fFoundBureaucrat)
			return ((CNervousSystemDoc *) aBureaucrat);
		else 
			return (NULL);
	}
}


/******************************************************************************
 GetMyNBrainPan

	Return a pointer to the NBrainPane that encloses me in the visual hierarchy.
 ******************************************************************************/

CNBrainPan	*CNObjectView::GetMyNBrainPan(void)
{	
	if (itsEnclosure) {
		if (member(itsEnclosure, CNBrainPan))
			return((CNBrainPan *) itsEnclosure);
		else    // We assume (member(itsEnclosure, CNObjectView))
			return((CNObjectView *) itsEnclosure)->GetMyNBrainPan();
	}
	
	else
		return(NULL);
}


/******************************************************************************
 ChangeSize {OVERRIDE}

		This is an overwrite of a CPane method.
		This ChangeSize does not keep the frame origin at (0, 0) like CPane.  CPane keeps
		the frame origin at (0, 0), so when the size changes, all the subviews
		need to be moved.  We do not want to do that much work, the frame origin
		can be negative if needed.  We had to overwrite CalcFrame and ResizeFrame to 
		support this overwrite.

		Change the size of a Pane. The delta rectangle parameter specifies
		how each side of the pane should change. Positive values mean a
		move to the right or down, negative values to the left or up.
 ******************************************************************************/

void	CNObjectView::ChangeSize(
	register Rect		*delta,			/* Movement for each side			*/
	Boolean				redraw)			/* Redraw Pane or not?				*/
{
    
    LongRect    oldAperture = aperture;     /* Save old aperture                */
    LongRect	oldFrame = frame;
	Rect 		tmp;

	if (redraw) {
		Refresh();						/* Force redraw of area currently	*/
										/*   occupied by the pane			*/
		RefreshBorder();				/*  and redraw border if there is one */
	}
	
	ResizeFrame(delta);					/* Change dimensions of pane		*/
						
    CalcAperture();                     /* Determine new aperture           */
	
	if (redraw) {
		Refresh();						/* Force draw of pane at new size	*/
		RefreshBorder();	/* TCL 1.1.1 DLP 10/1/91 */

										/* If autoRefresh is FALSE (and		*/
										/*   other safety checks), we do	*/
										/*   not want to redraw the area	*/
										/*   which is common to the old and	*/
										/*   new pane size. Note that the	*/
										/*   Refresh method will have set	*/
										/*   the port for us already.		*/
		if (!autoRefresh && ReallyVisible()) {
			if (SectLongRect(&aperture, &oldAperture, &oldAperture))
			{
				
				FrameToQDR( &oldAperture, &tmp);
				ValidRect( &tmp);
			}
			if (itsBorder)
			{
				Rect oldBorder, newBorder;
				
				FrameToWindR( &oldFrame, &oldBorder);
				itsBorder->CalcBorderRect( &oldBorder);
				FrameToWindR( &frame, &newBorder);
				itsBorder->CalcBorderRect( &newBorder);
				
				if (SectRect( &oldBorder, &newBorder, &tmp))
				{
					GetWindow()->Prepare();
					ValidRect( &tmp);
					Prepare();
				}
			}
		}
	}
	
	// Make all necessary changes up the visual hierarchy by using the depencendy relationships.
	BroadcastChange(NOBJECTVIEW_SUBVIEW_RESIZED, NULL);
}


/******************************************************************************
 ProviderChanged
 
 	Receive notification that a provider changed.
 	This is where we get automatic housekeeping on all itsNOVOwner lists.
 	When a subview is added to a NObjectView, it BroadcastChanges to its enclosure
 	and the enclosure then updates its ownership list, and uses BroadcastChanges to do 
 	its enclosure and ....  This insures that all CNObjectView::itsNOVOwner lists are 
 	automatically maintained and always up-to-date.
******************************************************************************/

void CNObjectView::ProviderChanged( CCollaborator *aProvider, long reason, void* info)
{	
	LongRect	delta;
	
	switch (reason) {
	
		case NOBJECTVIEW_SUBVIEW_ADDED:
			if (!itsNOVOwner) {
				itsNOVOwner = new(CNOViewOwner);
				itsNOVOwner->INOViewOwner(this);
			}
			itsNOVOwner->AcquireNOView((CNObjectView *) info);	
			itsNOVOwner->Union(((CNObjectView *) info)->itsNOVOwner);
			BroadcastChange(reason,  info);
			break;
			
		case NOBJECTVIEW_SUBVIEW_REMOVED:
			if (itsNOVOwner) {
				itsNOVOwner->RelinquishNOView((CNObjectView *) info);
				itsNOVOwner->Not(((CNObjectView *) info)->itsNOVOwner);
			}
			BroadcastChange(reason,  info);
 			break;

		case NHOTSPOT_SUBVIEW_ADDED:
			if (!itsHotSpotsNOVOwner) {
				itsHotSpotsNOVOwner = new(CNOViewOwner);
				itsHotSpotsNOVOwner->INOViewOwner(this);
			}
			itsHotSpotsNOVOwner->AcquireNOView((CNObjectView *) info);	
			itsHotSpotsNOVOwner->Union(((CNObjectView *) info)->itsHotSpotsNOVOwner);
			BroadcastChange(reason,  info);
			break;
			
		case NHOTSPOT_SUBVIEW_REMOVED:
			if (itsHotSpotsNOVOwner) {	
				itsHotSpotsNOVOwner->RelinquishNOView((CNObjectView *) info);
				itsHotSpotsNOVOwner->Not(((CNObjectView *) info)->itsHotSpotsNOVOwner);
			}
			BroadcastChange(reason,  info);
 			break;
 			
		case NOBJECTVIEW_SUBVIEW_RESIZED:
			EncloseWith((CNObjectView *) aProvider);
			BroadcastChange(reason,  info);
 			break;
	}

	inherited::ProviderChanged( aProvider, reason, info);
}


/******************************************************************************
 EncloseWith

	Adjust the size of "this" if the new subview does not fit within the existing frame.
	Note: a call to ChangeSize() will automatically recalculate "aSubview's" aperture,
	so we don't have to call aSubview->CalcAperture() here.
	
	"aSubview" has already been initialized (except for leaf-specific stuff), so it
	has already been added as a subview of this.
 ******************************************************************************/

void CNObjectView::EncloseWith(CNObjectView *aSubview)
{
	LongRect	subviewFrameRect;
	LongRect	thisFrameRect;
	LongRect	combinedFrameRect;
	LongRect	subviewDelta;
	Rect		tempRect;
	
	if (IncludesSubview(aSubview)) {
	
		// Get "aSubview's" frame.
		aSubview->GetFrame(&subviewFrameRect);
		aSubview->FrameToEnclR(&subviewFrameRect);
		
		// Determine whether the frame of "aSubview" is completely enclosed by itsEnclosure's frame.
		GetFrame(&thisFrameRect);
		UnionLongRect(&subviewFrameRect, &thisFrameRect, &combinedFrameRect);
		if (!(EqualLongRect(&thisFrameRect, &combinedFrameRect))) {
		
			// INV: aSubview is not completely contained in "this".
	
			// Initialize the amount of change needed.
			SetLongRect(&subviewDelta, 0, 0, 0, 0);
			subviewDelta.left = 	combinedFrameRect.left - 	thisFrameRect.left;
			subviewDelta.right = 	combinedFrameRect.right - 	thisFrameRect.right;
			subviewDelta.top = 		combinedFrameRect.top - 	thisFrameRect.top;
			subviewDelta.bottom = 	combinedFrameRect.bottom - 	thisFrameRect.bottom;
			
			// We assume that we don't need to do any shrinking.  
			if (subviewDelta.left > 0) subviewDelta.left = 0;
			if (subviewDelta.right < 0) subviewDelta.right = 0;
			if (subviewDelta.top > 0) subviewDelta.top = 0;
			if (subviewDelta.bottom < 0) subviewDelta.bottom = 0;

			// Change the size of "this" to accomodate the new subview.
			LongToQDRect(&subviewDelta, &tempRect);
			ChangeSize(&tempRect,TRUE);
		}
	}
}
	

/******************************************************************************
 ShrinkWrapWithout

	Shrink the size of "this" to its new possibly smaller size because aSubview is removed.
 ******************************************************************************/

void	CNObjectView::ShrinkWrapWithout(CNObjectView *aSubview)
{
	Rect		regionBBoxRect;
	LongRect	frameWithoutSubviews;
	LongRect	combinedFrameRect;
	long		i;
	long		numSubviews = 0;
	CNObjectView	*nextSubview = NULL;
	LongRect	subviewFrameRect;
	LongRect	thisFrameRect;
	LongRect	subviewDelta;
	Rect		tempRect;

	if (IncludesSubview(aSubview)) {

	  	// Calculate the new frame of "this" without the "aSubview".
	  	regionBBoxRect = (**itsRegion).rgnBBox;
	  	QDToLongRect(&regionBBoxRect, &frameWithoutSubviews);
	  	combinedFrameRect = frameWithoutSubviews;
		if (itsSubviews) {
			
			numSubviews = itsSubviews->GetNumItems();
			for (i = 1; i <= numSubviews; ++i) {
			
				// Access the next item in the subview list.
				nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
				
				// Accumulate all nextSubviews' frames, except "aSubview's".
				if (nextSubview != aSubview) {
					nextSubview->GetFrame(&subviewFrameRect);
					nextSubview->FrameToEnclR(&subviewFrameRect);
					UnionLongRect(&subviewFrameRect, &combinedFrameRect, &combinedFrameRect);
				}
			}
		}
		
	  	// Determine if "this" needs to be shrunk.
		if (!(EqualLongRect(&combinedFrameRect, &frameWithoutSubviews))) {
		
			// INV: "this" needs some shrinking.
	
			// Change the size of "this" to accomodate the new subview.
			GetFrame(&thisFrameRect);
			SetLongRect(&subviewDelta, 0, 0, 0, 0);
			subviewDelta.left = 	combinedFrameRect.left - 	thisFrameRect.left;
			subviewDelta.right = 	combinedFrameRect.right - 	thisFrameRect.right;
			subviewDelta.top = 		combinedFrameRect.top - 	thisFrameRect.top;
			subviewDelta.bottom = 	combinedFrameRect.bottom - 	thisFrameRect.bottom;
			LongToQDRect(&subviewDelta, &tempRect);
			ChangeSize(&tempRect,TRUE);
		}
	}
}
	
	
/******************************************************************************
 RemoveSubview

 ******************************************************************************/

void	CNObjectView::RemoveSubview(
	CView		*theSubview)
{	
	// End a relationship between this object and theSubview.
	CancelDependency(theSubview);
	
	// Update this object's list of owned objects.
	if (itsNOVOwner) {
		itsNOVOwner->RelinquishNOView((CNObjectView *) theSubview);
		itsNOVOwner->Not(((CNObjectView *) theSubview)->itsNOVOwner);
	}
	
	// Tell all dependents to update their lists of owned objects.
	BroadcastChange(NOBJECTVIEW_SUBVIEW_REMOVED,  ((void *) theSubview));
	
	// Update this object's list of HotSpots.
	if (member(theSubview, CNHotSpotView)) {
		if (itsHotSpotsNOVOwner) {
			itsHotSpotsNOVOwner->RelinquishNOView((CNObjectView *) theSubview);
			itsHotSpotsNOVOwner->Not(((CNObjectView *) theSubview)->itsHotSpotsNOVOwner);
			
			// Tell all dependents to update their lists of owned objects.
			BroadcastChange(NHOTSPOT_SUBVIEW_REMOVED, ((void *) theSubview));
		}
	}
	
	inherited::RemoveSubview(theSubview);
}

	
/******************************************************************************
 CalcFrame {OVERRIDE}
		
		This is an overwrite of a CPane method.
		We require this overwrite because our other overwrite of ChangeSize
		does not keep the frame origin at (0, 0) like CPane.  CPane keeps
		the frame origin at (0, 0), so when the size changes, all the subviews
		need to be moved.  We do not want to do that much work, the frame origin
		can be negative if needed.
		
		Determine the coordinates of the Pane's frame and origin based on
		the width, height, hEncl, vEncl. These instance variables must be
		set before invoking this method.
 ******************************************************************************/

void	CNObjectView::CalcFrame()
{
	long		hLocation;
	long		vLocation;
	
	frame.left = hFrameOrigin;
	frame.top = vFrameOrigin;
	frame.right = frame.left + width;
	frame.bottom = frame.top + height;
	
										/* Determine location of pane in	*/
										/*   Window coords					*/
	itsEnclosure->SubpaneLocation(hEncl, vEncl, &hLocation, &vLocation);

	hOrigin = - hLocation;				/* Origin maps between Frame and	*/
	vOrigin = - vLocation;				/*   Window coords. Origin is the	*/
										/*   top left corner of the window	*/
										/*   expressed in Frame coords		*/
	ForceNextPrepare();
}

	
/******************************************************************************
 ResizeFrame {OVERRIDE}

		This is an overwrite of a CPane method.
		We require this overwrite because our other overwrite of ChangeSize
		does not keep the frame origin at (0, 0) like CPane.  CPane keeps
		the frame origin at (0, 0), so when the size changes, all the subviews
		need to be moved.  We do not want to do that much work, the frame origin
		can be negative if needed.

		Adjust a Pane's frame when the size of the pane changes. The delta
		rectangle parameter specifies the change in location of each side
		of the Pane. Positive values mean a move to the right or down;
		negative values mean a move to the left or up.
 ******************************************************************************/

void	CNObjectView::ResizeFrame(
	register Rect	*delta)
{
										/* Adjust dimensions of the pane	*/
	width += delta->right - delta->left;
	height += delta->bottom - delta->top;

	frame.left += delta->left;			/* Shift edges by delta				*/
	frame.top += delta->top;
	frame.right += delta->right;
	frame.bottom += delta->bottom;
	
	// Note: hEncl, vEncl, hOrigin, vOrigin don't change, we're resizing the frame
	// not moving it within its enclosure.  The references to this pane's location in
	// itsEnclosure and the window still point to the (0, 0) of this pane's frame
	// coordinate system, and this hasn't moved.
	
	// Change our reference to the origin of the frame from its top left corner.
	// These are used in CNObjectView::CalcFrame();
	hFrameOrigin = frame.left;
	vFrameOrigin = frame.top;
	
	ForceNextPrepare();		
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of an NObjectView
 ******************************************************************************/

void	CNObjectView::Dispose(void)
{	
	fBeingDisposed = TRUE;
	
	// Tell itsEnclosure to resize without "this".
	if (member(itsEnclosure, CNObjectView))
		((CNObjectView *) itsEnclosure)->ShrinkWrapWithout(this);
	else if (member(itsEnclosure, CNBrainPan))
		((CNBrainPan *) itsEnclosure)->ShrinkWrapWithout(this);

	// Dispose of the ownership lists used for linking objects.
	if (itsNOVOwner) {
		itsNOVOwner->Dispose();
		itsNOVOwner = NULL;
	}
	if (itsHotSpotsNOVOwner) {
		itsHotSpotsNOVOwner->Dispose();
		itsHotSpotsNOVOwner = NULL;
	}
	
	// Dispose of the model for this view
	if (myNModelElem)
		myNModelElem->Dispose();
	
	if (IsVisible()) {
		Prepare();
		EraseRgn(itsRegion);
	}
	
	DisposeRgn(itsRegion);
	
		/* makes sure that the last undo task is disposed. Since the 	*/
		/* undo tasks usually have references back to this pane, 		*/
		/* it would be bad if this pane was disposed and the task was 	*/
		/* not informed.												*/

	inherited::Dispose();	// This call must be the last statement (see CObject::Dispose).
}


/******************************************************************************
 IsBeingDisposed 
 
******************************************************************************/

Boolean CNObjectView::IsBeingDisposed(void)
{
	return fBeingDisposed;
}


/******************************************************************************
 IncludesSubview
 
 	Return TRUE if the argument aNOView is already included in itsSubview list.
******************************************************************************/

Boolean CNObjectView::IncludesSubview(CNObjectView *aNOView)
{
	CList		*tempSubviews = NULL;
	Boolean		fIncludes = FALSE;
	
	if ((aNOView) && (itsSubviews)) {
	
		tempSubviews = GetSubviews();  // Create a working copy of itsSubviews
		
		if (tempSubviews->Includes(aNOView))
			fIncludes = TRUE;
		
		tempSubviews->Dispose();
	}
	
	return (fIncludes);
}


/******************************************************************************
 GetSubviews
 
******************************************************************************/

CList	*CNObjectView::GetSubviews(void)
{
	CList		*tempSubviews = NULL;

	if (itsSubviews) 
		tempSubviews = (CList *) itsSubviews->Copy();

	return (tempSubviews);
}


/******************************************************************************
 GetNumSubviews
 
******************************************************************************/

long	CNObjectView::GetNumSubviews(void)
{
	CList	*tempSubviews = NULL;
	long	numSubviews = 0;

	tempSubviews = GetSubviews();
	
	if (tempSubviews) {
		numSubviews = tempSubviews->GetNumItems();
		tempSubviews->Dispose();
	}
	
	return (numSubviews);
}


/******************************************************************************
 INOViewCopy 
 
 	Initialize a copy of this NOView.
******************************************************************************/

void CNObjectView::INOViewCopy(CNObjectView *theCopy)
{
	long	numItems = 0;
	long	i;
	CNObjectView	*nextSubview = NULL;
	CNObjectView	*nextSubviewCopy = NULL;

	// Copy the contents of itsRegion into theCopy.
	CopyRgn(itsRegion, theCopy->itsRegion);
	
	// Make an "Actual Copy" of all of the objects this object ownes.
	if (itsSubviews) {
		
		numItems = itsSubviews->GetNumItems();
		for (i = 1; i <= numItems; ++i) {
			
			// Access the next item in the subview list.
			nextSubview = (CNObjectView *) itsSubviews->NthItem(i);
			
			// Make a copy of this subview item.
			nextSubviewCopy = nextSubview->CopyNOViewInto(theCopy, theCopy);
		}
	}
}


/******************************************************************************
 CopyNOViewInto  {a place holder for inherited methods}
 
******************************************************************************/

CNObjectView *CNObjectView::CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor)
{
}


/******************************************************************************
 GetMyTopNOVEnclosure

	Return a pointer to my top-level-NOView-enclosure (my enclosure in the visual hierarchy
	that is directly enclosed by a NBrainPan.
 ******************************************************************************/

CNObjectView	*CNObjectView::GetMyTopNOVEnclosure(void)
{	
	if (itsEnclosure) {
		if (member(itsEnclosure, CNBrainPan))
			return(this);
		else    // We assume (member(itsEnclosure, CNObjectView))
			return((CNObjectView *) itsEnclosure)->GetMyTopNOVEnclosure();
	}
	
	else
		return(NULL);
}


/******************************************************************************
 GetMyNeuronEnclosure

	Return a pointer to my NNeuronView-enclosure (my NNeuronView enclosure in the 
	visual hierarchy.
 ******************************************************************************/

CNNeuronView	*CNObjectView::GetMyNeuronEnclosure(void)
{	
	if (itsEnclosure) {
		if (member(itsEnclosure, CNBrainPan))
			return(NULL);
		else if (member (this, CNNeuronView))
			return((CNNeuronView *) this);
		else    // We assume (member(itsEnclosure, CNObjectView))
			return((CNObjectView *) itsEnclosure)->GetMyNeuronEnclosure();
	}
	
	else
		return(NULL);
}


/******************************************************************************
 SetFSelected

 ******************************************************************************/

void	CNObjectView::SetFSelected(Boolean fChosen) 
{	
	// Set the selection state of "this"
	fSelected = fChosen;
	
	// If I'm selected, then so are all my enclosures.
	if (itsNOVOwner)
		itsNOVOwner->SetFSelectedAll(fChosen);
	
	// Initiate a reDraw of this item -- its selection-status has changed.
	Refresh();
}


/******************************************************************************
 SetFBeingDragged

	This method does not initiate a Refresh of this object, it relies on the 
	calling environment to manage the necessary graphical update.
 ******************************************************************************/

void	CNObjectView::SetFBeingDragged(Boolean fDragging) 
{	
	// Set the being-dragged state of "this"
	fBeingDragged = fDragging;

	// If I'm being dragged, then so are all my enclosures.
	if (itsNOVOwner)
		itsNOVOwner->SetFBeingDraggedAll(fDragging);
}


/******************************************************************************
 SetFDrawHotSpot

	This method does not initiate a Refresh of this object, it relies on the 
	calling environment to manage the necessary graphical update.
  ******************************************************************************/

void	CNObjectView::SetFDrawHotSpot(Boolean fDraw)
{
	// Set the draw-hot-spots state of "this"
	fDrawHotSpots = fDraw;
}


/******************************************************************************
 SetItsNBrainPan
 
		Binds "itsNBrainPan" to the the given reference; and uses a recursive
		loop to do the same for all the members of "itsSubviews" 
******************************************************************************/

void CNObjectView::SetItsNBrainPan(CNBrainPan *aNBrainPan)
{
	// Set itsNBrainPan of "this"
	itsNBrainPan = aNBrainPan;
}


/******************************************************************************
 CopyItsHotSpots 
 
		Return a copy of itsHotSpotsNOVOwner (not an actual copy)
 ******************************************************************************/

CNOViewOwner	*CNObjectView::CopyItsHotSpots(CBureaucrat *aSupervisor)
{
	CNOViewOwner *aNOVOwner = NULL;

	if (itsHotSpotsNOVOwner) 
		aNOVOwner = itsHotSpotsNOVOwner->CopyNOVOwnerFor(aSupervisor);

	return(aNOVOwner);
}


/******************************************************************************
 MoveBy
 
	This method does not re-Draw the object after it has been moved, it relies
	on the calling environment for the re-Drawing or Refreshing.
 ******************************************************************************/

void	CNObjectView::MoveBy(LongPt *delta)
{
	LongPt		zeroPt;
	
	SetLongPt(&zeroPt, 0, 0);
	
	if (!(EqualLongPt(delta, &zeroPt))) {
		Offset(delta->h, delta->v, FALSE);
	}
}


/******************************************************************************
 ToggleMoveToGBookmark

 ******************************************************************************/

void	CNObjectView::ToggleMoveToGBookmark(void)
{
	LongPt	tempPosition;
	
	tempPosition = graphicalBookmark;
	
	// Re-initialize the graphicalBookmark.
	InitGraphicalBookmark();
	
	// If I have moved since I placed my graphicalBookmark, then move back.
	if (!(EqualLongPt(&tempPosition, &graphicalBookmark)))
		Place(tempPosition.h, tempPosition.v, TRUE);
}


/******************************************************************************
 InitGraphicalBookmark

		A graphical bookmark for Undoing moves.
 ******************************************************************************/

void	CNObjectView::InitGraphicalBookmark(void) 
{
	LongRect	tempFrame;
	
	GetFrame(&tempFrame);
	FrameToEnclR(&tempFrame);
	graphicalBookmark.v = tempFrame.top;
	graphicalBookmark.h = tempFrame.left;
}


/******************************************************************************
 IsSelected

 ******************************************************************************/

Boolean	CNObjectView::IsSelected(void)
{
	return fSelected;
}


/******************************************************************************
 DoClick {OVERRIDE}

		Mouse click inside the NBrainPan
 ******************************************************************************/

void	CNObjectView::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{	
	Rect			tempFrame;
	LongRect		aNOVBounds;
	LongRect		tempFrameLong;
	Boolean			notifiable = TRUE;
	LongPt			trackStart;
	CNOVDraggerTask	*theNOVDragTask = NULL;  
	LongPt			tempLongPt;

	// Double check that we should be here.
	GetFrame(&tempFrameLong);
	LongToQDRect(&tempFrameLong, &tempFrame);
	if (!(PtInRect(hitPt, &tempFrame))) return;
			
	// Initialize stuff used in the Call to TrackMouse for CNDraggerTask.
	itsNBrainPan->GetBounds(&aNOVBounds);
	QDToLongPt( hitPt, &trackStart);
	
	// Pass the message up the visual hierarchy to some object that can handle other kinds
	// of gNTools.
	if (gNTool != POINTER) {
		
		// Initialize stuff used to pass the work to another object.
		// The hitPt must be converted into itsEnclosure's coordinate system.
		QDToLongPt(hitPt, &tempLongPt);
		FrameToEncl(&tempLongPt);
		LongToQDPt(&tempLongPt, &hitPt);
	
		itsEnclosure->DoClick(hitPt, modifierKeys, when);
	}
	
	// "this" belongs to a CNOVCollection, and (gNTool == POINTER), so 
	// itsEnclosure needs to do the dragging and selecting work.
	else if (member(itsEnclosure, CNOVCollection)) {
		
		// Initialize stuff used to pass the work to another object.
		// The hitPt must be converted into itsEnclosure's coordinate system.
		QDToLongPt(hitPt, &tempLongPt);
		FrameToEncl(&tempLongPt);
		LongToQDPt(&tempLongPt, &hitPt);
	
		((CNOVCollection *) itsEnclosure)->DoClickEnclosure(hitPt, modifierKeys, when, this);
	}
	
	// "itsEnclosure" is not a CNOVCollection, and (gNTool == POINTER),
	// so "this" needs to do its own selection and dragging work.
	else {   

		// The "shift" key is down, so just edit the selection list.
		if (modifierKeys & shiftKey) {  	// The "shift" key IS down.
			if (!fSelected) {
				itsNBrainPan->AddToSelection(this);
			}
			else {
				itsNBrainPan->RemoveFromSelection(this);
			}
		}
		
		// Try to "Drag" the selection, maybe after reseting it to only "this".
		else {  							// The "shift" key is NOT down.
			
			// First click is for selecting and/or dragging.
			if (gClicks == 1) {
				
				// If "this" isn't already part of the selection list then reset the
				// selection list and make this the only member.
				if (!fSelected) {
					itsNBrainPan->ResetSelectionList();
					itsNBrainPan->AddToSelection(this);
				}
				
				// Initialize the DragTask.
				theNOVDragTask = new(CNOVDraggerTask);
				theNOVDragTask->INOVDraggerTask(kGMyUndoMoveItem, this, this, itsNBrainPan);
				
				// Try to drag.  This "TrackMouse" needs to be done by itsNBrainPan -- 
				// that's where it's being dragged.
				FrameToItsNBrainPan(&trackStart);
				itsNBrainPan->TrackMouse(theNOVDragTask, &trackStart, &aNOVBounds);
				itsNBrainPan->Refresh();
				
				// If I didn't actually "Drag", then Dispose of the false-alarm DragTask.
				if (!(theNOVDragTask->DidYouDrag())) {	// A selection click, not a drag.				
					theNOVDragTask->Dispose();
				}
				
				// I did actually drag so init pointers to the DragTask for future use.
				else {  	// A drag click.						
					if (notifiable) {			
												
						itsSupervisor->Notify(theNOVDragTask);
					}
				}
			}
			
			// Second click is for getting the dialog box.
			else if (gClicks == 2) {
				// Perform a dialog with the user about the parameters of this item.
				DoDialog();
			}
		}
	}
}
	

/******************************************************************************
 FrameToItsNBrainPan   

	This method is similar to FrameToEncl but converts the given point into 
	the coordinate system of itsNBrainPan, which is not necessarily itsEnclosure.
 ******************************************************************************/

void	CNObjectView::FrameToItsNBrainPan(LongPt *thePoint)
{
	if (itsEnclosure) {
		FrameToEncl(thePoint);
		if (member(itsEnclosure, CNBrainPan))
			return;
		else
			((CNObjectView *) itsEnclosure)->FrameToItsNBrainPan(thePoint);
	}
	
	else 
		exit(1);
}


/******************************************************************************
 FrameToItsNBrainPanR   

	This method is similar to FrameToEnclR but converts the given rect into 
	the coordinate system of itsNBrainPan, which is not necessarily itsEnclosure.
 ******************************************************************************/

void	CNObjectView::FrameToItsNBrainPanR(LongRect *theRect)
{
	if (itsEnclosure) {
		FrameToEnclR(theRect);
		if (member(itsEnclosure, CNBrainPan))
			return;
		else
			((CNObjectView *) itsEnclosure)->FrameToItsNBrainPanR(theRect);
	}
	
	else 
		exit(1);
}


/******************************************************************************
 DoDialog 

	This method is meant to be overridden by the individual item classes.
 ******************************************************************************/

void	CNObjectView::DoDialog(void)
{
} 


/******************************************************************************
 Contains

		Check whether a point, specified in Window coordinates, is contained
		within the aperture of a NObjectView, AND within its drawn region.
 ******************************************************************************/

Boolean		CNObjectView::Contains(
	Point			windPt)
{
	LongPt	framePt;
	Point	tempPt;
	
	WindToFrame( windPt, &framePt);		/* Convert to Frame coordinates		*/
	
	if (PtInLongRect(&framePt, &aperture)) {
		LongToQDPt(&framePt, &tempPt);
		return(PtInRgn(tempPt, itsRegion));
	}
	else
		return(FALSE);
}





