#pragma once

/******************************************************************************
 CNObjectView.h

	Interface for the NObjectView Class
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
	
	This class is the core of the NervousSystem project.  
	This is an abstract class (no direct instances) that supports the majority
	of the behaviors of all of the NervousSystem visible objects.  This class is
	the heart of all the direct-manipulation supported by NervousSystem. 
	
	Unlike its parent, CPane, the "frame" of this class is not constrained at (0,0).
	This change to a non-constrained "frame" was needed to support the ability of "this"
	to easily enclose other grandchildren of CNObjectView, while retaining the 
	graphical models of "this" unchanged.  So, the graphical models of "this" don't
	have to be shifted when "this" encloses a new subview.  CNObjectView needs
	to be able to enclose others like itself dynamically, but CPane doesn't support 
	that behavior.  
	
	Since "frame" is no longer constrained to (0,0) we need a way to represent the 
	location of the origin of the "frame".  We introduce two new ivars:
	"hFrameOrigin" and "vFrameOrigin" that represent the origin of the "frame".
	
	We make good use of the inherited behavior from CCollaborator.  When "this" is
	changed in some important way, we "BroadcastChange()" to all those who need to 
	know about it.  This saves us the complexity of having to keep track of 
	who needs to know what, and then remembering to call them specifically 
	whenever something changes.  See the enumerated list of reasons for 
	"ProviderChanged" below.
 ******************************************************************************/
 
#define _H_CNObjectView
#define		PAT_MARCH_ANTS	300			/* Pattern for marching ants		*/

#define		BRUSH_WIDTH_AXON			6			/* Size of brush in pixels			*/
#define		HALF_BRUSH_WIDTH_AXON		3			/* Size of brush in pixels			*/
#define		BRUSH_WIDTH_HOTSPOT			1			/* Size of brush in pixels			*/
#define		HALF_BRUSH_WIDTH_HOTSPOT 	1			/* Size of brush in pixels			*/
#define		BRUSH_WIDTH_DENDRITE		3			/* Size of brush in pixels			*/
#define		HALF_BRUSH_WIDTH_DENDRITE 	2			/* Size of brush in pixels			*/
#define		BRUSH_WIDTH_SOMA			0			/* Size of brush in pixels			*/
#define		HALF_BRUSH_WIDTH_SOMA	 	0			/* Size of brush in pixels			*/
#define		BRUSH_WIDTH_SYNAPSE			0			/* Size of brush in pixels			*/
#define		HALF_BRUSH_WIDTH_SYNAPSE 	0			/* Size of brush in pixels			*/
#define		BRUSH_WIDTH_STIMULATOR		0			/* Size of brush in pixels			*/
#define		HALF_BRUSH_WIDTH_STIMULATOR	0			/* Size of brush in pixels			*/


#include "NervousSystemGlobals.h"
#include "CPane.h"						/* Interface for its superclass		*/
#include "LongCoordinates.h"

class CNBrainPan;
class CView;
class CNOViewOwner;
class CObject;
class CNObjectView;
class CPaneBorder;
class CNModel;
class CNervousSystemDoc;
class CBureaucrat;
class CDirectorOwner;
class CNNeuronView;


class CNObjectView : public CPane {		/* Class Declaration				*/

public:
								/** Instance Variables **/
	// Note on protection of these IVars: they should all be off limits to all other classes
	// except CNOViewOwner.  However, I had to make them "public" so that CNOViewOwner 
	// could access them.  It's not possible with this version of Think C, but these ivars
	// should be set to "protected" and to "friend" for CNOViewOwner.
	RgnHandle 		itsRegion;
	short			penWidth;
	CNOViewOwner 	*itsHotSpotsNOVOwner;	// A list of ALL the HotSpots I own.
	CNOViewOwner 	*itsNOVOwner;			// A list of ALL the objects I own.
	Boolean			fSelected;
	Boolean			fBeingDragged;
	Boolean			fDrawHotSpots;
	Boolean			fBeingCopied;
	CNBrainPan		*itsNBrainPan;
	CNModel		 	*myNModelElem;
	
								/** Instance Methods **/
	void		INObjectView(CView *anEnclosure, CBureaucrat *aSupervisor,
							short aWidth, short aHeight,
							short aHEncl, short aVEncl,
							SizingOption aHSizing, SizingOption aVSizing, 
							long aHFrameOrigin, long aVFrameOrigin);
	virtual void		Dispose(void);
	virtual void		INOViewCopy(CNObjectView *theCopy);
	virtual CNObjectView	*CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor);
	virtual Boolean		Contains(Point windPt);
	virtual void		DoClick(Point hitPt, short modifierKeys, long when);
	virtual void		Draw(Rect *area);

	virtual void		ObtainTraveler(CNObjectView **aNOView);
	virtual CNBrainPan	*GetMyNBrainPan(void);
	virtual CNervousSystemDoc	*GetMyNervousSysDoc(void);
	virtual CNObjectView	*GetMyTopNOVEnclosure(void);
	virtual CNNeuronView	*GetMyNeuronEnclosure(void);
	virtual void		InitGraphicalBookmark(void);
	virtual void		ToggleMoveToGBookmark(void);
	virtual void		MoveBy(LongPt *delta);	
	virtual Boolean		IsSelected(void);
	virtual CNOViewOwner	*CopyItsHotSpots(CBureaucrat *aSupervisor);
	virtual void		SetFSelected(Boolean fChosen);
	virtual void		SetFBeingDragged(Boolean fDragging);
	virtual void		SetFDrawHotSpot(Boolean fDraw);
	virtual void		SetItsNBrainPan(CNBrainPan *aNBrainPan);
	virtual void 		DoDialog(void);
	virtual void		FrameToItsNBrainPan(LongPt *thePoint);
	virtual void		FrameToItsNBrainPanR(LongRect *theRect);
	virtual Boolean		IncludesSubview(CNObjectView *aNOView);
	virtual void		AddSubview(CView *theSubview);
	virtual void		RemoveSubview(CView *theSubview);
	virtual void		ProviderChanged( CCollaborator *aProvider, long reason, void* info);
	virtual void		CalcFrame(void);
	virtual void		ResizeFrame(Rect *delta);
	virtual void		ChangeSize(Rect *delta, Boolean redraw);
	virtual void		EncloseWith(CNObjectView *aSubview);
	virtual void		GetFrameOrigin(long *theHFrameOrigin, long *theVFrameOrigin);
	virtual	RgnHandle	CalcCombinedSubviewRegion(void);
	virtual CList		*GetSubviews(void);
	virtual CNModel		*GetModel(void);
	virtual int			GetIndexOfSubview(CNObjectView *aSubview);
	virtual CList		*BuildListOfModelsOfSubviews(void);
	
protected:
	long		hFrameOrigin;  // h&vFrameOrigin are used in ::CalcFrame().  They are needed
	long		vFrameOrigin;  // because we allow frame coords other than 0,0.
	virtual void		ShrinkWrapWithout(CNObjectView *aSubview);
	virtual Boolean		IsBeingDisposed(void);
	virtual long		GetNumSubviews(void);

private:
	LongPt		graphicalBookmark;
	Boolean		fBeingDisposed;
};


// Some constants specific to DrawableObjects.
#define MAXLENDIALOGFLOAT 30


// Reasons for ProviderChanged.
// Note: the values bound to these enumerated reasons must not be the same as the values
// of other reasons the TCL uses (for example, "enum { bureaucratIsGopher = 1" in CBureaucrat.h,
// is used in the call "BroadcastChange( bureaucratIsGopher, 0);", in CBureaucrat::BecomeGopher).
enum
{
	NOBJECTVIEW_SUBVIEW_ADDED = 1001,
	NOBJECTVIEW_SUBVIEW_REMOVED = 1002,
	NHOTSPOT_SUBVIEW_ADDED = 1003,
	NHOTSPOT_SUBVIEW_REMOVED = 1004,
	NOBJECTVIEW_REPLACING_ANOTHER = 1005,
	NOBJECTVIEW_SUBVIEW_RESIZED = 1006
};



