/******************************************************************************
 CNSynapseViewTask.c

							The NSynapseViewTask Class
		
	SUPERCLASS = CNOViewTask
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).

 ******************************************************************************/

#include "Global.h"
#include "CNSynapseViewTask.h"
#include "LongCoordinates.h"
#include "NervousSystemGlobals.h"
#include "CNSynapseView.h"
#include <limits.h>
#include "LongQD.h"
#include "CNHotSpotView.h"
#include "CNBrainPan.h"


/******************************************************************************
 INSynapseViewTask

		Initialize a NSynapseViewTask object
 ******************************************************************************/

void	CNSynapseViewTask::INSynapseViewTask(
	short		aNameIndex, 
	CNBrainPan *aNBrainPan)
{
	inherited::INOViewTask(aNameIndex, aNBrainPan);
	
	// Initialize Graphic Data needed for "Redo".
	SetLongRect(&itsGraphicData, 0, 0, 0, 0);
}


/******************************************************************************
 CalcHSVBoundRect    

 ******************************************************************************/

void	CNSynapseViewTask::CalcHSVBoundRect(LongRect *boundingRect, LongPt aLocationPt)
{
	LongRect	tempRect;
	long		tempWidth;
	long		halfTempWidth;
	LongPt		halfWayPt;
	LongPt		offsetPt;
	
	// Initialize the bounding rect, adjusted for the pen width.
	tempWidth = SYNAPSE_HOTPT_SIZE + BRUSH_WIDTH_HOTSPOT;
	SetLongRect(&tempRect, 0, 0, tempWidth, tempWidth);
	
	// Put the bounding rect into window coordinates.
	halfTempWidth = tempWidth / 2;
	SetLongPt(&halfWayPt, halfTempWidth, halfTempWidth);
	offsetPt = aLocationPt;
	SubLongPt(&halfWayPt, &offsetPt);
	OffsetLongRect(&tempRect, offsetPt.h, offsetPt.v);
	
	// Make the assignment.
	*boundingRect = tempRect;
}


/******************************************************************************
 CalcGModelBoundRect

	Calculate the bounding rectangle of the points contained in itsGraphicalData.
	We assume that there is at least one point in this task's graphical model.
 ******************************************************************************/

void	CNSynapseViewTask::CalcGModelBoundRect(LongRect *boundRect)
{
	*boundRect = itsGraphicData;
}


/******************************************************************************
 CreateNewNOView     {this method is an override}

	Create a new instance of CNSynapseView, and bind it to the ivar "itsNewNOView".
	Initialize itsNewNOView with the information from this task.
	A CNHotSpotView is added as a subview to itsNewNOView.
	itsNewNOView is auotmatically resized to enclose the subview 
	(addSubview does this for us).
	
	Note: this method is an almost exact copy of CNSomaViewTask::CreateNewNOView.
 ******************************************************************************/

void	CNSynapseViewTask::CreateNewNOView(void) 
{
	LongRect	tempTaskGModel;  // A working copy of this task's itsGraphicalData
	RgnHandle 	tempTaskGRegion;
	LongRect	boundRectGData;  // A bounding Rect for the view of the graphic model
	LongRect	boundRectHSView;  // A bounding Rect for the HotSpotView
	CNHotSpotView *nHotSpot = NULL;
		
	// Init working copies of this task's graphic data.
	tempTaskGModel = itsGraphicData;
	tempTaskGRegion = NewRgn();
	
	
  // Create a new instance of CNSynapseView.
  
	// Calc boundRectHSView of the HotSpotView that-will-be-attached to the start point, and  
	// adjust for pen width.  In Window coordinates.
	CalcGModelBoundRect(&boundRectGData);
	InsetLongRect(&boundRectGData, -HALF_BRUSH_WIDTH_SYNAPSE, -HALF_BRUSH_WIDTH_SYNAPSE);
	
	// Create the new CNSynapseView instance, now that we know how big to make it.
	itsNewNOView = new(CNSynapseView);
	((CNSynapseView*) itsNewNOView)->INSynapseView(itsNBrainPan, itsNBrainPan,
				(boundRectGData.right - boundRectGData.left), 
				(boundRectGData.bottom - boundRectGData.top), 
				boundRectGData.left, boundRectGData.top,
				sizFIXEDSTICKY, sizFIXEDSTICKY,
				0, 0, NULL);
	
  // Now that itsNewNOView is created, initialize itsRegion and itsGraphicData
	
	// Init and Assign itsNewNOView's graphical model.
	// Init: Shift the temp graphical model to the soma's local coordinate system.
	OffsetLongRect(&tempTaskGModel, -boundRectGData.left, -boundRectGData.top);	
	((CNSynapseView *) itsNewNOView)->itsGraphicData = tempTaskGModel;
	
	// Init and Assign itsNewNOView's itsRegion.
	InitRegionWithGModel(tempTaskGRegion, &tempTaskGModel);
	CopyRgn(tempTaskGRegion, itsNewNOView->itsRegion);
	
	// Initialize itsNBrainPan->itsSelection to point to this new object.
	itsNBrainPan->AddToSelection(itsNewNOView);


  // Create A new CNHotSpotView instance for itsNewNOView.
  // This CNHotSpotView instance is a subview of itsNewNOView, and is enclosed by it.

	// Create a new CNHotSpotView instance.
	CalcHSVBoundRect(&boundRectHSView, locationPt);
	InsetLongRect(&boundRectHSView, -HALF_BRUSH_WIDTH_HOTSPOT, -HALF_BRUSH_WIDTH_HOTSPOT);
	
	nHotSpot = new(CNHotSpotView);
	((CNHotSpotView*) nHotSpot)->INHotSpotView(itsNewNOView, itsNewNOView, 
				(boundRectHSView.right - boundRectHSView.left),
				(boundRectHSView.bottom - boundRectHSView.top), 
				(boundRectHSView.left - boundRectGData.left), 
				(boundRectHSView.top - boundRectGData.top),
				sizFIXEDSTICKY, sizFIXEDSTICKY,
				0, 0);
	
	
	// Do housekeeping.
	DisposeRgn(tempTaskGRegion);
	nHotSpot = NULL;	
}


/******************************************************************************
 InitRegionWithGModel 

 ******************************************************************************/

void	CNSynapseViewTask::InitRegionWithGModel(RgnHandle tempRegion, LongRect *tempGraphicData) {
	
	itsNewNOView->Prepare();
	
	OpenRgn();
	LFrameOval(tempGraphicData);
	CloseRgn(tempRegion);
}


/******************************************************************************
 Dispose     "Dispose of a NOViewTask object"
 
	We do not Dispose of "*itsNBrainPan", or "*itsNewNOView", they are only pointers
	to objects who's existance is independent of this task.  The scopes of these 
	local pointers will expire automatically when the task object is Disposed.
 ******************************************************************************/

void	CNSynapseViewTask::Dispose()
{
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 BeginTracking

		Mouse down tracking is about to begin at the specified starting
		point. 
 ******************************************************************************/

void	CNSynapseViewTask::BeginTracking(
	LongPt		*startPt)
{
	LongPt	offsetPt;
	long	halfWidth;
	
	inherited::BeginTracking(startPt);
	
	// Initialize this Synapse's graphical model.
	locationPt = *startPt;
	SetLongRect(&itsGraphicData, 0, 0, SYNAPSE_SIZE, SYNAPSE_SIZE);
	halfWidth = SYNAPSE_SIZE / 2;
	SetLongPt(&offsetPt, ( startPt->h - halfWidth), ( startPt->v - halfWidth));
	OffsetLongRect(&itsGraphicData, offsetPt.h, offsetPt.v);
	
	// Initialize itsRegion.
	OpenRgn();
	LFrameOval(&itsGraphicData);
	CloseRgn(itsRegion);
		
	// Draw on the screen in itsNBrainPan while the mouse is down.
	itsNBrainPan->Prepare();
	PenNormal();
	FrameRgn(itsRegion);
}





