/******************************************************************************
 CNAxonModel.c

							The CNAxonModel Class
	
	The Axon Model
		
	SUPERCLASS = CNModelElem
	
	Source code for "NervousSystem"
	Copyright  1994 Joe Geigel. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
	

 ******************************************************************************/


#include "CNAxonModel.h"

#include <stdio.h>
#include <string.h>

#include "CNSimulator.h"
#include "CNDelayQueue.h"



int CNAxonModel::count = 0;


/*
 *	GetAxonData
 */
 
void CNAxonModel::GetAxonData (nAxonData *data)
{
	strcpy (data->aName, get_name());
	data->length = length;
	data->velocity = velocity;
	data->decay_coeff = decay_coeff;
	data->metered = metered;
}



/*
 *	SetAxonData
 */
 
void CNAxonModel::SetAxonData (nAxonData *data)
{
	set_name (data->aName);
	length = data->length;
	velocity = data->velocity;
	decay_coeff = data->decay_coeff;
	metered = data->metered;
	future_pot->Init ((int)(length/velocity));
}




/*
 *	prep_for_simulation - stuff you gotta do before simulating.
 */
 int CNAxonModel::prep_for_simulation (void)
 {
 		
/*
 *		if the ms_per_tick isn't equal to 1, you had best readjust
 *		your delay queue.
 */
 		if (CNSimulator::ms_per_tick() != 1.0) 
 			future_pot->Init((int)((1.0/CNSimulator::ms_per_tick()) * (length/velocity)));
/*
 *		first do what every good element should do
 */
 		return (inherited::prep_for_simulation());
 }	 
 
 
/*
 *	process - process the axon for a single time step, determining
 *			  the output potention for that time step.
 *
 */
 
 void CNAxonModel::process (void)
 {
 /*
  *		get the current output potential from the futures Queue
  */
  		inherited::set_output (future_pot->now());
  		
 /*
  *		set the future output potential taking into account the 
  *		decay determined by the decay coefficient
  */
   		future_pot->later (decay_coeff * inherited::get_input());
   		
 /*
  *		report if you are metered.
  */
  		if (metered)
  			CNSimulator::record_signal (get_input(), get_output(), report_id);
   		
 /*
  *		clear the input potential in preparation for next step.
  */
  		inherited::clear_input();
  		
 }
 

/*
 *		describe_self - Write info about yourself into a file.
 */
void CNAxonModel::describe_self (FILE *f) 
{
		fprintf (f, "*** Axon : %s ***\n", get_name());
		fprintf (f, "\t length = %f\n", length);
		fprintf (f, "\t velocity = %f\n", velocity);
		fprintf (f, "\t decay coefficient = %f\n\n\n", decay_coeff);
}



/******************************************************************************
 INAxon

		Initialize a CNAxonModel object
 ******************************************************************************/

int	CNAxonModel::INAxonModel ( void )
{

	char 	name[10];
	
	sprintf (name,"axon%d",count);
	
	inherited::init (name, 1, MAX_OUTPUT);

	count++;
	
	length = 5.0;
	velocity = 50.0;
	decay_coeff = 1.0;
	
	future_pot = new CNDelayQueue;
	
	return (future_pot->Init ((int)(length/velocity)));
}


/******************************************************************************
 CopyModel

		Returns a clone of yourself
 ******************************************************************************/

CNModel *CNAxonModel::CopyModel ( void )
{

	CNAxonModel	*A;
	
	A = new(CNAxonModel);
	A->INAxonModel ();
	
	A->RegisterAsCopy (this);
	
	A->length = length;
	A->velocity = velocity;
	A->decay_coeff = decay_coeff;
	
	A->future_pot->Init((int)(length/velocity));
	
	return A;
}







