/******************************************************************************
 CNDendriteModel.c

							The NDendriteModel Class
	
	Dendrite Model
		
	SUPERCLASS = CNModelElem
	
	Source code for "NervousSystem"
	Copyright  1994 Joe Geigel. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
	

 ******************************************************************************/

#include "CNDendriteModel.h"

#include <stdio.h>
#include <string.h>
#include <math.h> 

#include "CNDelayQueue.h"
#include "CNSimulator.h"


int CNDendriteModel::count = 0;


/*
 *	GetDendriteData 
 */
 void CNDendriteModel::GetDendriteData (nDendriteData *data)
 {
 	strcpy (data->aName, get_name());
	data->length = length;
	data->velocity = velocity;
	data->decay_coeff = decay_coeff;
	data->acc_in_decay = acc_in_decay;
	data->metered = metered;
}



/*
 *	SetDendriteData
 */
 void CNDendriteModel::SetDendriteData (nDendriteData *data)
 {
 	set_name(data->aName);
	length = data->length;
	velocity = data->velocity;
	decay_coeff = data->decay_coeff;
	acc_in_decay = data->acc_in_decay;
	metered = data->metered;
	future_pot->Init ((int)(length/velocity));
}
 
 

 
 /*
 *	prep_for_simulation - stuff you gotta do before simulating.
 */
 int CNDendriteModel::prep_for_simulation (void)
 {
 		
/*
 *		if the ms_per_tick isn't equal to 1, you had best readjust
 *		your delay queue.
 */
 
 		future_pot->Init((int)((1.0/CNSimulator::ms_per_tick()) * (length/velocity)));
 		
 /*
  *		redefine your accumulated input decay constant.  
  */
  		aid_per_tick =	(float)( pow ((double)acc_in_decay, 
  				(double)(CNSimulator::ms_per_tick())));
  		
/*
 *		resets
 */
 		acc_in_pot = 0.0;


/*
 *		first do what every good element should do
 */
 		return (inherited::prep_for_simulation());
 }
 
 
 /*
 *	process - process the axon for a single time step, determining
 *			  the output potention for that time step.
 *
 */
 
 void CNDendriteModel::process (void)
 {
 
 		float mult;
 		
 /*
  *		get the current output potential from the futures Queue
  */
  		inherited::set_output (future_pot->now());
  		
  		
 /*		calculate the accumulated input potential.  This potential
  *		is the current input plus a fraction of the previous accumulated
  *		input potential.  The fraction is given by the accumulated 
  *		input decay constant.
  */
   
  		acc_in_pot = inherited::get_input() + ((aid_per_tick * acc_in_pot)); 
			
  		
  		
 /*
  *		calculate and record the output that is to be produced after 
  *		ample delay.  This is determined by decaying the accumulated
  *		input potential by the decay coefficient
  */
  		future_pot->later (decay_coeff * acc_in_pot);
  		
  		
 /*
  *		give a report if you are metered
  */
  		if (metered)
  			CNSimulator::record_signal (get_input(), get_output(), report_id);
  
 
 /*
  *		clear the input potential in preparation for next step.
  */
  		inherited::clear_input();
  		
 }
 
 
 
/*
 *		describe_self - Write info about yourself into a file.
 */
void CNDendriteModel::describe_self (FILE *f) 
{
		fprintf (f,"*** Dendrite : %s ***\n", get_name());
		fprintf (f, "\t length = %f\n", length);
		fprintf (f, "\t velocity = %f\n", velocity);
		fprintf (f, "\t decay coefficient = %f\n", decay_coeff);
		fprintf (f, "\t accumulated input decay constant = %f\n\n\n", acc_in_decay);
}  	



/******************************************************************************
 INDendrite

		Initialize a NDendrite object
 ******************************************************************************/

int CNDendriteModel::INDendriteModel( void )
{
	char	name[20];
		
	sprintf (name,"dendrite%d",count);
	inherited::init (name, MAX_INPUTS, 1);
	
	count++;
	
	acc_in_pot = 0.0;
	length = 1.0;
	velocity = 5.0;
	decay_coeff = 0.5;
	acc_in_decay = 0.2;
	
	future_pot = new CNDelayQueue;
	return (future_pot->Init ((int)(length/velocity)));
	
}


/******************************************************************************
 CopyModel

		Returns a clone of yourself
 ******************************************************************************/

CNModel *CNDendriteModel::CopyModel ( void )
{

	CNDendriteModel	*D;
	
	D = new(CNDendriteModel);
	D->INDendriteModel ();
	
	D->RegisterAsCopy (this);
	
	
	D->acc_in_pot = acc_in_pot;
	D->length = length;
	D->velocity = velocity;
	D->decay_coeff = decay_coeff;
	D->acc_in_decay = acc_in_decay;
	
	D->future_pot->Init ((int)(length/velocity));
	
	return D;
}








