/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	methods for Object: InputCell
 *
 */
 
#include <stdio.h>
#include <string.h> 
 
#include "CNInputModel.h"
#include "CNModelElem.h"
#include "CNLists.h"
#include "CNSimulator.h"

// Prototypes for local functions
void DoFire (CObject *theObject);
void DoInputSimulationPrep (CObject *theObject);



/*************************************************************************************
 *
 *                             CNInputModel routines
 *
 *************************************************************************************/
 
 
/*
 *		define the global NodeList all_of_us to 0. Don't worry, it'll be
 *		redefined later.
 */
 
CNInputList *CNInputModel::all_of_us = 0;
 
 
/*
 *		constructor - Gotta add yourself to the global list.
 */
CNInputModel::CNInputModel(void)
 {
 		if (!all_of_us) {
 			all_of_us = new CNInputList;
 			all_of_us->Init ();
 		}
 		
 		all_of_us->add (this);
 }
 
 
/*
 *		destructor - Remove yourself from the global list.
 */
CNInputModel::~CNInputModel(void)
 {
 		all_of_us->del (this);
 }

 

/*
 *		class variable count - used to automatically name new CNInputModels.
 */
 
 int CNInputModel::count = 0;
 
 
/*
 *		Init - initialization - You give it a pointer to the CNModelElem to which 
 *				the input cell is connected.  This pointer is assumed to 
 *				be previously locked.
 *
 *				Returns a flag indicating if the initialization was successful
 */

int CNInputModel::init ()
{
		sprintf (name,"input%d", count);
		cur_t = 0.0;
		output = 0;
		count++;

}


void CNInputModel::connect(CNModelElem *E)
{
	output = E;
}


void CNInputModel::CopyConnect (void)
{
	if (copied_as && output)
		((CNInputModel *)copied_as)->connect ((CNModelElem *)(output->copied_as));
}



/*
 *		fir_val - The amount that gets fired at a given time step.
 *				  Should be defined for each subclass.
 */
float CNInputModel::fire_val (float time)
{ return 0.0;}

/*
 *		fire - Indicates a time click.  Fire away if you must.
 */
 
void CNInputModel::fire (void)
{
		cur_t += CNSimulator::ms_per_tick();
		if (output) 
			output->increment_input(fire_val(cur_t));
		
}

void CNInputModel::reset(void) { }


 	
/***************************************************************************************
 *      
 *                CNInputModel functions that act on the global cell list.
 *
 ***************************************************************************************/

//static CNInputModel *CNInputModel::first_cell (void) 
//  	{ return (all_of_us) ? all_of_us->first() : 0; }
  	
//static CNInputModel *CNInputModel::next_cell (void) 
//  	{ return (all_of_us) ? all_of_us->next() : 0; }


void DoFire (CObject *theObject)
{
	((CNInputModel *)theObject)->fire ();
}


void DoInputSimulationPrep (CObject *theObject)
{
	CNInputModel *E = (CNInputModel *)theObject;
	
	E->reset();
	
}


/*
 *		fire_inputs - Have each input cell of the class fire.
 */
static void CNInputModel::fire_inputs ( void )
{
	if (all_of_us) all_of_us->DoForEach (DoFire);
}


/*
 *		simulation_prep - Have all Elements prepare for simulation AND have them 
 *			register with a reporter.
 */
 static void CNInputModel::simulation_prep (void)
 {
 	if (all_of_us) all_of_us->DoForEach (DoInputSimulationPrep);
 }
   


/*************************************************************************************
 *
 *                             PulseInput routines
 *
 *************************************************************************************/


/*
 *		Init -initialization.  You give it a pointer to the CNModelElem to which it is
 *				connected, a period, and a voltage value.
 */ 
int PulseInput::Init (float per, float v)
{
		inherited::INModel();

		period = per;
		voltage = v;

		
		return (inherited::init());
}

 	
/*
 *		fire_val - Returns the amount to fire at a given time.
 */
  
float PulseInput::fire_val (float time)
{
		int int_per;
		int int_t;

		int_per = (int)(period/CNSimulator::ms_per_tick());
		int_t = (int)(time/CNSimulator::ms_per_tick());
		
		return (((int_t % int_per) == 0) ? voltage : 0.0);
}

/*
 *	GetPulseInputData 
 */
 void PulseInput::GetPulseInputData (nPulseInputData *data)
 {
 	strcpy (data->aName, name);
	data->voltage = voltage;
	data->period = period;
}



/*
 *	SetPulseInputData
 */
 void PulseInput::SetPulseInputData (nPulseInputData *data)
 {
 	strcpy(name, data->aName);
	voltage = data->voltage;
	period = data->period;
}

CNModel *PulseInput::CopyModel (void) { return NULL; }
 

/*************************************************************************************
 *
 *                             CNTrainCellModel routines
 *
 *************************************************************************************/

/*
 *		Init - initialization - You give it the CNModelElem to which it is connected.
 *				Creates a CNTrainCellModel with no pulse instructions.  Use add_pulse
 *				to add pulses to the list.
 */
 
 int CNTrainCellModel::Init ()
{

		inherited::INModel();
		n_signals = 0;
		cur_signal = 0;
		strcpy(fname, "<none>");
		return (inherited::init());
}


/*
 *		InitF - initialization.  You give it the name of a file where all the
 *				pulses are stored.  You also give it a flag indicating 
 *				if you should check to se if the filename has changed.  If
 *				you do the check and the name hasn't changed, a read won't be done.
 *
 *				Returns flag indiating if all went well.
 */
 
 int CNTrainCellModel::InitF (char *filename, int check)
 {
 		FILE *f;
 		int i, n;
 		float v, t;
 		
 		if (check && !strcmp(fname, filename)) return 1;
 		
 		
 		if (!(f = fopen (filename,"r"))) {
 			return 0;
 		}
 		
 		
 		if (fscanf (f,"%d\n", &n) != 1) return 0;
 		for (i=0; i< n; i++) {
 			if ((fscanf(f,"%f\t%f\n",&t, &v)) != 2) {
 				return 0;
 			}
 			
 			add_pulse (t, v);
 		}
 		
 		strcpy (fname, filename);
 		
 		return 1;
 }
 
 
 /*
 *	GetCNTrainCellData 
 */
 void CNTrainCellModel::GetTrainInputData (nTrainInputData *data)
 {
 	strcpy (data->aName, name);
 	strcpy (data->fname, fname);
}



/*
 *	SetPulseInputData
 */
 void CNTrainCellModel::SetTrainInputData (nTrainInputData *data)
 {
 	strcpy(name, data->aName);
 	InitF(data->fname, 1); 
 	
 }	



/*
 *	reset
 */
 void CNTrainCellModel::reset (void)
 {
 	cur_signal=0;
 }
 



/*
 *		add_pulse - Adds a pulse to your pulse list.  Returns a flag indicating if
 *				the addition was correct.
 *
 */
 
int CNTrainCellModel::add_pulse (float when, float how_much)
{
		if (n_signals == MAX_SIGNALS -1) return 0;
		
		signals[n_signals].time = when;
		signals[n_signals].voltage = how_much;
		
		n_signals++;

		return 1;
}


	
/*
 *		fire_val - Returns the amount to fire at a given time.
 */
  
float CNTrainCellModel::fire_val (float time)
{

		float v;
		int int_t;
		int int_signal_t;
		
		v = 0.0;
		
		int_t = (int)(time/CNSimulator::ms_per_tick());
		int_signal_t = (int)(signals[cur_signal].time/CNSimulator::ms_per_tick());
		
		if (int_t == int_signal_t) {
			v = signals[cur_signal]	.voltage;
			if (cur_signal < n_signals -1) cur_signal++;
		}
		
		return v;
}

CNModel *CNTrainCellModel::CopyModel(void)
{
	CNTrainCellModel *T;
	int i;
	
	T = new CNTrainCellModel;
	T->Init();
	
	T->RegisterAsCopy (this);
	
	T->n_signals = n_signals;
	strcpy (T->fname, fname);
	for (i = 0; i< MAX_SIGNALS; i++) {
		T->signals[i].time = signals[i].time;
		T->signals[i].voltage = signals[i].voltage;
	}

	return T;
}


 

