/********************************************************************************
 CNModelElem.c
 
 	Source code for "NervousSystem"
	Copyright  1994 Joe Geigel. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 
 	Biological Neural Net Simulator based on the Bon thesis
 
 
 	methods for Object: CNModelElem
 
 ********************************************************************************/
 
#include "CNModelElem.h"
#include "CNNodeModel.h"

#include "CNAxonModel.h"
#include "CNDendriteModel.h"
#include "CNSomaModel.h"
#include "CNSynapseModel.h"
#include "CNModel.h"
#include "CNInputModel.h"

#include <string.h>

#include "CNLists.h"
#include "CNReporter.h"

// Prototypes for local functions
void DoProcess (CObject *theObject);
void DoSimulationPrep (CObject *theObject, long param);
void DisconnectAndDispose (CObject *theObject, long param);	



/*
 *		define the global NodeList all_of_us to 0. Don't worry, it'll be
 *		redefined later.
 */
 
CNElementList *CNModelElem::all_of_us = 0;



/*
 *		constructor - Each element starts with an input and output potential
 *					  value of 0.0 and are unmetered with no inputs or
 *					  outputs;
 */
 void CNModelElem::CNModelElem (void)
 {
 		cur_in_pot = 0.0;
 		cur_out_pot = 0.0;
 		metered = FALSE;
 		report_id = -1;
 		inputs = new CNNodeList;
 		outputs = new CNNodeList;

 	if (!all_of_us) {
 			all_of_us = new CNElementList;
 			all_of_us->Init ();
 		}
 		
 		all_of_us->add (this);
 }
 
 
 
/*
 *		DisconnectAndDispose - Disconnects yourself from a node and then deletes it.
 */
 void DisconnectAndDispose (CObject *theObject, long param)	
{
	CNNodeModel *N = (CNNodeModel *)theObject;
	CNModelElem *E = (CNModelElem *)param;
	
	N->disconnect(E);
	delete N;
}
 	
 
/*
 *		destructor - Remove yourself from the global list.
 */
CNModelElem::~CNModelElem(void)
 {
 		// remove yourself from the global list
 		all_of_us->del (this);
 		
 		// disconnect and dispose of all input nodes
 		inputs->DoForEach (DisconnectAndDispose);
 		delete inputs;
 		
 		// disconnect and dispose of all output nodes
 		outputs->DoForEach1 (DisconnectAndDispose, (long)this);
 		delete outputs;
 		
 }

 
 /*
  *		init - Initialization usually performed by a subclass of CNModelElem.
  *
  */
  void CNModelElem::init (char *nme, int maxin, int maxout)
  {
  		inherited::INModel();
  
  		strcpy (name, nme);
  		inputs->Init (maxin);
  		outputs->Init (maxout);
  }
 
 
 
 /*
  *		process - Process your element for a simualtion step.  By default
  *				  nothing gets done here.  All processing must be defined 
  *				  by the subclass.
  */
 void CNModelElem::process (void) 
 {}
 
 
 
 

/*
 *		set_id - Sets your reporter id number
 */
 void CNModelElem::set_id (int id)  { report_id = id; }
 


/*
 *		prep_for_simulation - This routine should be called before a simulation
 *				it makes the CNModelElem known to the global CNReporter (report) so
 *				that the results of the simulation may be printed.  Note that 
 *				it only does so if it is metered.  As a result, the CNModelElem is 
 *				assigned a report_id
 */
 int CNModelElem::prep_for_simulation (void)
 {
 	return (metered);
  }
 
 

/*
 *		clear_input - Zero out your input potential
 */
 
 void CNModelElem::clear_input (void)
 {
 		cur_in_pot = 0.0;
 }
 
 
/*
 *		increment_input - Up your input potential by a given amount.
 */
 void CNModelElem::increment_input (float v)
 {
 		cur_in_pot += v;
 }
 
/*
 *		get_input - Returns the current input potential
 */
 float CNModelElem::get_input (void)
 {
 		return cur_in_pot;
 }
 
 /*
 *		set_input - Set your output potential to a given amount
 */
 void CNModelElem::set_output (float v)
 {
 		cur_out_pot = v;
 }
 
/*
 *		get_output - Returns the current output potential
 */
 float CNModelElem::get_output (void)
 {
 		return cur_out_pot;
 }
 
 
 /*		new_input_allowed, new_output_allowed - Returns a flag indicating if a
  *			new input or output can be added.
  */
  int CNModelElem::new_input_allowed (void)
  {
  		return (inputs->room_for_more());
  }


  int CNModelElem::new_output_allowed (void)
  {
  		return (outputs->room_for_more());
  } 
  
  
void CNModelElem::MakeIdentical (CNModel *M)
{
	CNModelElem *E = NULL;
	
	E = (CNModelElem *)M;
	
	metered = E->metered;
	strcpy (name, E->name);
}

  
  

void CNModelElem::CopyConnect (void)
{
  	inputs->ConnectCopy();
 }
 
 
/*
 *		remove_node - Removes a node from either your input or output list.
 */
 void CNModelElem::remove_node (CNNodeModel *N)
 {
 	if (inputs->Includes(N))
 		inputs->Remove(N);
 	else
 		outputs->Remove(N);
 }

 

/*
 *		connect - connect two elements together.  This staic function will be
 *				responsible for creating the node between the two elements.
 *				For now, all weights will be at 1.0.
 *
 *				The arguments to this function are the element who is supplied
 *				as the output and the element that is supplied as the input.  Note
 *				that the order of the argumenets only make a difference when the 
 *				input and output elements are of the same type.
 *
 *				In cases where they are not of the same type, what is an input and
 *				what is an output (or whether the connection is valid at all) is 
 *				determined via the table on Pg 53 of the thesis.
 *
 *				this function returns a flag indicating if the connection was
 *				successfully made.
 */ 
 
 #define M_AXON 1
 #define M_DENDRITE 2
 #define M_SOMA 3
 #define M_SYNAPSE 4
 #define M_STIM 5
 
 static int CNModelElem::connect (CNModel *out, CNModel *in)
 {
 	int out_type, in_type;
 	CNInputModel *I;
 	CNModelElem *In, *Out;
 	
	/*	determine the types of the input and outputs */
 	if (member (out, CNAxonModel)) out_type = M_AXON;
 	else if (member (out, CNDendriteModel)) out_type = M_DENDRITE;	
	else if (member (out, CNSomaModel)) out_type = M_SOMA;	  
	else if (member (out, CNSynapseModel)) out_type = M_SYNAPSE;
	else out_type = M_STIM;
	
 	if (member (in, CNAxonModel)) in_type = M_AXON;
 	else if (member (in, CNDendriteModel)) in_type = M_DENDRITE;	
	else if (member (in, CNSomaModel)) in_type = M_SOMA;	  
	else if (member (in, CNSynapseModel)) in_type = M_SYNAPSE;
	else in_type = M_STIM;
	
	
	/* make the connection dependent on input and output types */
	if (out_type == M_STIM) {
		switch (in_type) {
			case M_STIM : return 0;
			default:
				I = (CNInputModel *)out;
				I->connect ((CNModelElem *)in);
				return 1;
			}
	}
	
	if (in_type == M_STIM) {
		switch (out_type) {
			case M_STIM : return 0;
			default:
				I = (CNInputModel *)in;
				I->connect ((CNModelElem *)out);
				return 1;
			}
	}
	
	In = (CNModelElem *)in;
	Out = (CNModelElem *)out;
		
	if (out_type == M_DENDRITE) {
		switch (in_type) {
			case M_DENDRITE: return In->connect_as_input (Out, 1.0);
			case M_SOMA: return In->connect_as_input(Out, 1.0);
			case M_AXON: return 0;
			case M_SYNAPSE: return Out->connect_as_input(In, 1.0);
		}
	}
	
	if (out_type == M_SOMA) {
		switch (in_type) {
			case M_DENDRITE: return Out->connect_as_input (In, 1.0);
			case M_SOMA: return 0;
			case M_AXON: return In->connect_as_input(Out, 1.0);
			case M_SYNAPSE: return Out->connect_as_input(In, 1.0);
		}
	}
	
	if (out_type == M_AXON) {
		switch (in_type) {
			case M_DENDRITE: return 0;
			case M_SOMA: return Out->connect_as_input(In, 1.0);
			case M_AXON: return Out->connect_as_input(In, 1.0);
			case M_SYNAPSE: return In->connect_as_input(Out, 1.0);
		}
	}

	if (out_type == M_SYNAPSE) {
		switch (in_type) {
			case M_DENDRITE: return In->connect_as_input(Out, 1.0);
			case M_SOMA: return Out->connect_as_input(In, 1.0);
			case M_AXON: return Out->connect_as_input(In, 1.0);
			case M_SYNAPSE: return In->connect_as_input(Out, 1.0);
		}
	}
}


  
/*
 *		connect_as_input - Connect yourself with another CNModelElem.  This new element
 *				will be an input to you and will have the signal strength given
 *				by weight.  Note that weight must be between 0 and 1.  If less
 *				than 0 it is considered 0.  If greater than 1, it is considered
 *				1.
 *
 *				Returns a flag indicating if the addition was done correctly.
 */
 int CNModelElem::connect_as_input (CNModelElem *E, float weight)
 {
 
 		CNNodeModel *N;
 		float w;
 		
 		if (new_input_allowed() && E->new_output_allowed()) {
 			N = new CNNodeModel;
 			w = (weight < 0.0) ? 0.0 : ((weight > 1.0) ? 1.0 : weight);
 			N->Init (E, w, this);

 			add_input(N);
 			E->add_output(N);
 			
 			return 1;
 		}
 		
 		return 0;
 }
 
 
 
 /* 
  *		disconnect_input - Disconnect one of your inputs.  If the given CNModelElem
  *				isn't listed as one of your inputs, this routine does nothing.
  *
  */
  
  void CNModelElem::disconnect_input (CNModelElem *E)
  {
  		CNNodeModel *N;
  		
  		if (N = CNNodeModel::find_node (E, this)) {
  			delete_input(N);
  			E->delete_output(N);
  			
  			delete N;
  		}
  		
  }
  
  
  
  /* 
   *	add/delete_input/output - Adds or deletes a Node from an input or output 
   *		list.
   */
   
   void CNModelElem::add_input(CNNodeModel *N)  { inputs->add (N); }
   void CNModelElem::add_output(CNNodeModel *N)  { outputs->add (N); }
   void CNModelElem::delete_input(CNNodeModel *N)  { inputs->del (N); }
   void CNModelElem::delete_output(CNNodeModel *N)  { outputs->del (N); }
   
   
   
   
/*
 *		attach/detach_meter - Attaches or detaches a meter to the element.
 */
 
 void CNModelElem::attach_meter(void) { metered = TRUE; }
 void CNModelElem::detach_meter(void) { metered = FALSE; }
 void CNModelElem::set_meter (Boolean OnOff) { metered = OnOff; }
 
 
 
/*
 *		describe_self - Write a description of yourself into a file.  This is 
 *			virual and is meant to be defined by each subclass.
 */
 void CNModelElem::describe_self (FILE *f) {}
 
 
 /*
  *		get_name - Returns your name.
  */
const char *CNModelElem::get_name (void) { return name; }
  
  
/*
 *		set_name - Sets your name.
 */
 void CNModelElem::set_name (char *n) { strcpy (name, n); }


/***************************************************************************************
 *      
 *                CNModelElem functions that act on the global Element list.
 *
 ***************************************************************************************/
  
 	
// static CNModelElem *CNModelElem::first_elem (void) 
//  	{ return (all_of_us) ? all_of_us->first() : 0; }
  	
// static CNModelElem *CNModelElem::next_elem (void) 
//  	{ return (all_of_us) ? all_of_us->next() : 0; }
  
  
void DoProcess (CObject *theObject)
{
	((CNModelElem *)theObject)->process();
}
  

void DoSimulationPrep (CObject *theObject, long param)
{
	int id;

	CNModelElem *E = (CNModelElem *)theObject;
	CNReporter *R = (CNReporter *)param;
	
	if (E->prep_for_simulation()) {
		id = R->Register(E);
		E->set_id (id);
	}
}



  
/*
 *		process_elements - Have all CNModelElems process themselves
 */
static void CNModelElem::process_elements ( void )
{
	if (all_of_us) all_of_us->DoForEach (DoProcess);
}


/*
 *		simulation_prep - Have all Elements prepare for simulation AND have them 
 *			register with a reporter.
 */
 static void CNModelElem::simulation_prep (CNReporter *R)
 {
 	if (all_of_us) all_of_us->DoForEach1 (DoSimulationPrep, (long)R);
 }


 