/******************************************************************************
 CNSomaModel.c

							The NSomaModel Class
	
	Soma model
		
	SUPERCLASS = CNModelElem
	
	Source code for "NervousSystem"
	Copyright  1994 Joe Geigel. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
	

 ******************************************************************************/


#include "CNSomaModel.h"

#include <stdio.h>
#include <string.h>
#include <math.h> 
#include "CNSimulator.h"

int CNSomaModel::count = 0;


/*
 *	GetSomaData 
 */
 void CNSomaModel::GetSomaData (nSomaData *data)
{
	strcpy (data->aName, get_name());
	data->acc_in_decay = acc_in_decay;
	data->rest_thresh = orig_rest_thresh;
	data->max_rest_thresh = max_rest_thresh;
	data->min_rest_thresh = min_rest_thresh;
	data->thresh_rise = thresh_rise;
	data->damping_const = damping_const;
	data->t_thresh = t_thresh;
	data->learn_fire = learn_fire;
	data->learn_nofire = learn_nofire;
	data->rest_fire_pot = rest_fire_pot;
	data->fire_recvr_const = fire_recvr_const;
	data->metered = metered;
}
 
 
/*
 *	SetSomaData 
 */
 void CNSomaModel::SetSomaData (nSomaData *data)
{
	set_name(data->aName);
	acc_in_decay = data->acc_in_decay;
	orig_rest_thresh = data->rest_thresh;
	max_rest_thresh = data->max_rest_thresh;
	min_rest_thresh = data->min_rest_thresh;
	thresh_rise = data->thresh_rise;
	damping_const = data->damping_const;
	t_thresh = data->t_thresh;
	learn_fire = data->learn_fire;
	learn_nofire = data->learn_nofire;
	rest_fire_pot = data->rest_fire_pot;
	fire_recvr_const = data->fire_recvr_const;
	metered = data->metered;
}



/*
 *	process - process the soma for a single time step, determining
 *			  the output potention for that time step.
 *
 */
 
 void CNSomaModel::process (void)
 {
 
 		double damping;
 		double sine;
 
  		

 
 /*		calculate the accumulated input potential.  This potential
  *		is the current input plus a fraction of the previous accumulated
  *		input potential.  The fraction is given by the accumulated 
  *		input decay constant.
  */
  		acc_in_pot = inherited::get_input() + ((aid_per_tick * acc_in_pot));
  		
 
 /*
  *		increment our available firing potential.
  */
  		cur_fire_pot += (frc_per_tick * (rest_fire_pot - cur_fire_pot));
  		
 /*
  *		check to see if we're potent enough to fire.
  */
  		if (acc_in_pot > cur_thresh) {
  		
 /*			
  *			we're firing!
  *
  *			transfer all of your available firing potential to your
  *			output.  Note that this uses up all of your accumulated
  *			input potential as well.  The whole soma gets flushed by
  *			firing.
  *
  */
  			inherited::set_output (cur_fire_pot);
  			acc_in_pot = 0.0;
  			cur_fire_pot = 0.0;
  			t_fire = CNSimulator::ms_per_tick();
 
 /*
  *			adjust resting threshold due to learning.  The more you fire,
  *			the easier it is to fire.
  */
  			rest_thresh -= (learn_fire * (rest_thresh - min_rest_thresh));	
 			
  			
  		}
  		else {
 /*
  *			We're not strong enough to fire.
  *
  *			No output for you.
  */
  			inherited::set_output (0.0);
  			t_fire += CNSimulator::ms_per_tick();
  			
 /*
  *			adjust resting threshold due to learning.  The more you don't fire,
  *			the harder it is to fire.
  */
  			rest_thresh += (learn_nofire * (max_rest_thresh - rest_thresh));	
  		
 

 		}

 /*
  *		make a report if you are metered
  */
  		if (metered)
  			CNSimulator::record_signal (get_input(), get_output(), report_id);
 
 
 /*
  *		readjust threshold and current input potential for the next time 
  *		step.
  */
  		damping = exp (- (damping_const * t_fire) );
  		sine = sin (PI * t_fire / t_thresh);
  		cur_thresh = rest_thresh + thresh_rise * damping * sine;
  		inherited::clear_input();
 }
 
 
 
 /*
 *	prep_for_simulation - stuff you gotta do before simulating.
 */
 int CNSomaModel::prep_for_simulation (void)
 {
/*
 *		first do what every good element should do
 */
 		inherited::prep_for_simulation();
 		
 		
 /*
  *		redefine your accumulated input decay constant. and your fire 
  *		recovery constants.  
  */
  		aid_per_tick = (float)(pow ((double)acc_in_decay, 
  			(double)(CNSimulator::ms_per_tick())));
   		frc_per_tick = 1.0  -  
   			((float)(pow((double)(1.0 - fire_recvr_const),
   				(double)(CNSimulator::ms_per_tick()))));
   			
 /*
  *		resets
  */
  		rest_thresh = orig_rest_thresh;
		acc_in_pot = 0.0;
		cur_thresh = rest_thresh;
		t_fire = 0.0;


/*
 *		first do what every good element should do
 */
 		return (inherited::prep_for_simulation());
 		
 }
 
 


/*
 *		describe_self - Write info about yourself into a file.
 */
void CNSomaModel::describe_self (FILE *f) 
{
		fprintf (f,"*** Soma : %s ***\n", get_name());
		fprintf (f,"\t Accumulated input decay constant = %f\n", acc_in_decay);
		fprintf (f,"\t Starting resting threshold = %f\n", orig_rest_thresh);
		fprintf (f,"\t Refractory time = %f\n", t_thresh);
		fprintf (f,"\t Maximum resting threshold = %f\n", max_rest_thresh);   	
		fprintf (f,"\t Minimum resting threshold = %f\n", min_rest_thresh);   	
		fprintf (f,"\t Damping Constant = %f\n", damping_const);
		fprintf (f,"\t Learning coefficient - Firing = %f\n",learn_fire);
		fprintf (f,"\t Learning coefficient - Not Firing = %f\n",learn_nofire);
		fprintf (f,"\t Firing potential at rest = %f\n", rest_fire_pot);
		fprintf (f,"\t Fire recovery constant = %f\n", fire_recvr_const);
		fprintf (f,"\t Amplitude rise after firing = %f\n\n\n", thresh_rise);
}



/******************************************************************************
 INSoma

		Initialize a NSoma object
 ******************************************************************************/

int CNSomaModel::INSomaModel( void )
{
	char	name[20];

	sprintf (name,"soma%d",count);
	inherited::init (name, MAX_INPUTS, MAX_OUTPUT);

	acc_in_decay = 0.2;
	orig_rest_thresh = 10.0;
	t_thresh = 2.0;
	damping_const = 2.0;
	min_rest_thresh = 5.0;
	max_rest_thresh = 15.0;
	learn_fire = 0.0;
	learn_nofire = 0.0;
	rest_fire_pot = 70.0;
	fire_recvr_const = 0.6;
	thresh_rise = 1000.0;
	
	acc_in_pot = 0.0;
	cur_thresh = rest_thresh;
	cur_fire_pot = rest_fire_pot;
	t_fire = 0.0;
	
	count++;
}


/******************************************************************************
 CopyModel

		Returns a clone of yourself
 ******************************************************************************/

CNModel *CNSomaModel::CopyModel ( void )
{

	CNSomaModel	*S;
	
	S = new(CNSomaModel);
	S->INSomaModel ();
	
	S->RegisterAsCopy (this);
	
	S->acc_in_decay = acc_in_decay;
	S->orig_rest_thresh = orig_rest_thresh;
	S->t_thresh = t_thresh;
	S->damping_const = damping_const;
	S->min_rest_thresh = min_rest_thresh;
	S->max_rest_thresh = max_rest_thresh;
	S->learn_fire = learn_fire;
	S->learn_nofire = learn_nofire;
	S->rest_fire_pot = rest_fire_pot ;
	S->fire_recvr_const = fire_recvr_const;
	S->thresh_rise = thresh_rise;
	
	S->acc_in_pot = acc_in_pot;
	S->cur_thresh = cur_thresh;
	S->cur_fire_pot = cur_fire_pot;
	S->t_fire = t_fire;
	
	return S;
}








