/******************************************************************************
 CNSynapseModel.c

							The NSynapseModel Class
	
	Synapse Model	
		
	SUPERCLASS = CNModelElem
	
	Source code for "NervousSystem"
	Copyright  1994 Joe Geigel. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
	

 ******************************************************************************/


#include "CNSynapseModel.h"

#include <stdio.h>
#include <string.h>
#include <math.h> 

#include "CNSimulator.h"


int CNSynapseModel::count = 0;


/*
 *	GetSynapseData
 */
 
 void CNSynapseModel::GetSynapseData (nSynapseData *data)
 {
 	strcpy (data->aName, get_name());
	data->acc_in_decay = acc_in_decay;
	data->rest_thresh = rest_thresh;
	data->vesc_cnt = orig_vesc_cnt;
	data->min_vesc_cnt = min_vesc_cnt;
	data->max_vesc_cnt = max_vesc_cnt;
	data->thresh_rise = thresh_rise;
	data->damping_const = damping_const;
	data->t_fire = t_thresh;
	data->ves_learn_fire = ves_learn_fire;
	data->ves_learn_nofire = ves_learn_nofire;
	data->ffq_learn_fire = ffq_learn_fire;
	data->ffq_learn_nofire = ffq_learn_nofire;
	data->ffv_learn_fire = ffv_learn_fire;
	data->ffv_learn_nofire = ffv_learn_nofire;
	data->fire_pot = cur_fire_pot;
	data->ffq = orig_ffq;
	data->ffv = orig_ffv;
	data->fire_recvr_const = fire_recvr_const;
	data->metered = metered;
}



/*
 *	SetSynapseData
 */
 
 void CNSynapseModel::SetSynapseData (nSynapseData *data)
 {
 	set_name(data->aName);
	acc_in_decay = data->acc_in_decay;
	rest_thresh = data->rest_thresh;
	orig_vesc_cnt = data->vesc_cnt;
	min_vesc_cnt = data->min_vesc_cnt;
	max_vesc_cnt = data->max_vesc_cnt;
	thresh_rise = data->thresh_rise;
	damping_const = data->damping_const;
	t_thresh = data->t_fire;
	ves_learn_fire = data->ves_learn_fire;
	ves_learn_nofire = data->ves_learn_nofire;
	ffq_learn_fire = data->ffq_learn_fire;
	ffq_learn_nofire = data->ffq_learn_nofire;
	ffv_learn_fire = data->ffv_learn_fire;
	ffv_learn_nofire = data->ffv_learn_nofire;
	cur_fire_pot = data->fire_pot;
	orig_ffq = data->ffq;
	orig_ffv = data->ffv;
	fire_recvr_const = data->fire_recvr_const;
	metered = data->metered;
}



 /*
 *	prep_for_simulation - stuff you gotta do before simulating.
 */
 int CNSynapseModel::prep_for_simulation (void)
 {
		
 /*
  *		redefine your accumulated input decay constant. and your fire 
  *		recovery constants.  
  */
  		aid_per_tick = (float)(pow ((double)acc_in_decay, 
  			(double)(CNSimulator::ms_per_tick())));
    	frc_per_tick = 1.0  -  
   			((float)(pow((double)(1.0 - fire_recvr_const),
   				(double)(CNSimulator::ms_per_tick()))));
   			
 /*
  *		set starting values for some members.
  */
  		fire_fract_quant = orig_ffq;
		fire_fract_var = orig_ffv;
		rest_thresh = orig_rest_thresh;

		acc_in_pot = 0.0;
		cur_thresh = rest_thresh;
		fire_fract = 1.0;
		t_fire = 0.0;
		
	
 	
 /*
 *		first do what every good element should do
 */
 		return(inherited::prep_for_simulation());
 		
 }
 
 
 
 
  
/*
 *	process - process the synapse for a single time step, determining
 *			  the output potention for that time step.
 *
 */
 
 void CNSynapseModel::process (void)
 {
 
 		double damping;
 		double sine;
 		double f;
 
 /*		calculate the accumulated input potential.  This potential
  *		is the current input plus a fraction of the previous accumulated
  *		input potential.  The fraction is given by the accumulated 
  *		input decay constant.
  */
  		acc_in_pot = inherited::get_input() + aid_per_tick *  acc_in_pot;
  		

 /*
  *		increment our available firing potential.
  */
  		cur_fire_pot += (frc_per_tick * (cur_vesc_cnt - cur_fire_pot));
  		
  		
 /*
  *		check to see if we're potent enough to fire.
  */
  		if (acc_in_pot > cur_thresh) {
  		
 /*			
  *			we're firing!
  *
  *			figure out what fraction of your firing potential will
  *			be let go during the fire and then fire away!  
  *
  *			Remember to take into account whether this synapse is
  *			inhibitory or excitory.
  *
  *			Note that your accumulated input potential gets flushed
  *			by the firing.
  *
  */
  			f = fire_fract_quant + fire_fract_var * (acc_in_pot - cur_thresh);
  			fire_fract = (f > 1.0) ? 1.0 : f;
  			
  			f = fire_fract * cur_fire_pot;
  			inherited::set_output ((type == INHIBITORY) ? -f : f);
  			
  			acc_in_pot = 0.0;
  			cur_fire_pot -= f;
  			t_fire = CNSimulator::ms_per_tick();
 
 /*
  *			Modify params due to learning
  */
  			cur_vesc_cnt += ves_learn_fire * (max_vesc_cnt - cur_vesc_cnt);
  			fire_fract_quant *= ffq_learn_fire;
  			fire_fract_var *= ffv_learn_fire;
 			
  			
  		}
  		else {
 /*
  *			We're not strong enough to fire.
  *
  *			No output for you.
  */
  			inherited::set_output (0.0);
  			t_fire += CNSimulator::ms_per_tick();
  			
 /*
  *			adjust params due to learning.  
  */
  			cur_vesc_cnt -= ves_learn_nofire * (cur_vesc_cnt - min_vesc_cnt);
  			fire_fract_quant *= ffq_learn_nofire;
  			fire_fract_var *= ffv_learn_nofire;
 			  			
 

  		}
 		
 /*
  *		make a report if you are metered.
  */
  		if (metered)
  			CNSimulator::record_signal (get_input(), get_output(), report_id);	
 
 
 /*
  *		readjust threshold and current input potential for the next time 
  *		step.
  */
  		damping = exp (- (damping_const * t_fire) );
  		sine = sin (PI * t_fire / t_thresh);
  		cur_thresh = rest_thresh + thresh_rise * damping * sine;
  		inherited::clear_input();
 }
 



/*
 *		describe_self - Write info about yourself into a file.
 */
void CNSynapseModel::describe_self (FILE *f) 
{
		fprintf (f,"*** Synapse : %s ***\n", get_name());
		fprintf (f,"\t Accumulated input decay constant = %f\n", acc_in_decay);
		fprintf (f,"\t Starting resting threshold = %f\n", orig_rest_thresh);
		fprintf (f,"\t Refractory time = %f\n", t_thresh);
		fprintf (f,"\t Damping Constant = %f\n", damping_const);
		fprintf (f,"\t Starting Vesicle Count = %f\n", orig_vesc_cnt);
		fprintf (f,"\t Maximum Vesicle Count = %f\n", max_vesc_cnt);   	
		fprintf (f,"\t Minimum Vesicle Count = %f\n", min_vesc_cnt);
		fprintf (f,"\t Fire recovery constant = %f\n", fire_recvr_const);
		fprintf (f,"\t Quantum firing potential term = %f\n", orig_ffq);   	
		fprintf (f,"\t Variable firing potential term = %f\n", orig_ffv); 
		fprintf (f,"\t Vesicle Count learning coefficient - Firing = %f\n", 
			ves_learn_fire);
		fprintf (f,"\t Vesicle Count learning coefficient - Not Firing = %f\n", 
			ves_learn_nofire);
		fprintf (f,"\t Quantum firing learning coefficient - Firing = %f\n", 
			ffq_learn_fire);
		fprintf (f,"\t Quantum firing learning coefficient - Not Firing = %f\n", 
			ffq_learn_nofire);  
		fprintf (f,"\t Variable firing learning coefficient - Firing = %f\n", 
			ffv_learn_fire);
		fprintf (f,"\t Variable firing learning coefficient - Not Firing = %f\n", 
			ffv_learn_nofire);
		
		fprintf (f,"\t Amplitude rise after firing = %f\n\n\n", thresh_rise);
}   	



/******************************************************************************
 INSynapse

		Initialize a NSynapse object
 ******************************************************************************/

int	CNSynapseModel::INSynapseModel( void )
{

	char	name[20];
	
	sprintf (name,"synapse%d",count);
	inherited::init(name, MAX_INPUTS, 1);
	
	type = EXCITORY;
	
	acc_in_decay = 0.2;
	rest_thresh = 20.0;
	orig_rest_thresh = rest_thresh;
	t_thresh = 2.0;
	damping_const = 2.0;
	orig_vesc_cnt = 500.0;
	cur_vesc_cnt = orig_vesc_cnt;
	min_vesc_cnt = 100.0;
	max_vesc_cnt = 1000.0;
	cur_fire_pot = 500.0;
	fire_recvr_const = 0.1;
	orig_ffq = 0.03;
	orig_ffv = 0.0;
	ves_learn_fire = 0.0;
	ves_learn_nofire = 0.0;
	ffq_learn_fire = 1.0;
	ffq_learn_nofire = 1.0;
	ffv_learn_fire = 1.0;
	ffv_learn_nofire = 1.0;
	thresh_rise = 1000.0;
	
	acc_in_pot = 0.0;
	cur_thresh = rest_thresh;
	fire_fract = 1.0;
	t_fire = 0.0;
	
	count++;
	
	return 1;
}



/******************************************************************************
 CopyModel

		Returns a clone of yourself
 ******************************************************************************/

CNModel *CNSynapseModel::CopyModel ( void )
{

	CNSynapseModel	*S;
	
	S = new(CNSynapseModel);
	S->INSynapseModel ();
	
	S->RegisterAsCopy (this);
	S->type = type;
	
	S->acc_in_decay = acc_in_decay;
	S->rest_thresh = rest_thresh;
	S->t_thresh = t_thresh;
	S->damping_const = damping_const;
	S->orig_vesc_cnt = orig_vesc_cnt;
	S->min_vesc_cnt = min_vesc_cnt;
	S->max_vesc_cnt = max_vesc_cnt;
	S->cur_fire_pot = cur_fire_pot;
	S->fire_recvr_const = fire_recvr_const;
	S->orig_ffq = orig_ffq ;
	S->orig_ffv = orig_ffv;
	S->ves_learn_fire = ves_learn_fire;
	S->ves_learn_nofire = ves_learn_nofire;
	S->ffq_learn_fire = ffq_learn_fire;
	S->ffq_learn_nofire = ffq_learn_nofire;
	S->ffv_learn_fire = ffv_learn_fire;
	S->ffv_learn_nofire = ffv_learn_nofire;
	S->thresh_rise = thresh_rise;
	
	S->acc_in_pot = acc_in_pot ;
	S->cur_thresh = cur_thresh;
	S->fire_fract = fire_fract;
	S->t_fire = t_fire;
	
	return S;
}










